#pragma once

#define MIX_MODEL_COUNT 3

struct Gaussian
{
	float mean;
	float std;
	float weight;
};

class Histogram
{
public:

	Histogram( int bucketCount, float minValue, float maxValue );
	~Histogram();

	int GetBucketCount() const;
	int GetValueCount() const;
	float GetMinValue() const;
	float GetMaxValue() const;

	float GetValue( int bucketIndex ) const;
	int GetFrequency( int bucketIndex ) const;
	int GetMaxFrequency() const;

	float GetMedian() const;
	float GetMean() const;
	float GetTukeyMean() const;

	float GetHighestBucketValue();
	float GetValleyAfterPeakValue();

	// implementation easier with constant bounds
	//void SetValueBound( float minValue, float maxValue );

	void AddValue( float value );

	void Clear();

	void FitMixtureOfGaussians();
	void ResetMixtureModel();
	Gaussian GetMixModel( int modelIndex ) const;

	// utility methods
	static double Get1DGaussianSample( double x, double mean, double std );

protected:

	int *m_bucketFrequency;
	int m_bucketCount;
	float m_minValue;
	float m_maxValue;

	int m_maxFrequency;
	int m_valueCount;

	int highestBucket;

	int highestBucketIndex;

	// mixture of gaussians
	Gaussian m_mixModel[MIX_MODEL_COUNT];
	//float *m_bucketMixWeight[MIX_MODEL_COUNT];
};