#pragma once

#include <QVector>
#include <float.h>
#include <algorithm>

#include <QtOpenGL>

#include "Vector.h"
#include "eigensolver.h"

class Mesh {

public:

    Mesh();
    void SetVertsAndFaces(const QVector <Vector3> & v, const QVector <QVector <int> > & f);
    void SetTriangleBounds(QVector <Vector3> b);
    void SetBitmapData(int *data, int w, int h);
    void FlipFaceNorms();

    QVector <Vector3> & getVerts();
    QVector <QVector <int> > & getFaces();
    QVector <Vector3> & getFaceNorms();
    QVector <QSet <int> > & getVertNeigh();
    QVector <Vector3> & getTriangleBounds();
    void GetBitmapData(int *& data, int *&ldata, int &w, int &h);

    void DrawSimple();

    float GetBoundRad();
    Vector3 GetBoundLo();
    Vector3 GetBoundHi();
    Vector3 GetBoundCentre();

    void MakeSpiralMesh(float rad);
    QList <Vector3> GetSpiralMeshSpine();

    void Decimate(QVector <int> & vSegmentation);
    void Noisify();

private:

    void FillInInfo();
    float GetPlaneNeighbourhoodErrorMetric(int v, QVector <QSet <int> > & curNeigh);
    float GetRandomGaussian(float fMean, float fStd);

    QVector <Vector3> m_vVertices; //vertices vector
    QVector <QVector <int> > m_vFaces; //Faces vector

    QVector <Vector3> m_vFaceNorms;
    QVector <QSet <int> > m_vVertNeigh; //neighbour vertex vector

    float boundingSphereRad;
    Vector3 boundingSphereCentre;
    Vector3 meshLoBounds;
    Vector3 meshHiBounds;

    int drawSimpleDispList;

    QVector <Vector3> triangleBounds;
    int *bitmapData;
    int *bitmapLData;
    int bitmapWidth;
    int bitmapHeight;

};

