#include "OFFFile.h"

OFFFile::OFFFile()
{

}

bool OFFFile::Read(char * filename)
{

        FILE * _f = fopen(filename,"r");

        if (_f == NULL)
            return false;

        char strLine[255] = {0};
        char ch = 0;

        while (!feof(_f))
	{
		//float x = 0.0f, y = 0.0f, z = 0.0f;

		// Get the beginning character of the current line in the .obj file
		ch = (char)fgetc(_f);

		switch(ch)
		{
		case 'O':

			qDebug("OFF FILE DETECTED");
			//assume that file is an OFF (Object File Format) file, model is from PSB library
			fgets(strLine,100,_f);
			int nVerts, nFaces;
			fscanf(_f,"%i %i",&nVerts,&nFaces);
			fgets(strLine,100,_f);

			qDebug("Mesh has %i verts, %i faces",nVerts,nFaces);

			for (int i=0;i<nVerts;i++) {
				Vector3 vNewVertex;
                                //fscanf(_f, "%lf %lf %lf", &vNewVertex.x, &vNewVertex.y, &vNewVertex.z);
                                fscanf(_f, "%f %f %f", &vNewVertex.x, &vNewVertex.y, &vNewVertex.z);
				fgets(strLine, 100, _f);
				m_vVertices.push_back(vNewVertex);
			}
			qDebug("Finished reading verts");

			for (int i=0;i<nFaces;i++) {

				QVector <int> newFace;

				int numVForFace;
				int val1, val2, val3;
				fscanf(_f, "%d %d %d %d", &numVForFace, &val1, &val2, &val3);
				fgets(strLine, 100, _f);

				newFace.push_back(val1);
				newFace.push_back(val2);
				newFace.push_back(val3);

				m_vFaces.push_back(newFace);

			}
			qDebug("Finished reading faces");

			break;

		case '\n':

                        // blank line, do nothing
			break;

		default:

                        // read past line
			fgets(strLine, 100, _f);
			break;
		}

	}

        return true;

}

QVector <Vector3> & OFFFile::GetVerts()
{
    return m_vVertices;
}

QVector <QVector <int> > & OFFFile::GetFaces()
{
    return m_vFaces;
}

void OFFFile::Normalize()
{

    Vector3 minv = Vector3(FLT_MAX, FLT_MAX, FLT_MAX);
    Vector3 maxv = Vector3(-FLT_MAX, -FLT_MAX, -FLT_MAX);

    for (int i=0; i<m_vVertices.size(); ++i) {

        Vector3 & v = m_vVertices[i];

        minv.x = qMin(minv.x, v.x);
        minv.y = qMin(minv.y, v.y);
        minv.z = qMin(minv.z, v.z);

        maxv.x = qMax(maxv.x, v.x);
        maxv.y = qMax(maxv.y, v.y);
        maxv.z = qMax(maxv.z, v.z);

    }

    Vector3 vdiff = maxv - minv;
    Vector3 cent = minv + vdiff * 0.5f;
    Vector3 rad = vdiff.Length() * 0.5f;
    float diff = qMax(vdiff.x, qMax(vdiff.y, vdiff.z));

    for (int i=0; i<m_vVertices.size(); ++i) {

        Vector3 & v = m_vVertices[i];

        v.x = (v.x - cent.x) / diff;
        v.y = (v.y - cent.y) / diff;
        v.z = (v.z - cent.z) / diff;

    }

}

bool OFFFile::Write(char * filename)
{

    FILE * fp = fopen(filename,"w");

    if (!fp) {
        qDebug() << "Couldn't open " << filename << " for write.";
        return false;
    }

    //write header
    fprintf(fp, "OFF\n");
    fprintf(fp, "%i %i %i\n", m_vVertices.size(), m_vFaces.size(), m_vVertices.size() * 4);

    //write verts
    for (int i=0; i<m_vVertices.size(); ++i) {

        Vector3 & v = m_vVertices[i];

        fprintf(fp, "%f %f %f\n", v.x, v.y, v.z);

    }

    //write faces
    for (int i=0; i<m_vFaces.size(); ++i) {

        QVector <int> & f = m_vFaces[i];

        fprintf(fp, "%i ", f.size());

        for (int j=0; j<f.size(); ++j) {
            fprintf(fp, "%i ", f[j]);
        }

        fprintf(fp, "\n");

    }

    fclose(fp);

    return true;

}
