// ObjFile.cpp: implementation of the ObjFile class.
//
//////////////////////////////////////////////////////////////////////

#include "ObjFileSimple.h"

ObjFileSimple::ObjFileSimple() {



}

ObjFileSimple::~ObjFileSimple() {



}

bool ObjFileSimple::SimpleRead(const char *sFilePath) {

    FILE *f=fopen(sFilePath,"r");

    if (f==NULL) {
        qDebug("Could not open %s!",sFilePath);
        return false;
    }

    char buffer[500];

    qDebug("Reading...");
    while (!feof(f)) {                

        fgets(buffer, 500, f);      

        QString eachString=QString(buffer);
        QStringList eachStringList=eachString.split(" ");

        if (eachStringList[0].compare("v")==0) {

            Vector3 newvert;
            newvert.x=eachStringList[1].toFloat();
            newvert.y=eachStringList[2].toFloat();
            newvert.z=eachStringList[3].toFloat();

            vertlist.push_back(newvert);

            //qDebug("read vert: %f %f %f",newvert.x,newvert.y,newvert.z);

        }
        else if (eachStringList[0].compare("vn") == 0) {

            Vector3 newnorm;
            newnorm.x=eachStringList[1].toFloat();
            newnorm.y=eachStringList[2].toFloat();
            newnorm.z=eachStringList[3].toFloat();

            normlist.push_back(newnorm);

        }
        else if (eachStringList[0].compare("f") == 0) {

            tFace newface;

            for (int i=1; i<eachStringList.size(); ++i) {

                if (eachStringList[i].contains("/")) {

                    QStringList eachList;
                    eachList=eachStringList[i].split("/");

                    bool vertok = false, normok = false;
                    int vertind, normind;

                    vertind=eachList[0].toInt(&vertok);

                    if (eachList.size() > 1) {
                        normind=eachList.last().toInt(&normok);
                    }

                    if (vertok)
                        newface.vertIndex.push_back(vertind);
                    if (normind)
                        newface.normalIndex.push_back(normind);

                    //qDebug("read part of face: %i//%i",vertind,normind);

                }
                else {

                    bool vertok = false;
                    int vertind = eachStringList[i].toInt(&vertok);

                    if (vertok) {
                        newface.vertIndex.push_back( vertind );
                    }

                }

            }

            facelist.push_back(newface);
        }


    }

    fclose(f);

    qDebug() << "File read successful.";
    qDebug() << "Num verts: " << vertlist.size() << " Num faces: " << facelist.size();

    return true;

}

void ObjFileSimple::GetVerts(QVector <Vector3> & verts)
{
    verts = vertlist;
}

void ObjFileSimple::Normalize()
{

    Vector3 minv = Vector3(FLT_MAX, FLT_MAX, FLT_MAX);
    Vector3 maxv = Vector3(-FLT_MAX, -FLT_MAX, -FLT_MAX);

    for (int i=0; i<vertlist.size(); ++i) {

        Vector3 & v = vertlist[i];

        minv.x = qMin(minv.x, v.x);
        minv.y = qMin(minv.y, v.y);
        minv.z = qMin(minv.z, v.z);

        maxv.x = qMax(maxv.x, v.x);
        maxv.y = qMax(maxv.y, v.y);
        maxv.z = qMax(maxv.z, v.z);

    }

    Vector3 vdiff = maxv - minv;
    Vector3 cent = minv + vdiff * 0.5f;
    Vector3 rad = vdiff.Length() * 0.5f;
    float diff = qMax(vdiff.x, qMax(vdiff.y, vdiff.z));

    for (int i=0; i<vertlist.size(); ++i) {

        Vector3 & v = vertlist[i];

        v.x = (v.x - cent.x) / diff;
        v.y = (v.y - cent.y) / diff;
        v.z = (v.z - cent.z) / diff;

    }
}

bool ObjFileSimple::Write(const char * filename)
{

    FILE * fp = fopen(filename,"w");

    if (!fp) {
        qDebug() << "Couldn't open " << filename << " for write.";
        return false;
    }

    //write header
    fprintf(fp, "# Normalized OBJ file\n");

    //write verts
    for (int i=0; i<vertlist.size(); ++i) {

        Vector3 & v = vertlist[i];

        fprintf(fp, "v %f %f %f\n", v.x, v.y, v.z);

    }

    //write faces
    for (int i=0; i<facelist.size(); ++i) {

        tFace & f = facelist[i];

        fprintf(fp, "f", f.vertIndex.size());

        for (int j=0; j<f.vertIndex.size(); ++j) {
            fprintf(fp, " %i", f.vertIndex[j]);
        }

        fprintf(fp, "\n");

    }

    fclose(fp);

    return true;

}

void ObjFileSimple::GetFaces(QVector <QVector <int> > & faces)
{

    faces.clear();

    //this triangulates each face (if more than 3 edges) and 0-indexes the vertindexes
    for (int i=0; i<facelist.size(); ++i) {

        QVector <int> & vs = facelist[i].vertIndex;

        for (int f=0; f+2<vs.size(); ++f) {

            QVector <int> eachface;

            eachface.push_back(vs[0] - 1);
            eachface.push_back(vs[f+1] - 1);
            eachface.push_back(vs[f+2] - 1);

            faces.push_back(eachface);

        }

    }

}
