/*
	File:	SampleCamera.h
	Author:	Azam Khan
	Date:	16dec00

	Usage:	Implements a 3D camera using 3 points, basically
			the parameters to gluLookAt i.e. (eye, center, up).
*/

#pragma once

#include "Vector.h"

class SampleCamera
{

public:

	enum ViewType {
		VIEW_PERSPECTIVE = 0,
		VIEW_ORTHOGRAPHIC
	};

	SampleCamera();
	SampleCamera(
		double,double,double,
		double,double,double,
		double,double,double );

	const SampleCamera& operator=( const SampleCamera& rhs );

	double GetAspectRatio();
	void SetAspectRatio( double dAspectRatio );
	void Draw();

	void SetCamera( const Vector3 &vEye, const Vector3 &vCent, const Vector3 &vUp );
	void GetCamera( Vector3 &vEye, Vector3 &vCent, Vector3 &vUp ) const;

	void SetUp( Vector3 vNewUp );
	void SetEye( Vector3 vNewEye );
	void SetLoc( Vector3 vLoc );

	void SetFOV( double horizFOV, double vertFOV );

	void GetOrthoSize( double &width, double &height ) const;
	void SetOrthoSize( double width, double height );

	inline double	GetHorizFOV() const			{ return m_dHorizFov;  }
	inline double	GetVertFOV() const			{ return m_dVertFov;  }
	inline Vector3	GetEye() const				{ return m_vEye; }
	inline Vector3	GetCent() const				{ return m_vCenter; }
	inline Vector3	GetUp() const				{ return m_vUp; }

	bool GetOrthographic() const				{ return m_bOrthographic; }
	void SetOrthographic( bool ortho )			{ m_bOrthographic = ortho; }

	bool GetPerspWithOrtho() const				{ return m_bPerspWithOrtho; }
	void SetPerspWithOrtho( bool ortho )		{ m_bPerspWithOrtho = ortho; }

	// PROJECTION_TRANSITION
	// Note: Comment out to test non-implementation of projection transition
	double GetProjectionTransition() const			{ return m_dProjTransition; }
	void SetProjectionTransition( double trans )	{ m_dProjTransition = trans; }

	//JAMES ADDED FOR HOVERCAM
	// These functions mimic the default camera actions in Maya.
	void Tumble( double x, double y );
	void PanCenter			( double x, double y );
	void ZoomTowardCenter	( double t );
	void RollUp( double dTheta );
	void setNearFarPlaneDist(double n, double f);
	void getNearFarPlaneDist(double &n, double &f);
        void SetCent( Vector3 v);

protected:
	Vector3 m_vEye;		// Where we are looking from.
	Vector3 m_vCenter;	// What we are looking at.
	Vector3 m_vUp;		// Which way is up.

	// These are the field of view demi-angles in degrees
	// The horizontal one remains constant at 45 
	// while the vertical one adjusts according to the aspect ratio
	double	m_dHorizFov;
	double	m_dVertFov;	

	// frustum extents for the orthographic view
	double m_dOrthoWidth, m_dBaseOrthoWidth;
	double m_dOrthoHeight, m_dBaseOrthoHeight;

	double	m_dAspectRatio;
	bool	m_bOrthographic;
	bool	m_bPerspWithOrtho;

	// PROJECTION_TRANSITION
	// Note: Comment out to test non-implementation of projection transition
	// Designates degree of interpolation between perspective and orthographic projection:
	// where 0.0 is perspective and 1.0 is orthographic
	double m_dProjTransition;

	double nearPlaneDist;
	double farPlaneDist;

        //light params are now members
        GLfloat light_diff[4];
        GLfloat light_spec[4];
        GLfloat light_position[4];

};

// eof
