#include "ShadowMesh.h"

ShadowMesh::ShadowMesh() {	

}

void ShadowMesh::LoadInputMesh(char *filename) {

	inputMesh=new ObjFile();
	inputMesh->Read(filename);	

}

float ShadowMesh::GetBoundingSphereRad() {
	return boundingSphereRad;
}

void ShadowMesh::PlaceVertsBeneathSphere(float boundingSphereRad) {
	
	unsigned int numVerts=inputMesh->getNumVerts(0);
	Vector3 *inputVerts=inputMesh->getVerts(0);	
	
	for (unsigned int i=0;i<numVerts;i++)
		inputVerts[i]=Vector3(inputVerts[i].x*boundingSphereRad*2.0f,
			inputVerts[i].y*boundingSphereRad-boundingSphereRad/2.0f,
			inputVerts[i].z*boundingSphereRad*2.0f);

}

void ShadowMesh::DrawMesh() {
	
        //unsigned int numVerts=inputMesh->getNumVerts(0);
        //Vector3 *inputNorms=inputMesh->getNorms(0);
	Vector3 *inputVerts=inputMesh->getVerts(0);		

	//iterate through each face for display
	unsigned int numFaces=inputMesh->getNumFaces(0);
	tFace *inputFaces=inputMesh->getFaces(0);

	if (shadeValues.empty())
		ComputeShadows();

	glBegin(GL_TRIANGLES);
	for (unsigned int i=0;i<numFaces;i++) {
		for (unsigned int j=0;j<inputFaces[i].vertIndex.size();j++) {

			unsigned int vertIndex=inputFaces[i].vertIndex[j]-1;

			glColor3f(shadeValues[vertIndex].x,
				shadeValues[vertIndex].y,
				shadeValues[vertIndex].z); 
			glVertex3f(inputVerts[vertIndex].x,
				inputVerts[vertIndex].y,
				inputVerts[vertIndex].z);

		}
	}
	glEnd();

}

void ShadowMesh::DrawMeshUnshaded() {
	
	/*
	unsigned int numVerts=inputMesh->getNumVerts(0);	
	Vector3 *inputNorms=inputMesh->getNorms(0);
	Vector3 *inputVerts=inputMesh->getVerts(0);		

	//iterate through each face for display
	unsigned int numFaces=inputMesh->getNumFaces(0);
	tFace *inputFaces=inputMesh->getFaces(0);

	glBegin(GL_TRIANGLES);
	for (unsigned int i=0;i<numFaces;i++) {
		for (unsigned int j=0;j<inputFaces[i].vertIndex.size();j++) {

			unsigned int vertIndex=inputFaces[i].vertIndex[j]-1;
			unsigned int normIndex=inputFaces[i].normalIndex[j]-1;

			glNormal3f(inputNorms[normIndex].x,
				inputNorms[normIndex].y,
				inputNorms[normIndex].z); 
			glVertex3f(inputVerts[vertIndex].x,
				inputVerts[vertIndex].y,
				inputVerts[vertIndex].z);

		}
	}
	glEnd();
	*/
	inputMesh->DrawSimple();

}

void ShadowMesh::ComputeShadows() {

	unsigned int numVerts=inputMesh->getNumVerts(0);	
	Vector3 *inputVerts=inputMesh->getVerts(0);		
	Vector3 *inputNorms=inputMesh->getNorms(0);

	unsigned int numFaces=inputMesh->getNumFaces(0);
	tFace *inputFaces=inputMesh->getFaces(0);

	//first calculate shading for each vertex
	if (shadeValues.empty()) {		
		shadeValues.reserve(numVerts);
		for (unsigned int i=0;i<numVerts;i++)
			shadeValues.push_back(Vector3(0.3,0.3,0.3));	
	}


	/*
	for (unsigned int i=0;i<numVerts;i++) {		
	*/

	for (unsigned int f=0;f<numFaces;f++) 
		for (unsigned int j=0;j<inputFaces[f].vertIndex.size();j++) {

		/*
		 * Shade each vertex using plane projections from vertex hemisphere
		 */

		unsigned int i=inputFaces[f].vertIndex[j]-1;
		unsigned int n=inputFaces[f].normalIndex[j]-1;

		Vector3 lightDir=pointLightPos-inputVerts[i];
		lightDir.Normalize();		
		float cosFalloff=lightDir.Dot(inputNorms[n]);

		if (cosFalloff<0.0f||LineShadowCasterIntersect(pointLightPos, inputVerts[i]))
			shadeValues[i]=Vector3(0.3,0.3,0.3);
		else
			shadeValues[i]=Vector3(0.3,0.3,0.3)+Vector3(0.5,0.5,0.5)*fabs(cosFalloff);

	}

}

void ShadowMesh::SetShadowCasters(vector <vector <Triangle> > &planeCuts) {

	shadowCasters=planeCuts;

}

void ShadowMesh::SetPointLightPosition(Vector3 p) {
	pointLightPos=p;
}

bool ShadowMesh::LineShadowCasterIntersect(Vector3 pt1, Vector3 pt2) {

	Vector3 intPt;

	//method 1
	/*
	for (unsigned int i=0;i<shadowCasters.size();i++) 
		for (unsigned int j=0;j<shadowCasters[i].size();j++) {
			
			if (LineTriangleIntersect(shadowCasters[i][j],pt1,pt2,intPt)) {
				if ((intPt-pt1).Length()<(pt2-pt1).Length()) //plane must come BETWEEN light and surface
					return true;
			}

		}	
		*/

	//method 2
	for (unsigned int i=0;i<shadowCasters.size();i++) {

		if (shadowCasters[i].empty())
			continue;

		if (!LinePlaneIntersect(shadowCasters[i][0], pt1, pt2, intPt))
			continue;

		for (unsigned int j=0;j<shadowCasters[i].size();j++) {
			
			if (PointTriangleIntersect(shadowCasters[i][j], intPt)) {
				if ((intPt-pt1).Length()<(pt2-pt1).Length()) //plane must come BETWEEN light and surface
					return true;
			}

		}	
	}

	return false;

}

bool ShadowMesh::LinePlaneIntersect(Triangle &tri, Vector3 &pt1, Vector3 &pt2, Vector3 &intersectPoint) {

	Vector3    u, v, n;             // triangle vectors
    Vector3    dir, w0, w;          // ray vectors
    float     r, a, b;             // params to calc ray-plane intersect

    // get triangle edge vectors and plane normal
    u = tri.p2-tri.p1;
    v = tri.p3-tri.p1;
	n=u.Cross(v);             // cross product
    if (n.Length()==0.0)            // triangle is degenerate
        return false;                 // do not deal with this case

    dir = pt2-pt1;             // ray direction vector
    w0 = pt1 - tri.p1;
    a = -n.Dot(w0);
    b = n.Dot(dir);
    if (fabs(b) < 0.00001) {     // ray is parallel to triangle plane
        return false;             // ray disjoint from plane
    }

    // get intersect point of ray with triangle plane
    r = a / b;
    //if (r < 0.0)                   // ray goes away from triangle
    //     return NULL;                  // => no intersect
    // for a segment, also test if (r > 1.0) => no intersect

    intersectPoint = pt1 + dir*r;           // intersect point of ray and plane	
	return true;

}

bool ShadowMesh::PointTriangleIntersect(Triangle &tri, Vector3 &intersectPoint) {

	// is I inside T?
	Vector3 u = tri.p2-tri.p1;
    Vector3 v = tri.p3-tri.p1;

    float    uu, uv, vv, wu, wv, D;
    uu = u.Dot(u);
    uv = u.Dot(v);
    vv = v.Dot(v);
    Vector3 w = intersectPoint - tri.p1;
    wu = w.Dot(u);
    wv = w.Dot(v);
    D = uv * uv - uu * vv;

    // get and test parametric coords
    float s, t;
    s = (uv * wv - vv * wu) / D;
    if (s < 0.0 || s > 1.0)        // I is outside T
        return false;
    t = (uv * wu - uu * wv) / D;
    if (t < 0.0 || (s + t) > 1.0)  // I is outside T
        return false;	

    //return new Vector(intersectPoint.x,intersectPoint.y,intersectPoint.z); // I is in T
	return true;

}

bool ShadowMesh::LineTriangleIntersect(Triangle &tri, Vector3 &pt1, Vector3 &pt2, Vector3 &intersectPoint) {
	
	// intersect_RayTriangle(): intersect a ray with a 3D triangle
//    Input:  a ray R, and a triangle T
//    Output: *I = intersection point (when it exists)
//    Return: -1 = triangle is degenerate (a segment or point)
//             0 = disjoint (no intersect)
//             1 = intersect in unique point I1
//             2 = are in the same plane

    Vector3    u, v, n;             // triangle vectors
    Vector3    dir, w0, w;          // ray vectors
    float     r, a, b;             // params to calc ray-plane intersect

    // get triangle edge vectors and plane normal
    u = tri.p2-tri.p1;
    v = tri.p3-tri.p1;
	n=u.Cross(v);             // cross product
    if (n.Length()==0.0)            // triangle is degenerate
        return false;                 // do not deal with this case

    dir = pt2-pt1;             // ray direction vector
    w0 = pt1 - tri.p1;
    a = -n.Dot(w0);
    b = n.Dot(dir);
    if (fabs(b) < 0.00001) {     // ray is parallel to triangle plane
        return false;             // ray disjoint from plane
    }

    // get intersect point of ray with triangle plane
    r = a / b;
    //if (r < 0.0)                   // ray goes away from triangle
    //     return NULL;                  // => no intersect
    // for a segment, also test if (r > 1.0) => no intersect

    intersectPoint = pt1 + dir*r;           // intersect point of ray and plane	

    // is I inside T?
    float    uu, uv, vv, wu, wv, D;
    uu = u.Dot(u);
    uv = u.Dot(v);
    vv = v.Dot(v);
    w = intersectPoint - tri.p1;
    wu = w.Dot(u);
    wv = w.Dot(v);
    D = uv * uv - uu * vv;

    // get and test parametric coords
    float s, t;
    s = (uv * wv - vv * wu) / D;
    if (s < 0.0 || s > 1.0)        // I is outside T
        return false;
    t = (uv * wu - uu * wv) / D;
    if (t < 0.0 || (s + t) > 1.0)  // I is outside T
        return false;	

    //return new Vector(intersectPoint.x,intersectPoint.y,intersectPoint.z); // I is in T
	return true;

}
