#pragma once

#include <QtOpenGL>
#include <QVector>
#include <algorithm>
#include <cfloat>

#include "Vector.h"

#define TWO_OVER_PI 0.63662f

enum MeshFeatureType {

    P_PCA,
    A_PCA,
    A_RIDGE,
    A_VALLEY,
    P_SYM,
    A_SYM,
    P_GLOBALXYZ,
    P_GLOBALPCA,
    P_PERPSUPPORT,
    P_SYMSUPPORT

};

struct BinElement {

    BinElement() {
        fi=0;
        ftype=0;
        value=0.0f;
    }

    BinElement(int i, int t, float v) {
        fi=i;
        ftype=t;
        value=v;
    }

    bool operator==(const BinElement &rhs) {
        return fi==rhs.fi;
    }

    int fi;
    int ftype;
    float value;

};

struct PlaneSpaceBin {

    QVector <BinElement> f; //bin elements of features and values (initially)
    QVector <BinElement> sf; //bin elements of selected features and values

    //total value of features in bin (currently)
    float totalvalue;    

    //mapping to cartesian spaces
    Vector3 cartesianN;
    float cartesianD;

};

class BinSpace {

public:

    BinSpace();

    void AddAnisotropicPointFeature(Vector3 norm, float d, float rad, int fi, int ftype, float value, Vector3 anisoDir);
    void AddPointFeature(Vector3 norm, float d, float rad, int fi, int ftype, float value);
    void AddCapsuleFeature(Vector3 norm, Vector3 cent, float varyd, float rad, int fi, int ftype, float value);
    void AddAxisFeature(Vector3 eachaxis, Vector3 cent, float rad, int fi, int ftype, float value);
    void AddAnisotropicAxisFeature(Vector3 eachaxis, Vector3 cent, float rad, int fi, int ftype, float value, Vector3 anisoDir);

    void RemoveFeaturesOfMaximalBin(PlaneSpaceBin maxBin);

    void SetSelectThreshold(float f);
    float GetSelectThreshold();

    int GetIncr();

    void CartesianToPolar(Vector3 n, float d, int &theta, int &phi, int &r);
    void CartesianToPolarF(Vector3 n, float d, float & theta, float & phi, float & r);
    void PolarToCartesian(int theta, int phi, int r, Vector3 &n, float &d);
    void PolarToCartesianF(float theta, float phi, float r, Vector3 &n, float &d);

    void Clear();
    bool Empty(int theta, int phi, int r);

    PlaneSpaceBin GetMaxValueBin();
    PlaneSpaceBin GetMaxFeaturesBin();
    PlaneSpaceBin GetMaxValueBin_IncludeSelected();

    PlaneSpaceBin GetNearestPopulatedBin(const int ptheta, const int pphi, const int pr, float & dist);    
    PlaneSpaceBin GetBin(int theta, int phi, int r);
    QVector <const PlaneSpaceBin *> GetAllPopulatedBins();   

    void UpdateDispList();
    void Draw();
    void SetDrawCursor(bool d);
    void SetCursorPos(int t, int p, int r);

    const static int m_nMaxTheta;
    const static int m_nMaxPhi;
    const static int m_nMaxR;
    const static int m_nIncr;

private:

    void AddFeatureToBin(int theta, int phi, int r, int fi, int ftype, float value);
    float GetCartesianDistance(const Vector3 & n1, const float & d1, const Vector3 & n2, const float & d2) const;

    Vector3 SetColorByIndex(int index, float alpha = 1.0f);
    void DrawBin(int theta, int phi, int r);

    QVector <QVector <QVector <PlaneSpaceBin> > > binSpace;
    int m_DispList;    
    float m_fSelectThreshold;

    QVector <Vector3> distinctColours;

    bool m_bCursorDraw;
    int m_cursorTheta;
    int m_cursorPhi;
    int m_cursorR;

};
