#include "eigensolver.h"

void EigenSolver::PCAForPoints(const QVector <Vector3> & points, QList <Vector3> &eigenvecs, QList <float> &eigenvals, Vector3 &centroid, float &maxdist) {

    centroid=Vector3(0,0,0);

    for (int i=0; i<points.size(); ++i)
        centroid+=points[i];

    float recip=1.0f/(float)points.size();
    centroid=centroid*recip;

    //Go through mesh again constructing covariance matrix
    float fSumXX=0;
    float fSumXY=0;
    float fSumXZ=0;

    float fSumYY=0;
    float fSumYZ=0;
    float fSumZZ=0;

    maxdist=0.0f;

    for (int i=0; i<points.size(); ++i) {

        Vector3 kDiff=points[i]-centroid;

        maxdist=qMax(kDiff.Length(), maxdist);

        fSumXX+= kDiff.x * kDiff.x; // sum of the squares of the differences.
        fSumXY+= kDiff.x * kDiff.y; // sum of the squares of the differences.
        fSumXZ+= kDiff.x * kDiff.z; // sum of the squares of the differences.

        fSumYY+= kDiff.y * kDiff.y;
        fSumYZ+= kDiff.y * kDiff.z;
        fSumZZ+= kDiff.z * kDiff.z;

    }

    fSumXX *= recip;
    fSumXY *= recip;
    fSumXZ *= recip;
    fSumYY *= recip;
    fSumYZ *= recip;
    fSumZZ *= recip;

    // setup the eigensolver
    EigenSolver kES;

    kES.mElement[0][0] = fSumXX;
    kES.mElement[0][1] = fSumXY;
    kES.mElement[0][2] = fSumXZ;

    kES.mElement[1][0] = fSumXY;
    kES.mElement[1][1] = fSumYY;
    kES.mElement[1][2] = fSumYZ;

    kES.mElement[2][0] = fSumXZ;
    kES.mElement[2][1] = fSumYZ;
    kES.mElement[2][2] = fSumZZ;

    // compute eigenstuff... we will return all 3 eigenvectors
    kES.DecrSortEigenStuff();

    maxdist=maxdist/sqrt(2);

    for (int i=0; i<3; ++i) {
        eigenvecs.push_back(Vector3(kES.mElement[0][i]*maxdist,
                                    kES.mElement[1][i]*maxdist,
                                    kES.mElement[2][i]*maxdist));
        eigenvals.push_back(kES.m_afDiag[i]);
    }

}
