#include "glwidget.h"

bool GLWidget::EXPERIMENTMODE=false;

GLWidget::GLWidget(QWidget *parent)
    : QGLWidget(parent)
{

    qDebug("Constructing glwidget");
    defaultClearColor=QColor::fromRgb(80,80,160);

    setFocusPolicy(Qt::StrongFocus);   

    planeMesh=new PlaneMesh(&mainCamera);

    m_progressBar = new QProgressBar;
    connect(planeMesh, SIGNAL(percentComplete(int)), m_progressBar, SLOT(setValue(int)));
    connect(planeMesh, SIGNAL(algoFinished()), this, SLOT(UpdateUI()));

    mouse_leftbtn=false;
    mouse_middlebtn=false;
    mouse_rightbtn=false;

    curMeshFilename=0;

    numPlanesSB=new QSpinBox;
    clusterThresholdDSB=new QDoubleSpinBox;

    renderMeshFeaturesCheckBox=new QCheckBox;    
    drawOutputCB=new QCheckBox;
    drawTriangulationCheckBox=new QCheckBox;
    drawChainInColourCheckBox=new QCheckBox;
    useHumanPlaneCuts=new QCheckBox;
    interpSegmentBoundaryCheckBox=new QCheckBox;
    interpTanFactorSpinBox=new QDoubleSpinBox;

    exportContoursButton=new QPushButton;

    thresholdRidgeValleySpinbox =new QDoubleSpinBox;
    smoothSigmaSpinbox=new QDoubleSpinBox;

    for (int i=0;i<NUM_FEATURE_TYPES;i++) {
        m_useFeatCBs[i]=new QCheckBox;
        m_wtDSBs[i]=new QDoubleSpinBox;
        m_radDSBs[i]=new QDoubleSpinBox;
    }    

    neighbourhoodDSB=new QDoubleSpinBox;

    drawBinSpaceCB=new QCheckBox;

    selectBinThresholdSpinbox= new QDoubleSpinBox;

    drawBinSpaceCB = new QCheckBox;
    m_annotateCheckbox = new QCheckBox;    

    m_coverAllSegmentsCheckbox = new QCheckBox("Cover all segments");

    m_puppets = new QCheckBox;

    SetCameraPosition();

}

GLWidget::~GLWidget()
{
    makeCurrent();
}

PlaneMesh * GLWidget::GetPlaneMesh() {

    return planeMesh;

}

void GLWidget::SetSideGLWidget(SideGLWidget * w) {

    m_pSideGLWidget=w;

}

void GLWidget::SetCameraPosition() {

    Vector3 newEye=Vector3(1,1,1);
    newEye.Normalize();
    newEye=newEye*planeMesh->GetBoundRad()*BOUNDRAD_VIEWDISTANCE_FACTOR;

    mainCamera.SetCamera(newEye+planeMesh->GetBoundCentre(),planeMesh->GetBoundCentre(),Vector3(0,1,0));	
    mainCamera.SetAspectRatio((double)width()/(double)height());

}

QSize GLWidget::minimumSizeHint() const
{
    return QSize(400,300);
}

QSize GLWidget::sizeHint() const
{    
    return QSize(6000, 4000);
}

void GLWidget::LoadInputMesh() {

    //put up a proper dialog for the file
    QString filename = QFileDialog::getOpenFileName(this, tr("Load Model"), "../objs", tr("Model Files (*.off *.obj)"));

    if (filename == QString::null)
        return;

    if (!planeMesh->LoadInputMesh(filename.toAscii().data(),
                                  filename.toAscii().data(),
                                  NULL,
                                  useHumanPlaneCuts->isChecked())) {

        QMessageBox msgBox;
        msgBox.setText(QString("Unable to load input ")+objFilenameTextEdit->text()+QString("."));
        msgBox.exec();

    }
    else
        objFilenameTextEdit->setText( filename );

    planesListWidget->clear();
    SetCameraPosition();

    UpdatePlaneList();
    updateGL();

    m_pSideGLWidget->updateGL();

}

void GLWidget::LoadInputMeshFromSequence() {

    if (m_meshFilenames.empty()) {

        QList <QString> meshFilenames;
        QList <QString> modelNames;
        QList <QString> segNames;

        bool doWholeSegBenchmark = false; //XXX

        //more extra models

        if (doWholeSegBenchmark) {

            for (int i=352; i<=400; ++i) {

                const QString num = QString::number(i);

                m_meshFilenames.push_back(QString("../MeshsegBenchmark-1.0/off/")+num+QString(".off"));
                m_modelNames.push_back(num+QString(".off"));

                m_segNames.push_back(QString("../MeshsegBenchmark-1.0/data/seg/Benchmark/")+num+QString("/")+num+QString("_0.seg"));

            }

        }
        else {
            //karan extra models

            /*
            meshFilenames.push_back(QString("../objs/1.off")); modelNames.push_back("1.off");
            meshFilenames.push_back(QString("../objs/35.off")); modelNames.push_back("35.off");
            meshFilenames.push_back(QString("../objs/41.off")); modelNames.push_back("41.off");
            meshFilenames.push_back(QString("../objs/50.off")); modelNames.push_back("50.off");
            meshFilenames.push_back(QString("../objs/104.off")); modelNames.push_back("104.off");
            meshFilenames.push_back(QString("../objs/315.off")); modelNames.push_back("315.off");
            meshFilenames.push_back(QString("../objs/317.off")); modelNames.push_back("317.off");
            */

            //web study set
            /*
            meshFilenames.push_back(QString("../objs/1.off")); modelNames.push_back("Girl"); //human
            meshFilenames.push_back(QString("../objs/2.off")); modelNames.push_back("Man"); //human
            meshFilenames.push_back(QString("../objs/22.off")); modelNames.push_back("Cup"); //human
            meshFilenames.push_back(QString("../objs/35.off")); modelNames.push_back("Goblet"); //human
            meshFilenames.push_back(QString("../objs/50.off")); modelNames.push_back("Glasses"); //human
            meshFilenames.push_back(QString("../objs/66.off")); modelNames.push_back("Plane"); //human
            meshFilenames.push_back(QString("../objs/104.off")); modelNames.push_back("Chair"); //human
            meshFilenames.push_back(QString("../objs/105.off")); modelNames.push_back("Chair2"); //human
            meshFilenames.push_back(QString("../objs/141.off")); modelNames.push_back("Table"); //human
            meshFilenames.push_back(QString("../objs/317.off")); modelNames.push_back("Head"); //human
            meshFilenames.push_back(QString("../objs/385.off")); modelNames.push_back("Donkey"); //human
            */

            ///* training set
            meshFilenames.push_back(QString("../objs/2.off")); modelNames.push_back("Human"); //human

            meshFilenames.push_back(QString("../objs/21.off")); modelNames.push_back("Cup 1"); //cup
            meshFilenames.push_back(QString("../objs/22.off")); modelNames.push_back("Cup 2");
            meshFilenames.push_back(QString("../objs/24.off")); modelNames.push_back("Cup 3");
            meshFilenames.push_back(QString("../objs/25.off")); modelNames.push_back("Cup 4");
            meshFilenames.push_back(QString("../objs/30.off")); modelNames.push_back("Cup 5");

            meshFilenames.push_back(QString("../objs/62.off")); modelNames.push_back("Plane 1");//planes
            meshFilenames.push_back(QString("../objs/63.off")); modelNames.push_back("Plane 2");
            meshFilenames.push_back(QString("../objs/64.off")); modelNames.push_back("Plane 3");
            meshFilenames.push_back(QString("../objs/65.off")); modelNames.push_back("Plane 4");
            meshFilenames.push_back(QString("../objs/66.off")); modelNames.push_back("Plane 5");

            meshFilenames.push_back(QString("../objs/105.off")); modelNames.push_back("Chair");//chair

            meshFilenames.push_back(QString("../objs/141.off")); modelNames.push_back("Table 1"); //tables
            meshFilenames.push_back(QString("../objs/142.off")); modelNames.push_back("Table 2");
            meshFilenames.push_back(QString("../objs/143.off")); modelNames.push_back("Table 3");
            meshFilenames.push_back(QString("../objs/144.off")); modelNames.push_back("Table 4");
            meshFilenames.push_back(QString("../objs/145.off")); modelNames.push_back("Table 5");

            meshFilenames.push_back(QString("../objs/342.off")); modelNames.push_back("Bearing");//bearing

            meshFilenames.push_back(QString("../objs/385.off")); modelNames.push_back("Donkey");//donkey
            //*/

            srand(randomSeedSpinbox->value());

            while (!meshFilenames.empty()) {

                int eachindex=0;

                if (EXPERIMENTMODE)
                    eachindex=rand()%meshFilenames.size();

                m_meshFilenames.push_back(meshFilenames[eachindex]);
                meshFilenames.removeAt(eachindex);

                m_modelNames.push_back(modelNames[eachindex]);
                modelNames.removeAt(eachindex);

            }

        }

    }    


    if (m_segNames.empty())
        planeMesh->LoadInputMesh(m_meshFilenames[curMeshFilename].toAscii().data(),
                                 m_modelNames[curMeshFilename].toAscii().data(),
                                 NULL,
                                 useHumanPlaneCuts->isChecked());
    else
        planeMesh->LoadInputMesh(m_meshFilenames[curMeshFilename].toAscii().data(),
                                 m_modelNames[curMeshFilename].toAscii().data(),
                                 m_segNames[curMeshFilename].toAscii().data(),
                                 useHumanPlaneCuts->isChecked());

    planesListWidget->clear();
    SetCameraPosition();

    SetWeightsForAll();

    objFilenameTextEdit->setText(m_meshFilenames[curMeshFilename]);

    ++curMeshFilename;
    curMeshFilename = curMeshFilename % m_meshFilenames.size();

    //some people get confused if they see no mesh, so we can re-enable showing the mesh if unchecked
    //renderInputMeshCheckBox->setChecked(true);

    updateGL();

    m_pSideGLWidget->updateGL();

}

void GLWidget::initializeGL()
{

    glEnable(GL_NORMALIZE);
    glEnable(GL_DEPTH_TEST);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    glShadeModel(GL_SMOOTH);   

}

void GLWidget::paintGL()
{   

    QColor clearColour = planeMesh->GetBackgroundColour();

    glClearColor(clearColour.redF(),
                 clearColour.greenF(),
                 clearColour.blueF(), 1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  //clears the colour and depth buffers   

    if (m_puppets->isChecked()) {

        planeMesh->DrawPaperPuppetStuff();

    }
    else {

        glPushMatrix();

        mainCamera.Draw();

        if (EXPERIMENTMODE)
            drawGrid();

        if (renderInputVertsCheckBox->isChecked())
            planeMesh->DrawInputVerts();
        if (renderInputWireframeCheckBox->isChecked())
            planeMesh->DrawInputWireframe();
        if (renderInputMeshCheckBox->isChecked())
            planeMesh->DrawInputMesh(1.0);

        if (useHumanPlaneCuts->isChecked())
            planeMesh->DrawHumanCuts();

        if (m_annotateCheckbox->isChecked()) {
            planeMesh->DrawAnnotations();
        }
        else {

            if (EXPERIMENTMODE) {
                if (renderInputMeshCheckBox->isChecked())
                    planeMesh->DrawOutputMesh(false,true);
                else
                    planeMesh->DrawOutputMesh(true,false);

            }
            else {
                if (drawOutputCB->isChecked())
                    planeMesh->DrawOutputMesh(drawTriangulationCheckBox->isChecked(),drawChainInColourCheckBox->isChecked());

            }

        }


        if (renderMeshFeaturesCheckBox->isChecked())
            planeMesh->DrawMeshFeatures();

        glPopMatrix();

    }

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    if (glGetError()) {
        qDebug("GLproblem\n");
    }

}

void GLWidget::resizeGL(int width, int height)
{

    glViewport(0, 0, width, height);
    mainCamera.SetAspectRatio((double)width/(double)height);

    winWidth=width;
    winHeight=height;

}

void GLWidget::mousePressEvent(QMouseEvent *event)
{       

    lastPos = event->pos();

    makeCurrent();
    mainCamera.Draw();

    if (m_annotateCheckbox->isChecked()) {

        if (event->buttons() & Qt::RightButton)
            planeMesh->AddToAnnotatePlane(lastPos.x(), lastPos.y(), true); //start new stroke
        else if (event->buttons() & Qt::LeftButton)
            planeMesh->AddNewAnnotationPatch();

    }
    else {
        planeMesh->mousePressEvent(event,this->width(),this->height());
    }

    updateGL();

    if (drawBinSpaceCB->isChecked())
        m_pSideGLWidget->updateGL();

}

void GLWidget::mouseReleaseEvent(QMouseEvent *event) {

    planeMesh->mouseReleaseEvent(event,this->width(),this->height());
    lastPos=event->pos();

    UpdatePlaneList();
    updateGL();

}

void GLWidget::mouseMoveEvent(QMouseEvent *event)
{

    int x=event->x();
    int y=event->y();

    int cur_mouse_x=lastPos.x();
    int cur_mouse_y=lastPos.y();

    bool doSideUpdate=false;

    makeCurrent();
    mainCamera.Draw();

    if (m_navigationCheckbox->isChecked()) {

        //do cool nav stuff for left drag, right drag does dolly to change hovercam-like distance
        if (event->buttons() & Qt::LeftButton)
            DoNavigation((float)(x-cur_mouse_x)/(float)winWidth, (float)(y-cur_mouse_y)/(float)winHeight);
        else if (event->buttons() & Qt::RightButton)
            mainCamera.ZoomTowardCenter((float)(y-cur_mouse_y)/(float)winHeight * mainCamera.GetEye().Length());
    }
    else if (m_annotateCheckbox->isChecked()) {

        planeMesh->SelectAnnotatePlane(x, y);

        if (event->buttons() & Qt::LeftButton) {
            mainCamera.Tumble((float)(x-cur_mouse_x)/(float)winWidth, (float)(y-cur_mouse_y)/(float)winHeight);           
        }
        else if (event->buttons() & Qt::RightButton) {
            planeMesh->AddToAnnotatePlane(x, y, false);
        }

    }
    else {

        if (!planeMesh->mouseMoveEvent(event,this->width(),this->height())) {

            if (event->buttons() & Qt::LeftButton)
                mainCamera.Tumble((float)(x-cur_mouse_x)/(float)winWidth, (float)(y-cur_mouse_y)/(float)winHeight);
            else if (event->buttons() & Qt::RightButton) {
                mainCamera.ZoomTowardCenter((float)(y-cur_mouse_y)/(float)winHeight * mainCamera.GetEye().Length());
            }

        }
        else
            doSideUpdate = true;

    }

    lastPos=event->pos();

    UpdatePlaneList();
    updateGL();    

    if (doSideUpdate && drawBinSpaceCB->isChecked())
        m_pSideGLWidget->updateGL();

}

void GLWidget::drawGrid() {

    const int gridBound=10;
    const int gridInterval=1;

    glDisable(GL_LIGHTING);

    glPushMatrix();

    glTranslated(0,planeMesh->GetBoundLo().y,0);

    glColor3f(0,0,0);
    glBegin(GL_LINES);

    for (int i=-gridBound;i<=gridBound;i+=gridInterval) {
        if (i==0)
            glColor3f(0,0,0);
        else
            glColor3f(.7,.7,.7);
        glVertex3f(i,0,gridBound);
        glVertex3f(i,0,-gridBound);
    }

    for (int i=-gridBound;i<=gridBound;i+=gridInterval) {
        if (i==0)
            glColor3f(0,0,0);
        else
            glColor3f(.7,.7,.7);
        glVertex3f(gridBound,0,i);
        glVertex3f(-gridBound,0,i);
    }
    glEnd();
    glColor3f(1,1,1);

    glPopMatrix();

}

QWidget *GLWidget::GetRightPaneWidget() {

    QFormLayout *rightPaneLayout = new QFormLayout;
    QFormLayout *rightPaneLayout2 = new QFormLayout;
    //add stuff that was glui here...

    randomSeedSpinbox=new QSpinBox();
    randomSeedSpinbox->setRange(0,19);
    randomSeedSpinbox->setValue(0);

    renderInputVertsCheckBox=new QCheckBox("Vert");
    connect(renderInputVertsCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));
    renderInputWireframeCheckBox=new QCheckBox("Edge");
    connect(renderInputWireframeCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));
    renderInputMeshCheckBox=new QCheckBox("Tri");
    connect(renderInputMeshCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));
    renderInputMeshCheckBox->setChecked(true);

    useSegmentationCheckBox=new QCheckBox("Use Segmentation");
    useSegmentationCheckBox->setChecked(false);
    connect(useSegmentationCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(SetUseSegmentations()));    

    if (EXPERIMENTMODE)
        objFilenameTextEdit=new QLineEdit(QString("Specify seed #, then load"));
    else
        objFilenameTextEdit=new QLineEdit(QString(DEFAULT_MESH));
    objFileLoadButton=new QPushButton(QString("Load..."));
    connect(objFileLoadButton,SIGNAL(clicked()),
            this,SLOT(LoadInputMesh()));

    loadNextFilePushButton=new QPushButton(QString("Load From Sequence"));
    connect(loadNextFilePushButton,SIGNAL(clicked()),
            this,SLOT(LoadInputMeshFromSequence()));

    addPlanePushButton=new QPushButton(QString("Add New"));
    connect(addPlanePushButton,SIGNAL(clicked()),
            this,SLOT(AddNewPlane()));    

    planesListWidget=new QListWidget();
    connect(planesListWidget,SIGNAL(currentRowChanged(int)),
            this,SLOT(SelectPlane()));

    QWidget *editModeGroupBox=new QWidget();
    QHBoxLayout *editModeGroupLayout=new QHBoxLayout;
    //editModeGroupBox->setTitle(QString("Manipulate Mode"));
    QRadioButton *defaultEditRadioButton=new QRadioButton("Default");
    connect(defaultEditRadioButton,SIGNAL(clicked()),
            this,SLOT(SetPlaneManipulation_Default()));
    defaultEditRadioButton->setChecked(true);
    QRadioButton *threeBallRadioButton=new QRadioButton("3-Ball");
    connect(threeBallRadioButton,SIGNAL(clicked()),
            this,SLOT(SetPlaneManipulation_ThreeBall()));
    editModeGroupLayout->addWidget(defaultEditRadioButton);
    editModeGroupLayout->addWidget(threeBallRadioButton);
    editModeGroupBox->setLayout(editModeGroupLayout);

    QPushButton *exportOBJPushButton=new QPushButton("Export OBJ File");
    connect(exportOBJPushButton,SIGNAL(released()),
            this,SLOT(ExportOBJ()));
    outputObjPrefixTextEdit=new QLineEdit(QString("yournamehere"));

    QPushButton *clearCurrentPlanePushButton=new QPushButton("Erase Current");
    connect(clearCurrentPlanePushButton,SIGNAL(clicked()),
            this,SLOT(EraseCurrentPlane()));
    QPushButton *clearAllPlanesPushButton=new QPushButton("Clear All");
    connect(clearAllPlanesPushButton,SIGNAL(clicked()),
            this,SLOT(ClearAllPlanes()));

    numPlanesSB->setRange(1,12);
    numPlanesSB->setValue(DEFAULT_K);

    clusterThresholdDSB->setRange(0.1,2.0);
    clusterThresholdDSB->setSingleStep(0.1);
    clusterThresholdDSB->setValue(1.0);   

    QHBoxLayout *visLayout=new QHBoxLayout;
    visLayout->addWidget(renderInputVertsCheckBox);
    visLayout->addWidget(renderInputWireframeCheckBox);
    visLayout->addWidget(renderInputMeshCheckBox);
    QWidget *visSelectWidget=new QWidget;
    visSelectWidget->setLayout(visLayout);

    renderMeshFeaturesCheckBox->setText("Draw Mesh Features");    
    connect(renderMeshFeaturesCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));   
    renderMeshFeaturesCheckBox->setChecked(false);

    drawOutputCB->setText("Draw Output");    
    connect(drawOutputCB,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));
    drawOutputCB->setChecked(true);

    drawTriangulationCheckBox->setText("Output Triangulate");    
    connect(drawTriangulationCheckBox, SIGNAL(toggled(bool)), planeMesh, SLOT(SetDoTriangulation(bool)));
    connect(drawTriangulationCheckBox, SIGNAL(toggled(bool)), this, SLOT(updateGL()));
    drawTriangulationCheckBox->setChecked(false);

    drawChainInColourCheckBox->setText("Output Coloured");
    drawChainInColourCheckBox->setChecked(true);
    connect(drawChainInColourCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(updateGL()));

    useHumanPlaneCuts->setText("Load/Draw Human Cuts");    
    connect(useHumanPlaneCuts,SIGNAL(toggled(bool)),
            this,SLOT(ToggleUseHumanPlaneCuts()));
    useHumanPlaneCuts->setChecked(false);

    interpSegmentBoundaryCheckBox->setText("Hermite Interp");    
    connect(interpSegmentBoundaryCheckBox,SIGNAL(toggled(bool)),
            this,SLOT(SetHermiteInterp()));
    interpSegmentBoundaryCheckBox->setChecked(true);

    interpTanFactorSpinBox->setRange(0.1,10.0);    
    connect(interpTanFactorSpinBox,SIGNAL(valueChanged(double)),
            this,SLOT(SetHermiteTanFactor()));
    interpTanFactorSpinBox->setValue(1.0);
    interpTanFactorSpinBox->setSingleStep(0.1);

    neighbourhoodDSB->setRange(0.01,0.3);
    neighbourhoodDSB->setSingleStep(0.01);
    neighbourhoodDSB->setValue(0.1);

    double weightmin=0.0;
    double weightmax=10.0;
    double weightstep=0.0001;

    float radmin=0.01f;
    float radmax=0.5f;
    float radstep=0.01f;
    float radval=0.1f;

    for (int i=0;i<NUM_FEATURE_TYPES;i++) {

        m_useFeatCBs[i]->setChecked( true );
        m_useFeatCBs[i]->setText(planeMesh->featureTypeStrings[i]);
        connect(m_useFeatCBs[i],SIGNAL(toggled(bool)),this,SLOT(SetAlgoFeatureParameters()));

        m_wtDSBs[i]->setRange(weightmin,weightmax);
        m_wtDSBs[i]->setSingleStep(weightstep);
        m_wtDSBs[i]->setValue(1.0);
        m_wtDSBs[i]->setDecimals(7);
        connect(m_wtDSBs[i],SIGNAL(valueChanged(double)),this,SLOT(SetAlgoFeatureParameters()));

        m_radDSBs[i]->setRange(radmin,radmax);
        m_radDSBs[i]->setSingleStep(radstep);
        m_radDSBs[i]->setValue(radval);
        m_radDSBs[i]->setDecimals(3);
        connect(m_radDSBs[i],SIGNAL(valueChanged(double)),this,SLOT(SetAlgoFeatureParameters()));

    }   

    selectBinThresholdSpinbox->setRange(0.05,3.00);
    selectBinThresholdSpinbox->setSingleStep(0.05);
    //selectBinThresholdSpinbox->setValue(1.7);
    selectBinThresholdSpinbox->setValue(1);
    connect(selectBinThresholdSpinbox, SIGNAL(valueChanged(double)), this, SLOT(SetAlgoFeatureParameters()));

    thresholdRidgeValleySpinbox->setRange(0.0,100.0);
    thresholdRidgeValleySpinbox->setSingleStep(1.0);
    thresholdRidgeValleySpinbox->setValue(15.0);
    connect(thresholdRidgeValleySpinbox, SIGNAL(valueChanged(double)), this, SLOT(SetAlgoFeatureParameters()));

    smoothSigmaSpinbox->setRange(0.0,1.0);
    smoothSigmaSpinbox->setSingleStep(0.01);
    smoothSigmaSpinbox->setValue(0.01);
    connect(smoothSigmaSpinbox, SIGNAL(valueChanged(double)), this, SLOT(SetAlgoFeatureParameters()));

    //this sets all default values
    SetWeightsForAll();
    SetAlgoFeatureParameters();

    QGridLayout *featureboxlayout=new QGridLayout;

    featureboxlayout->addWidget(new QLabel("Feature"),1,1);
    for (int i=0;i<NUM_FEATURE_TYPES;i++)
        featureboxlayout->addWidget(m_useFeatCBs[i],i+2,1);

    featureboxlayout->addWidget(new QLabel("Weight"),1,2);
    for (int i=0;i<NUM_FEATURE_TYPES;i++)
        featureboxlayout->addWidget(m_wtDSBs[i],i+2,2);

    featureboxlayout->addWidget(new QLabel("Radius"),1,3);
    for (int i=0;i<NUM_FEATURE_TYPES;i++)
        featureboxlayout->addWidget(m_radDSBs[i],i+2,3);

    QWidget *featurewidget=new QWidget;
    featurewidget->setLayout(featureboxlayout); 

    exportContoursButton->setText("Export Contours");
    connect(exportContoursButton,SIGNAL(clicked()),
            this,SLOT(ExportContours()));

    QPushButton * flipNormalsButton = new QPushButton("Flip Normals");
    connect(flipNormalsButton, SIGNAL(clicked()), this, SLOT(FlipMeshNormals()));

    QPushButton * backColourButton = new QPushButton("Set Background Colour");
    connect(backColourButton, SIGNAL(clicked()), this, SLOT(SetBackgroundColour()));

    QPushButton * meshLightButton = new QPushButton("Set MeshLight Colour");
    connect(meshLightButton, SIGNAL(clicked()), this, SLOT(SetMeshLightColour()));

    m_slabThickness = new QDoubleSpinBox;
    m_slabThickness->setRange(0.0, 1.0);
    m_slabThickness->setDecimals(4);
    m_slabThickness->setValue(0.005f);

    if (EXPERIMENTMODE)
        rightPaneLayout->addRow(new QLabel("Seed number:"),randomSeedSpinbox);
    rightPaneLayout->addRow(objFilenameTextEdit);
    rightPaneLayout->addRow(loadNextFilePushButton);
    if (!EXPERIMENTMODE)
        rightPaneLayout->addRow(objFileLoadButton);    

    rightPaneLayout->addRow(visSelectWidget);    

    rightPaneLayout->addRow(addPlanePushButton);        
    rightPaneLayout->addRow(planesListWidget);
    rightPaneLayout->addRow(editModeGroupBox);
    rightPaneLayout->addRow(clearCurrentPlanePushButton);

    rightPaneLayout->addRow(outputObjPrefixTextEdit);
    rightPaneLayout->addRow(exportOBJPushButton);

    if (!EXPERIMENTMODE) {
        rightPaneLayout->addRow(exportContoursButton);               
        rightPaneLayout->addRow(flipNormalsButton);
        rightPaneLayout->addRow(backColourButton);
        rightPaneLayout->addRow(meshLightButton);
        rightPaneLayout->addRow(new QLabel("Slab Thickness"), m_slabThickness);
    }  

    //populate panel with our complete interface
    QPushButton * doBinningButton = new QPushButton;
    doBinningButton->setText("Binning Algorithm");
    connect(doBinningButton,SIGNAL(clicked()),
            this,SLOT(DoBinningAlgorithm()));

    drawBinSpaceCB->setText("Draw Bin Space");
    drawBinSpaceCB->setChecked(false);
    connect(drawBinSpaceCB,SIGNAL(clicked()),
            this,SLOT(ToggleDrawBinSpace()));

    QPushButton * computeMatrixAButton = new QPushButton;
    computeMatrixAButton->setText("Compute Matrix A");
    connect(computeMatrixAButton,SIGNAL(clicked()),
            this,SLOT(BatchComputeMatrixA()));

    QPushButton * analysisUserDataButton = new QPushButton;
    analysisUserDataButton->setText("Analyze User Cuts");
    connect(analysisUserDataButton,SIGNAL(clicked()),
            this,SLOT(DoAnalysisOnUserData()));

    rightPaneLayout2->addRow(m_coverAllSegmentsCheckbox, doBinningButton);
    rightPaneLayout2->addRow(m_progressBar);

    rightPaneLayout2->addRow(featurewidget);
    rightPaneLayout2->addRow(new QLabel("Bin Select Threshold"),selectBinThresholdSpinbox);    
    rightPaneLayout2->addRow(new QLabel("Curvature Threshold"),thresholdRidgeValleySpinbox);
    rightPaneLayout2->addRow(new QLabel("Curvature Smooth"),smoothSigmaSpinbox);
    rightPaneLayout2->addRow(renderMeshFeaturesCheckBox, drawOutputCB);

    rightPaneLayout2->addRow(drawTriangulationCheckBox, drawChainInColourCheckBox);
    rightPaneLayout2->addRow(useHumanPlaneCuts, useSegmentationCheckBox);

    rightPaneLayout2->addRow(interpSegmentBoundaryCheckBox, interpTanFactorSpinBox);

    m_annotateCheckbox = new QCheckBox("Annotate Mode");
    rightPaneLayout2->addRow(drawBinSpaceCB, m_annotateCheckbox);

    m_navigationCheckbox = new QCheckBox("Navigation Mode");
    rightPaneLayout2->addRow(m_navigationCheckbox);
    //rightPaneLayout2->addRow(computeMatrixAButton);
    //rightPaneLayout2->addRow(analysisUserDataButton);
    QPushButton * singleAButton = new QPushButton("Single Model Matrix AC");
    connect(singleAButton, SIGNAL(clicked()), this, SLOT(ComputeMatrixA()));
    //rightPaneLayout2->addRow(singleAButton);
    QPushButton * m_processPSBButton = new QPushButton("BATCH PSB");
    connect(m_processPSBButton, SIGNAL(clicked()), this, SLOT(BatchProcessWholePSB()));
    //rightPaneLayout2->addRow(m_processPSBButton);
    QPushButton * m_featExport = new QPushButton("Export Features OBJ");
    connect(m_featExport, SIGNAL(clicked()), this, SLOT(ExportFeatures()));
    rightPaneLayout2->addRow(m_featExport);

    QPushButton * m_decimateButton = new QPushButton("Mesh Decimate");
    QPushButton * m_noisifyButton = new QPushButton("Mesh Noisify");
    QPushButton * m_segmentifyButton = new QPushButton("Cuts Segmentify");
    QPushButton * m_randomCutsButton = new QPushButton("Add Random Cuts");
    QPushButton * m_addPCAPlanesButton = new QPushButton("Add PCA Planes");
    QPushButton * m_study1Button = new QPushButton("Study 1 Button");

    connect(m_decimateButton, SIGNAL(clicked()), planeMesh, SLOT(MeshDecimate()));
    connect(m_noisifyButton, SIGNAL(clicked()), planeMesh, SLOT(MeshNoisify()));
    connect(m_segmentifyButton, SIGNAL(clicked()), planeMesh, SLOT(CutsSegmentify()));
    connect(m_randomCutsButton, SIGNAL(clicked()), planeMesh, SLOT(AddRandomCuts()));
    connect(m_addPCAPlanesButton, SIGNAL(clicked()), planeMesh, SLOT(AddPCAPlanes()));
    connect(m_study1Button, SIGNAL(clicked()), planeMesh, SLOT(DoUserStudyAnalysis()));

    rightPaneLayout2->addRow(m_decimateButton, m_noisifyButton);
    rightPaneLayout2->addRow(m_segmentifyButton, m_randomCutsButton);
    rightPaneLayout2->addRow(m_addPCAPlanesButton, m_study1Button);

    m_aestheticCut = new QCheckBox("Aesthetic Cut");
    m_aestheticThresh = new QDoubleSpinBox;
    m_aestheticThresh->setRange(0, 1);
    m_aestheticThresh->setValue(0.8);
    m_aestheticThresh->setSingleStep(0.05);

    connect(m_aestheticCut, SIGNAL(toggled(bool)), this, SLOT(SetAestheticCuts()));
    connect(m_aestheticThresh, SIGNAL(valueChanged(double)), this, SLOT(SetAestheticCuts()));

    rightPaneLayout2->addRow(m_aestheticCut, m_aestheticThresh);

    m_puppets = new QCheckBox("Paper Puppets");
    connect(m_puppets, SIGNAL(toggled(bool)), this, SLOT(SetPaperPuppets()));
    QCheckBox * doHolesCheckBox = new QCheckBox("Detect Holes");
    connect(doHolesCheckBox, SIGNAL(toggled(bool)), planeMesh, SLOT(SetCheckForHoles(bool)));
    doHolesCheckBox->setChecked(true);
    rightPaneLayout2->addRow(m_puppets, doHolesCheckBox);

    QWidget *rightPaneWidget1 = new QWidget;
    rightPaneWidget1->setLayout(rightPaneLayout);

    QWidget *rightPaneWidget2 = new QWidget;
    rightPaneWidget2->setLayout(rightPaneLayout2);

    QGridLayout *finalLayout=new QGridLayout;

    finalLayout->addWidget(rightPaneWidget1,1,1);
    if (!EXPERIMENTMODE)
        finalLayout->addWidget(rightPaneWidget2,1,2); //don't show this to experiment users    

    QWidget *finalWidget=new QWidget;
    finalWidget->setLayout(finalLayout);

    return finalWidget;

}

void GLWidget::SetUseSegmentations() {

    planeMesh->SetUseSegmentations(useSegmentationCheckBox->isChecked());
    planeMesh->UpdatePlaneCuts();

    UpdatePlaneList();
    updateGL();

}

void GLWidget::EraseCurrentPlane() {

    planeMesh->ErasePlane(planesListWidget->currentRow());
    UpdatePlaneList();
    planeMesh->SetSelectedPlaneIndex(planesListWidget->currentRow());

    updateGL();


}

void GLWidget::ClearAllPlanes() {

    planeMesh->ClearPlanes();
    UpdatePlaneList();

    updateGL();

}

void GLWidget::ExportOBJ() {

    QStringList filenameSplit=objFilenameTextEdit->text().split(QString("/"));

    QString outputFolder=QString("../")+
                         outputObjPrefixTextEdit->text();

    QString inputOBJFilename=outputFolder+
                          QString("/")+
                          filenameSplit[filenameSplit.size()-1]+
                          QString("_inputmesh.obj");

    QString totalOBJFilename=outputFolder+
                          QString("/")+
                          filenameSplit[filenameSplit.size()-1]+
                          QString(".obj");

    QString totalGeomOBJFilename=outputFolder+
                          QString("/")+
                          filenameSplit[filenameSplit.size()-1]+
                          QString("_cutgeom.obj");

    QString totalSlabOBJFilename=outputFolder+
                          QString("/")+
                          filenameSplit[filenameSplit.size()-1]+
                          QString("_slab.obj");

    qDebug("Making directory %s if not already existing.",outputObjPrefixTextEdit->text().toAscii().data());
#ifdef WIN32
    _mkdir(outputFolder.toAscii().data());
#else
    mkdir(outputFolder.toAscii().data(),S_IRWXU | S_IRWXG | S_IRWXO);
#endif

    planeMesh->PerformPlaneCuts();
    bool result=planeMesh->ExportOutputMesh_OBJ(totalOBJFilename.toAscii().data());

    QMessageBox msgBox;
    if (result)
        msgBox.setText(QString("Exporting ")+totalOBJFilename+QString(" successful."));
    else
        msgBox.setText(QString("Exporting ")+totalOBJFilename+QString(" failed."));
    msgBox.exec();

    result = planeMesh->ExportOutputMeshAsGeom_OBJ(totalGeomOBJFilename.toAscii().data(), 0.005f);
    result = planeMesh->ExportOutputMeshAsSlabs_OBJ(totalSlabOBJFilename.toAscii().data(), m_slabThickness->value());
    result = planeMesh->ExportInputMesh_OBJ(inputOBJFilename.toAscii().data());

}

void GLWidget::AddNewPlane() {

    planeMesh->AddNewPlane();

    UpdatePlaneList();
    planesListWidget->setCurrentRow(planesListWidget->count()-1);

    updateGL();
    m_pSideGLWidget->updateGL();

}

void GLWidget::UpdatePlaneList() {


    if (planeMesh->GetNumPlanes()!=(unsigned int)planesListWidget->count()) {

        int curRow=planesListWidget->currentRow();
        planesListWidget->clear();

        for (unsigned int i=0;i<planeMesh->GetNumPlanes();i++) {
            QListWidgetItem *newItem=new QListWidgetItem;
            newItem->setText(QString("Plane")+QString::number(i)+QString(" (Area: ")+QString::number(planeMesh->GetPlaneCutArea(i))+QString(")"));
            planesListWidget->addItem(newItem);
        }

        planesListWidget->setCurrentRow(curRow);

    }
    else {

        for (int i=0;i<planesListWidget->count();i++) {
            QListWidgetItem *eachItem=planesListWidget->item(i);
            eachItem->setText(QString("Plane")+QString::number(i)+QString(" (Area: ")+QString::number(planeMesh->GetPlaneCutArea(i))+QString(")"));
        }

    }

}

void GLWidget::SetPlaneManipulation_Default() {
    planeMesh->SetPlaneManipMethod(NONE);
    updateGL();
}

void GLWidget::SetPlaneManipulation_ThreeBall() {
    planeMesh->SetPlaneManipMethod(NONEBALL);
    updateGL();
}

void GLWidget::SelectPlane() {

    planeMesh->SetSelectedPlaneIndex(planesListWidget->currentRow());
    updateGL();

    m_pSideGLWidget->updateGL();

}

void GLWidget::keyReleaseEvent(QKeyEvent *event) {

    switch (event->key()) {

    case Qt::Key_X:
        planeMesh->SnapPlaneNormalToXAxis();
        planeMesh->UpdatePlaneCuts();
        break;

    case Qt::Key_Y:
        planeMesh->SnapPlaneNormalToYAxis();
        planeMesh->UpdatePlaneCuts();
        break;

    case Qt::Key_Z:
        planeMesh->SnapPlaneNormalToZAxis();
        planeMesh->UpdatePlaneCuts();
        break;

    case Qt::Key_A:
        mainCamera.RollUp(0.05);
        break;

    case Qt::Key_S:
        mainCamera.RollUp(-0.05);
        break;

    default:
        break;

    }

    updateGL();

}

void GLWidget::SetHermiteInterp() {

    planeMesh->SetCubicHermiteInterpSegmentChains(interpSegmentBoundaryCheckBox->isChecked());
    planeMesh->UpdatePlaneCuts();
    updateGL();

}

void GLWidget::SetHermiteTanFactor() {

    planeMesh->SetCubicHermiteTanScaleFactor(interpTanFactorSpinBox->value());
    planeMesh->UpdatePlaneCuts();
    updateGL();

}

void GLWidget::DoBinningAlgorithm() {

    //set up progress bar here... maybe send this off to another thread?
    m_progressBar->setTextVisible(true);
    m_progressBar->setRange(0, 1000);
    m_progressBar->setValue(0);   

    planeMesh->SetCoverAllSegments(m_coverAllSegmentsCheckbox->isChecked());
    planeMesh->start();

}

void GLWidget::UpdateUI() {

    UpdatePlaneList();
    updateGL();
    m_pSideGLWidget->updateGL();

}

void GLWidget::ExportContours() {

    QStringList filenameSplit=objFilenameTextEdit->text().split(QString("/"));

    QString outputFolder=QString("../")+
                         outputObjPrefixTextEdit->text();

    QString totalContourFilename=outputFolder+
                          QString("/")+
                          filenameSplit.last()+
                          QString(".contour");

    QString totalAIFilename=outputFolder+
                          QString("/")+
                          filenameSplit.last()+
                          QString(".ai");

    QString totalSVGFilename=outputFolder+
                          QString("/")+
                          filenameSplit.last()+
                          QString(".svg");

    qDebug("Making directory %s if not already existing.",outputObjPrefixTextEdit->text().toAscii().data());

#ifdef WIN32
    _mkdir(outputFolder.toAscii().data());
#else
    mkdir(outputFolder.toAscii().data(),S_IRWXU | S_IRWXG | S_IRWXO);
#endif

    bool result1 = planeMesh->ExportContours(totalContourFilename.toAscii().data());
    bool result2 = planeMesh->ExportToIllustratorAI(totalAIFilename.toAscii().data());
    bool result3 = planeMesh->ExportToSVG(totalSVGFilename.toAscii().data());


    QMessageBox msgBox;
    if (result1 && result2 && result3)
        msgBox.setText(QString("Exporting ") + totalContourFilename + QString(" ") + totalAIFilename + QString(" ") + totalSVGFilename + QString(" successful."));
    else if (!result1)
        msgBox.setText(QString("Exporting ") + totalContourFilename + QString(" failed."));
    else if (!result2)
        msgBox.setText(QString("Exporting ") + totalAIFilename + QString(" failed."));
    else if (!result3)
        msgBox.setText(QString("Exporting ") + totalSVGFilename + QString(" failed."));

    msgBox.exec();

}

void GLWidget::ExportFeatures() {

    QStringList filenameSplit=objFilenameTextEdit->text().split(QString("/"));

    QString outputFolder=QString("../")+
                         outputObjPrefixTextEdit->text();

    QString totalFilename=outputFolder+
                          QString("/")+
                          filenameSplit.last()+
                          QString("_feat.obj");

    qDebug("Making directory %s if not already existing.",outputObjPrefixTextEdit->text().toAscii().data());
#ifdef WIN32
    _mkdir(outputFolder.toAscii().data());
#else
    mkdir(outputFolder.toAscii().data(),S_IRWXU | S_IRWXG | S_IRWXO);
#endif

    bool result=planeMesh->ExportFeatures_OBJ(totalFilename.toAscii().data());

    QMessageBox msgBox;
    if (result)
        msgBox.setText(QString("Exporting ")+totalFilename+QString(" successful."));
    else
        msgBox.setText(QString("Exporting ")+totalFilename+QString(" failed."));
    msgBox.exec();

}

void GLWidget::SetAlgoFeatureParameters() {

    bool use[NUM_FEATURE_TYPES];
    float weights[NUM_FEATURE_TYPES];
    float radii[NUM_FEATURE_TYPES];

    for (int i=0;i<NUM_FEATURE_TYPES;i++) {

        use[i]=m_useFeatCBs[i]->isChecked();
        weights[i]=m_wtDSBs[i]->value();
        radii[i]=m_radDSBs[i]->value();

    }

    planeMesh->SetUseMeshFeatures(use);
    planeMesh->SetWeightMeshFeatures(weights);
    planeMesh->SetRadiiMeshFeatures(radii);

    for (int i=0;i<NUM_FEATURE_TYPES;i++) {

        m_wtDSBs[i]->setEnabled(m_useFeatCBs[i]->isChecked());
        m_radDSBs[i]->setEnabled(m_useFeatCBs[i]->isChecked());

    }   

    planeMesh->SetSelectBinThreshold(selectBinThresholdSpinbox->value());

    planeMesh->SetCurvatureSmoothing(smoothSigmaSpinbox->value());

    planeMesh->SetCurvatureThreshold(thresholdRidgeValleySpinbox->value());


    updateGL();

}

void GLWidget::ToggleUseHumanPlaneCuts() {

    updateGL();

}

void GLWidget::ToggleDrawBinSpace() {

    planeMesh->UpdateBinSpaceDispList();
    m_pSideGLWidget->setVisible(drawBinSpaceCB->isChecked());    

}

void GLWidget::DoAnalysisOnUserData() {

    if (planeMesh)
        planeMesh->DoAnalysisOnUserData();

}

void GLWidget::SetWeightsForAll() {

    /*
    const double weightsFor12[10] = {
        0.6794463,
        0.3130431,
        0.3657238,
        0.7233772,
        0.5000544,
        1.0002444,
        0.6080623,
        0.9997082,
        0.5078993,
        1.0002110
    };
    for (int i = 0; i < 10; ++i)
        m_wtDSBs[i]->setValue(weightsFor12[i]);
    */

    const double weightsFor20[10] = {
        0.5157650,
        0.9171339,
        0.5053664,
        0.7891481,
        0.6585792,
        1.0000000,
        0.5256169,
        0.4681394,
        0.5832651,
        1.0000000
    };
    for (int i = 0; i < 10; ++i)
        m_wtDSBs[i]->setValue(weightsFor20[i]);

    /*
    // 68/101 planes
    const double weightsFor20[10] = {
        0.3982913,
        0.7790139,
        0.4699165,
        0.9487925,
        0.5140686,
        0.9999381,
        0.4112076,
        0.4357163,
        0.4936622,
        1.0000883
    };
    for (int i = 0; i < 10; ++i)
        m_wtDSBs[i]->setValue(weightsFor20[i]);
        */


/*
    const double weightsForCups[10] = {
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000,
1.0000000
    };
    for (int i = 0; i < 10; ++i)
        m_wtDSBs[i]->setValue(weightsForCups[i]);
*/

/*
    const double weightsForPlanes[10] = {
0.6257889,
1.0000442,
0.7336633,
1.0000442,
1.0000442,
1.0000442,
0.8647797,
0.9886063,
0.9999639,
1.0000442
    };
    for (int i = 0; i < 10; ++i)
        m_wtDSBs[i]->setValue(weightsForPlanes[i]);
*/

}


void GLWidget::DoNavigation(float mdx, float mdy) {

    float m[16]; glGetFloatv(GL_MODELVIEW_MATRIX, m);
    Vector3 right = Vector3(m[0], m[4], m[8]);
    Vector3 up = Vector3(m[1], m[5], m[9]);

    //first get current distance from closest point (we want to maintain that)
    float lenBefore, lenAfter;
    Vector3 closestPt;

    planeMesh->GetClosestCurvePointForNav(mainCamera.GetEye(), lenBefore, closestPt);

    //now perform a camera translation wrt right and up vectors
    Vector3 newEye = mainCamera.GetEye() + right * (-mdx)  + up * mdy;

    //maintain eye distance with curves
    planeMesh->GetClosestCurvePointForNav(newEye, lenAfter, closestPt);

    Vector3 diff = newEye - closestPt;
    if (diff.Length() > 0.0f) {
        diff.Normalize();
        diff = diff * lenBefore;
    }

    newEye = closestPt + diff;

    mainCamera.SetEye( newEye );

    //move eye lookat point (only in amount relative to input speed, to avoid jarring change in viewing dir)
    float amountToMove = fabs(mdx) + fabs(mdy);
    Vector3 newCent = mainCamera.GetCent();

    if ((closestPt - newCent).Length() < amountToMove) {
        newCent = closestPt;
    }
    else if ((closestPt - newCent).Length() > 0.0f) {
        Vector3 centDiff = closestPt - newCent;
        centDiff.Normalize();
        centDiff = centDiff * amountToMove;

        newCent += centDiff;
    }

    mainCamera.SetCent( newCent );

    //also should do something about up vector...
    float angleToMove = amountToMove * 10.0f;

    Vector3 curUp = mainCamera.GetUp();
    if (acosf(curUp.Dot(Vector3(0,1,0))) < angleToMove) {
        curUp = Vector3(0,1,0);
    }
    else {
        Vector3 axis = curUp.Cross(Vector3(0,1,0));
        axis.Normalize();
        curUp = planeMesh->GetRotatedAxis(angleToMove, curUp, axis);
    }

    mainCamera.SetUp(curUp);


}

void GLWidget::BatchProcessWholePSB()
{

    QVector <int> skipThese;
    skipThese.push_back(303);
    skipThese.push_back(333);
    skipThese.push_back(351);
    skipThese.push_back(354);
    skipThese.push_back(359);
    skipThese.push_back(383);

    for (int i=303; i<=400; ++i) {

        //crashes on: 303, 333, 351, 354, 359, 383
        if (skipThese.contains(i))
            continue;

        QString numstr = QString::number(i);
        QString modelfilename = QString("MeshsegBenchmark-1.0/off/") + numstr + QString(".off");
        QString filename = numstr + QString(".off");
        QString segfilename = QString("MeshsegBenchmark-1.0/data/seg/Benchmark/") + numstr + QString("/") + numstr + QString("_0.seg");

        QFile eachfile(modelfilename);
        if (!eachfile.exists())
            continue;

        planeMesh->LoadInputMesh(modelfilename.toAscii().data(), filename.toAscii().data(), segfilename.toAscii().data(), false);
        planeMesh->DoBinningAlgorithm();

        makeCurrent();
        updateGL();


        QString imgfilename = QString("./psb-output/") + numstr + QString(".png");
        QImage img = grabFrameBuffer();

        qDebug() << img.width() << img.height();

        bool saved = img.save(imgfilename, "png");
        qDebug() << "saved " << imgfilename << "?" << saved;


        QString slabfilename = QString("./psb-output/") + numstr + QString("_slab.obj");
        planeMesh->ExportOutputMeshAsSlabs_OBJ(slabfilename.toAscii().data(), m_slabThickness->value());
        QString contourfilename = QString("./psb-output/") + numstr + QString(".contour");
        planeMesh->ExportContours(contourfilename.toAscii().data());

    }

}

void GLWidget::ComputeMatrixA() {

    //if (!planeMesh)
    //      return;
    planeMesh->ComputeMatrixA(QString("ACmatrices/AC.m"));
    updateGL();

}

void GLWidget::BatchComputeMatrixA() {

    //if (!planeMesh)
    //      return;

    const int num_models = 19;
    const int meshes[num_models] = {2, 21, 22, 24, 25, 30, 62, 63, 64, 65, 66, 105, 141, 142, 143, 144, 145, 342, 385};

    for (int i=0; i<num_models; ++i) {

        QString numstr = QString::number(meshes[i]);
        QString modelfilename = QString("../objs/") + numstr + QString(".off");
        QString filename = numstr + QString(".off");
        QString segfilename = QString("../objs/") + numstr + QString("_0.seg");

        QFile eachfile(modelfilename);
        if (!eachfile.exists())
            continue;

        planeMesh->LoadInputMesh(modelfilename.toAscii().data(), filename.toAscii().data(), segfilename.toAscii().data(), true);
        planeMesh->ComputeMatrixA(QString("ACmatrices/AC") + numstr + QString(".m"));

        updateGL();

    }

}

void GLWidget::SetAestheticCuts()
{

    if (planeMesh) {
        planeMesh->SetAestheticCuts(m_aestheticCut->isChecked(), m_aestheticThresh->value());
        planeMesh->UpdatePlaneCuts();
    }

    updateGL();

}

void GLWidget::SetPaperPuppets()
{

    if (planeMesh) {
        planeMesh->ComputePaperPuppetStuff();
    }
    updateGL();

}

void GLWidget::FlipMeshNormals()
{

    if (planeMesh) {
        planeMesh->FlipMeshNormals();
    }

    updateGL();
}

void GLWidget::SetBackgroundColour()
{

    if (planeMesh) {
        planeMesh->SetBackgroundColour(QColorDialog::getColor());
    }

    updateGL();

}

void GLWidget::SetMeshLightColour()
{

    if (planeMesh) {
        planeMesh->SetMeshLightColour(QColorDialog::getColor());
    }

    updateGL();

}
