#pragma once

#include <QtGui>
#include <QtOpenGL>
#include <QtConcurrentRun>

#include <math.h>

#include <QGLWidget>
#ifdef WIN32
#include <direct.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "SampleCamera.h"
#include "PlaneMesh.h"
#include "sideglwidget.h"

#define DEFAULT_WINDOW_WIDTH 800
#define DEFAULT_WINDOW_HEIGHT 540

#define DEFAULT_MESH "../objs/385.off"
#define DEFAULT_K 6

class SideGLWidget;

class GLWidget : public QGLWidget
{
    Q_OBJECT

public:
    GLWidget(QWidget *parent = 0);
    ~GLWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;

    QWidget *GetRightPaneWidget();
    PlaneMesh *GetPlaneMesh();
    void SetSideGLWidget(SideGLWidget *w);

private slots:   

    void ExportOBJ();
    void ExportContours();
    void ExportFeatures();

    void LoadInputMesh();
    void LoadInputMeshFromSequence();

    void AddNewPlane();
    void SetPlaneManipulation_Default();
    void SetPlaneManipulation_ThreeBall();
    void SelectPlane();
    void EraseCurrentPlane();
    void ClearAllPlanes();

    void UpdatePlaneList();    
    void SetUseSegmentations();

    void SetHermiteInterp();
    void SetHermiteTanFactor();    

    void SetAlgoFeatureParameters();

    void DoBinningAlgorithm();

    void ToggleUseHumanPlaneCuts();

    void ToggleDrawBinSpace();

    void ComputeMatrixA(); //for single model, useful for editing superproxy
    void BatchComputeMatrixA();
    void DoAnalysisOnUserData();        

    void DoNavigation(float mdx, float mdy);

    void BatchProcessWholePSB();

    void SetAestheticCuts();
    void SetPaperPuppets();

    void UpdateUI();

    void FlipMeshNormals();

    void SetBackgroundColour();
    void SetMeshLightColour();

protected:

    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void keyReleaseEvent(QKeyEvent *event);

private:      

    void SetCameraPosition();
    void drawGrid();

    void SetWeightsForAll();

    QColor defaultClearColor;
    QPoint lastPos;        

    SampleCamera mainCamera;

    PlaneMesh *planeMesh;
    SideGLWidget *m_pSideGLWidget;    

    static bool EXPERIMENTMODE;

    int start_mouse_x, start_mouse_y;
    bool mouse_leftbtn, mouse_middlebtn, mouse_rightbtn;
    int winWidth, winHeight;

    //UI elements we keep track of
    QCheckBox *renderInputVertsCheckBox;
    QCheckBox *renderInputWireframeCheckBox;
    QCheckBox *renderInputMeshCheckBox;
    QCheckBox *renderMeshFeaturesCheckBox;
    QCheckBox *useSegmentationCheckBox;
    QCheckBox *interpSegmentBoundaryCheckBox;
    QDoubleSpinBox *interpTanFactorSpinBox;

    QSpinBox *numPlanesSB;
    QDoubleSpinBox *clusterThresholdDSB;

    QLineEdit *objFilenameTextEdit;
    QPushButton *objFileLoadButton;

    QPushButton *addPlanePushButton;
    QPushButton *addPlane3ptPushButton;
    QLineEdit *outputObjPrefixTextEdit;

    QPushButton *findPlanesPushButton;

    QPushButton *loadNextFilePushButton;
    unsigned int curMeshFilename;

    QList <QString> m_meshFilenames;
    QList <QString> m_modelNames;
    QList <QString> m_segNames;

    QListWidget *planesListWidget;

    QSpinBox *randomSeedSpinbox;

    QDoubleSpinBox *thresholdRidgeValleySpinbox;
    QDoubleSpinBox *smoothSigmaSpinbox;   

    QCheckBox *drawOutputCB;
    QCheckBox *drawTriangulationCheckBox;
    QCheckBox *drawChainInColourCheckBox;

    QCheckBox *useHumanPlaneCuts;    

    QPushButton *exportContoursButton;   

    QCheckBox *m_useFeatCBs[NUM_FEATURE_TYPES];
    QDoubleSpinBox *m_wtDSBs[NUM_FEATURE_TYPES];
    QDoubleSpinBox *m_radDSBs[NUM_FEATURE_TYPES];

    QDoubleSpinBox *neighbourhoodDSB;

    QCheckBox *drawBinSpaceCB;   

    QDoubleSpinBox *selectBinThresholdSpinbox;

    QCheckBox * m_navigationCheckbox;
    QCheckBox * m_annotateCheckbox;  

    QProgressBar * m_progressBar;
    QCheckBox *m_coverAllSegmentsCheckbox;

    QDoubleSpinBox * m_slabThickness;

    QCheckBox *m_aestheticCut;
    QDoubleSpinBox * m_aestheticThresh;

    QCheckBox *m_puppets;   

};

