#include "kmeanscluster.h"

kMeansCluster::kMeansCluster() {

}

kMeansCluster::~kMeansCluster() {

    m_vDataPts.clear();
    m_vMeans.clear();

}

void kMeansCluster::AddDataPoint(Vector3 p) {

    m_vDataPts.push_back(p);

}

QVector <Vector3> kMeansCluster::GetMeans(int k) {

    int nDataPts = m_vDataPts.size();

    int nearMeanIndex[nDataPts];

    //initialize
    for (int i=0; i<nDataPts; ++i)
        nearMeanIndex[i]=-1;

    //seed means

    Vector3 dataLoBound = Vector3(FLT_MAX, FLT_MAX, FLT_MAX);
    Vector3 dataHiBound = Vector3(-FLT_MAX, -FLT_MAX, -FLT_MAX);

    m_vMeans.clear();

    for (int i=0; i<nDataPts; ++i) {

        dataLoBound.x = (dataLoBound.x < m_vDataPts[i].x) ? dataLoBound.x : m_vDataPts[i].x ;
        dataLoBound.y = (dataLoBound.y < m_vDataPts[i].y) ? dataLoBound.y : m_vDataPts[i].y ;
        dataLoBound.z = (dataLoBound.z < m_vDataPts[i].z) ? dataLoBound.z : m_vDataPts[i].z ;

        dataHiBound.x = (dataHiBound.x > m_vDataPts[i].x) ? dataHiBound.x : m_vDataPts[i].x ;
        dataHiBound.y = (dataHiBound.y > m_vDataPts[i].y) ? dataHiBound.y : m_vDataPts[i].y ;
        dataHiBound.z = (dataHiBound.z > m_vDataPts[i].z) ? dataHiBound.z : m_vDataPts[i].z ;

    }

    for (int j=0; j<k; ++j) {

        float xRand = ((float)rand()) / RAND_MAX;
        float yRand = ((float)rand()) / RAND_MAX;
        float zRand = ((float)rand()) / RAND_MAX;

        m_vMeans.push_back(Vector3(0,0,0));
        m_vMeans[j].x = dataLoBound.x + xRand * (dataHiBound.x - dataLoBound.x);
        m_vMeans[j].y = dataLoBound.y + yRand * (dataHiBound.y - dataLoBound.y);
        m_vMeans[j].z = dataLoBound.z + zRand * (dataHiBound.z - dataLoBound.z);

    }

    bool bConverged=false;

    while (!bConverged) {       

        //assignment (expectation step)        
        for (int i=0; i<nDataPts; ++i) {

            float fMinDist = FLT_MAX;
            int nMinIndex = -1;

            for (int j=0; j<k; ++j) {

                float fEachDist = (m_vMeans[j]-m_vDataPts[i]).Length();

                if (fEachDist < fMinDist) {

                    fMinDist = fEachDist;
                    nMinIndex = j;

                }

            }

            //if (nMinIndex != nearMeanIndex[i])
            //    bConverged=false;

            nearMeanIndex[i]=nMinIndex;

        }

        //update (maximization step)
        bConverged = true;

        for (int j=0; j<k; ++j) {

            Vector3 newMean = Vector3(0,0,0);
            int nPtsInCluster = 0;

            for (int i=0; i<nDataPts; ++i) {

                if (j == nearMeanIndex[i]) {

                    newMean+=m_vDataPts[i];
                    ++nPtsInCluster;

                }

            }

            if ( nPtsInCluster > 0 ) {

                newMean = newMean * (1.0f / (float)nPtsInCluster);

                if ((newMean - m_vMeans[j]).Length() > 1.0f )
                    bConverged=false;

                m_vMeans[j] = newMean;

            }

        }

    }

    return m_vMeans;

}
