//Fragment shader
const int maxNumPlanes = 10;
const int globalMaxCurveProfileSamples = maxNumPlanes * 8;
uniform vec3 pointLightPos;
uniform float pointLightRad;
uniform int myPlaneNum;
uniform vec4 myPlaneParams[maxNumPlanes];
uniform vec3 myPlaneCentroids[maxNumPlanes];
uniform float myPlaneBoundRads[maxNumPlanes];
uniform int myPlaneIsHole[maxNumPlanes];
uniform int myPlaneCommonIndex[maxNumPlanes];

uniform float myPlaneCurveProfileThetas[globalMaxCurveProfileSamples];
uniform float myPlaneCurveProfileRads[globalMaxCurveProfileSamples];
uniform int myPlaneCurveProfileStartIndex[maxNumPlanes];
uniform int myPlaneCurveProfileEndIndex[maxNumPlanes];
uniform vec3 myPlaneParamsRotVec1[maxNumPlanes];
uniform vec3 myPlaneParamsRotVec2[maxNumPlanes];

uniform int useCosInterp;

varying vec3 myV;
varying vec3 myN;

const float pi = 3.14159;

float GetCurveProfileBoundRad(in int pInd, in vec3 intPt) {

	float curveProfileBoundRad = 1.0;
	vec3 intPtNorm=normalize(intPt-myPlaneCentroids[pInd]);
	
	float dotProdOfVecs=dot(intPtNorm,myPlaneParamsRotVec1[pInd]);
	float curveProfileTheta=acos(dotProdOfVecs);
	
	if (dotProdOfVecs>=0.9999)
		curveProfileTheta=0.0;
	else if (dotProdOfVecs<=-0.9999)
		curveProfileTheta=pi;
		
	if (dot(intPtNorm,myPlaneParamsRotVec2[pInd])<0.0) {
		curveProfileTheta=2.0*pi-curveProfileTheta;
	}	

	if (curveProfileTheta<myPlaneCurveProfileThetas[myPlaneCurveProfileStartIndex[pInd]]) {
		float interpVal=(curveProfileTheta-(myPlaneCurveProfileThetas[myPlaneCurveProfileEndIndex[pInd]-1]-2.0*pi))/(myPlaneCurveProfileThetas[myPlaneCurveProfileStartIndex[pInd]]-(myPlaneCurveProfileThetas[myPlaneCurveProfileEndIndex[pInd]-1]-2.0*pi));
		curveProfileBoundRad=myPlaneCurveProfileRads[myPlaneCurveProfileEndIndex[pInd]-1]*(1.0-interpVal)+myPlaneCurveProfileRads[myPlaneCurveProfileStartIndex[pInd]]*interpVal;
	}
	else if (curveProfileTheta>myPlaneCurveProfileThetas[myPlaneCurveProfileEndIndex[pInd]-1]) {					
		float interpVal=(curveProfileTheta-myPlaneCurveProfileThetas[myPlaneCurveProfileEndIndex[pInd]-1])/((myPlaneCurveProfileThetas[myPlaneCurveProfileStartIndex[pInd]]+2.0*pi)-myPlaneCurveProfileThetas[myPlaneCurveProfileEndIndex[pInd]-1]);
		curveProfileBoundRad=myPlaneCurveProfileRads[myPlaneCurveProfileEndIndex[pInd]-1]*(1.0-interpVal)+myPlaneCurveProfileRads[myPlaneCurveProfileStartIndex[pInd]]*interpVal;														
	}
	else {
		
		for (int j=myPlaneCurveProfileStartIndex[pInd];j<myPlaneCurveProfileEndIndex[pInd]-1;j++) {
							
			if (myPlaneCurveProfileThetas[j]<=curveProfileTheta&&myPlaneCurveProfileThetas[j+1]>=curveProfileTheta) {
				
				if (useCosInterp==0) {
					float interpVal=(curveProfileTheta-myPlaneCurveProfileThetas[j])/(myPlaneCurveProfileThetas[j+1]-myPlaneCurveProfileThetas[j]);
					curveProfileBoundRad=myPlaneCurveProfileRads[j]*(1.0-interpVal)+myPlaneCurveProfileRads[j+1]*interpVal;
					//float interpVal=smoothstep(myPlaneCurveProfileRads[j],myPlaneCurveProfileRads[j+1],curveProfileTheta);
					//curveProfileBoundRad=myPlaneCurveProfileRads[j]*(1.0-interpVal)+myPlaneCurveProfileRads[j+1]*interpVal;
				}
				else {	
					float interpVal=pi*(curveProfileTheta-myPlaneCurveProfileThetas[j])/(myPlaneCurveProfileThetas[j+1]-myPlaneCurveProfileThetas[j]);
					interpVal=(cos(interpVal)+1.0)/2.0;
					curveProfileBoundRad=myPlaneCurveProfileRads[j]*interpVal+myPlaneCurveProfileRads[j+1]*(1.0-interpVal);
				}
				break;						
				
			}					
		}
	}
	
	return curveProfileBoundRad;

}

void main (void)
{	
	
	gl_FragColor=vec4(0.0,0.0,0.0,1.0);
	
	myN=normalize(myN);
	float origIntensity=clamp(dot(normalize(pointLightPos-myV),myN),0.0,1.0);
	float curIntensity=origIntensity;
		
	if (dot(myN,pointLightPos-myV)>0.0) {		
			
		for (int i=0;i<myPlaneNum;i++) {
		
			if (myPlaneIsHole[i]==1)
				continue;
		
			//1.  find the point/plane intersection		
			
			vec3 n = normalize(myPlaneParams[i].xyz);
			vec3 w0 = myV - myPlaneCentroids[i];
			vec3 dir = pointLightPos-myV;
			
			float a = dot(-n,w0);
			float b = dot(n,dir);
					
			float r=a/b;
			
			vec3 intPt = myV + dir*r;				
			
			if (dot(pointLightPos-intPt,pointLightPos-myV) > 0.0&& //(radiate OUTWARD from light source)				
				distance(pointLightPos,intPt) < distance(pointLightPos,myV)) {//&& //(following ray from light, hits plane BEFORE surface)					
				//dot(myN,myPlaneParams[i].xyz) > 0.0) { //(do not shadow interior of meshes)
			
				//2.  if its in (VARYING) bounding rad of mesh, and not within any holes, shade accordingly (soft or hard)							
			
				float intPtDistFromCentroid = distance(intPt, myPlaneCentroids[i]); 
				float curveProfileBoundRad=GetCurveProfileBoundRad(i,intPt);
				
				//float normPenumbraDist = (intPtDistFromCentroid - (myPlaneBoundRads[i]-pointLightRad))/pointLightRad;			
				float normPenumbraDist = (intPtDistFromCentroid - (curveProfileBoundRad-pointLightRad))/pointLightRad;			
						
				//if (intPtDistFromCentroid < myPlaneBoundRads[i]-pointLightRad) {
				if (intPtDistFromCentroid < curveProfileBoundRad-pointLightRad) {
					curIntensity = 0.0;
				}
				//else if (intPtDistFromCentroid < myPlaneBoundRads[i]+pointLightRad) {
				else if (intPtDistFromCentroid < curveProfileBoundRad+pointLightRad) {
				
					//penumbra is calculated by portion of spherical area light unobstructed					
					//curIntensity = min(curIntensity,origIntensity*arcSinPenumb/pi);				
					//curIntensity = curIntensity*arcSinPenumb/pi;
					
					//should be using arcsin to approximate integration of occluded surface area of sphere
					curIntensity = min(curIntensity, origIntensity*normPenumbraDist/2.0);
				}				
				
				//3. determine if intPt also crosses any holes within COMMON PLANE				
				for (int j=0;j<myPlaneNum;j++) {				
				
					if (myPlaneCommonIndex[i]!=myPlaneCommonIndex[j]||
						myPlaneIsHole[j]==0)
						continue;
						
					float intPtDistFromCentroid = distance(intPt, myPlaneCentroids[j]); 
					float curveProfileBoundRad=GetCurveProfileBoundRad(j,intPt);
					float normPenumbraDist = (intPtDistFromCentroid - (curveProfileBoundRad-pointLightRad))/pointLightRad;			
						
					if (intPtDistFromCentroid<curveProfileBoundRad-pointLightRad) {					
						curIntensity=origIntensity;				
					}
					else if (intPtDistFromCentroid<curveProfileBoundRad+pointLightRad) {
						curIntensity=max(curIntensity,origIntensity*(1.0-normPenumbraDist/2.0));
					}
						
				}					
							
			}
		
		}
						
	}	
	
	//gl_FragColor=vec4(curIntensity,curIntensity,curIntensity,1.0);
	gl_FragColor=vec4(origIntensity*curIntensity,origIntensity*curIntensity,1.0*curIntensity,1.0);
	
}