#include "sideglwidget.h"

SideGLWidget::SideGLWidget(PlaneMesh *p)
{

    m_pPlaneMesh=p;

    qDebug("instantiating sidewidget");

    setFocusPolicy(Qt::StrongFocus); 

    SetCameraPosition();

}

QSize SideGLWidget::minimumSizeHint() const
{
    return QSize(400,300);
}

QSize SideGLWidget::sizeHint() const
{    
    return QSize(6000, 4000);
}

void SideGLWidget::SetCameraPosition() {

    Vector3 lookAt=Vector3(0,0,0);
    Vector3 startAt=lookAt+Vector3(1.5f,1.5f,1.5f);

    m_camera.SetCamera(startAt,lookAt,Vector3(0,1,0));
    m_camera.SetAspectRatio((double)width()/(double)height());

}

void SideGLWidget::SetGLWidget(GLWidget *w) {

    m_pGLWidget=w;

}

void SideGLWidget::initializeGL() {

    glEnable(GL_NORMALIZE);
    glEnable(GL_DEPTH_TEST);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    glShadeModel(GL_SMOOTH);

    glEnable(GL_LIGHT0);

}

void SideGLWidget::paintGL() {

    glClearColor(1,1,1,1);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  //clears the colour and depth buffers

    glPushMatrix();

    m_camera.Draw();

    m_pPlaneMesh->DrawBinSpace();

    glPopMatrix();

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    if (glGetError()) {
        qDebug("GLproblem within sideglwidget\n");
    }

}

void SideGLWidget::resizeGL(int width, int height) {

    glViewport(0, 0, width, height);
    m_camera.SetAspectRatio((double)width/(double)height);

    m_nWidth=width;
    m_nHeight=height;

}

void SideGLWidget::mousePressEvent(QMouseEvent *event) {

    //makeCurrent();
    //mainCamera.Draw(); //comment out stuff related to picking/selection (will change)
    //planeMesh->mousePressEvent(event,this->width(),this->height());

    m_bMovedMouseAfterClick = false;
    lastPos = event->pos();
    updateGL();

}

void SideGLWidget::mouseReleaseEvent(QMouseEvent *event) {

    //m_pPlaneMesh->mouseReleaseEvent(event,this->width(),this->height());

    lastPos=event->pos();
    updateGL();

    /*
    if (!m_bMovedMouseAfterClick) {

        makeCurrent();
        m_camera.Draw();

        bool bHitGeometry;
        Vector3 v;
        bHitGeometry=m_pPlaneMesh->UnprojectPos(lastPos,v);

        if (!bHitGeometry)
            return;

        qDebug("Before any conversion: %f %f %f",v.x,v.y,v.z);

        //convert unprojected point from theta,phi,r to model space
        //glScalef(5.0f/360.0f,5.0f/360.0f,5.0f/360.0f);
        //glTranslatef(-180.0f,-90.0f,-50.0f);
        int theta=(int)(v.x*72.0f)+180;
        int phi=(int)(v.y*72.0f)+90;
        int r=(int)(v.z*72.0f)+50;

        qDebug("Sampled point: %i %i %i",theta,phi,r);

        Plane p;
        m_pPlaneMesh->PolarToCartesian(theta,phi,r,p.norm,p.pt);
        p.cent=p.norm*p.pt;
        m_pPlaneMesh->SetPlaneRotAxesFromNorm(p);

        //modify params of currently selected plane
        if (m_pPlaneMesh->GetSelectedPlaneIndex()<m_pPlaneMesh->GetNumPlanes())
            m_pPlaneMesh->SetPlaneParams(m_pPlaneMesh->GetSelectedPlaneIndex(),p);

        m_pPlaneMesh->CartesianToPolar(p.norm,p.pt,theta,phi,r);
        qDebug("Converted back point: %i %i %i",theta,phi,r);

        m_pGLWidget->updateGL();

    }
    */

}

void SideGLWidget::mouseMoveEvent(QMouseEvent *event) {

    int x=event->x();
    int y=event->y();

    if (event->buttons() & Qt::LeftButton)
        m_camera.Tumble((float)(x-lastPos.x())/(float)m_nWidth, (float)(y-lastPos.y())/(float)m_nHeight);
    else if (event->buttons() & Qt::RightButton)
        m_camera.ZoomTowardCenter((float)(y-lastPos.y())/(float)m_nHeight);

    lastPos=event->pos();
    m_bMovedMouseAfterClick = true;

    updateGL();

}

void SideGLWidget::keyReleaseEvent(QKeyEvent *) {

}
