#include "PlaneMesh.h"

void MeshChain::Draw(float pushOut) const {

    /*
    glBegin(GL_POINTS);
    for (int i=0; i<segments.size(); ++i) {

        Vector3 normskew=segments[i].n  * pushOut;
        (segments[i].pt1 + normskew).Draw();
        (segments[i].pt2 + normskew).Draw();

    }
    glEnd();
    */

    if (isClosed)
        glBegin(GL_LINE_LOOP);
    else
        glBegin(GL_LINE_STRIP);

    for (int i=0; i<segments.size(); ++i) {

        //JAMES EDIT
        segments[i].pt1.Draw();

        /*
        const Vector3 normskew=segments[i].n * pushOut;
        (segments[i].pt1 + normskew).Draw();
        */
        //(meshSegments[i].pt2 + normskew).Draw();

    }

    glEnd();

}

void MeshChain::Reverse() {

    std::reverse(segments.begin(), segments.end());

    for (int j=0; j<segments.size(); ++j) {

        Vector3 temp=segments[j].pt1;
        segments[j].pt1=segments[j].pt2;
        segments[j].pt2=temp;

    }

}

float MeshChain::Length() const
{

    float len = 0.0f;

    for (int j=0; j<segments.size()-1; ++j) {
        len += (segments[j+1].pt1 - segments[j].pt1).Length();
    }

    if (isClosed) {
        len += (segments.first().pt1 - segments.last().pt1).Length();
    }

    return len;
}

Vector3 MeshChain::StartTan() const
{
    Vector3 tan = segments[0].pt1 - segments[1].pt1;
    tan.Normalize();
    return tan;
}

Vector3 MeshChain::EndTan() const
{
    Vector3 tan = segments.last().pt1 - segments[segments.size()-2].pt1;
    tan.Normalize();
    return tan;
}

void PlanarCut::DrawSegments() const {

    glBegin(GL_LINES);
    for (int i=0; i<segments.size(); ++i) {

        segments[i].pt1.Draw();
        segments[i].pt2.Draw();

    }
    glEnd();

}

void PlanarCut::DrawChains(float pushOut) const {

    for (int i=0; i<chains.size(); ++i)
        chains[i].Draw(pushOut);

}

void PlanarCut::DrawTrisWireframe() const {

    glBegin(GL_LINES);
    for (int j=0; j<tris.size(); ++j) {

        tris[j].p1.Draw();
        tris[j].p2.Draw();

        tris[j].p2.Draw();
        tris[j].p3.Draw();

        tris[j].p3.Draw();
        tris[j].p1.Draw();

    }
    glEnd();

}

void PlanarCut::DrawTrisFilled() const {

    glBegin(GL_TRIANGLES);
    for (int j=0; j<tris.size(); ++j) {

        glNormal3d(params.norm.x,params.norm.y,params.norm.z);

        tris[j].p1.Draw();
        tris[j].p2.Draw();
        tris[j].p3.Draw();

    }
    glEnd();

}

PlaneMesh::PlaneMesh(SampleCamera *cam) :
    inputMesh(NULL),
    camera(cam),
    selectedPlaneIndex(INT_MAX),
    m_bDoTriangulation(false),
    m_bCheckForHoles(true),
    planeManip(NONE),
    m_bUseFaceSeg(false),
    m_bCubicHermiteSegmentInterp(true),
    m_bForceUseOneChainOnly(false),
    m_fHermiteTanScaleFactor(1.0f),
    m_fCurvatureSmoothing(0.05f),
    m_baesthetic(false),
    m_baestheticthresh(0.8f)
{

    m_backColor = QColor(255, 255, 255);
    m_meshLightColor = QColor(255.0f * 0.8f, 255.0f * 0.8f, 255.0f * 0.8f);

    inputMesh=new Mesh();

    //ridge valley stuff...
    meshfilename=new char[500];
    themesh=NULL; //trimesh2's version of the mesh    

    featureTypeStrings[0]="Segment PCA Planes";
    featureTypeStrings[1]="Segment PCA Axes",
    featureTypeStrings[2]="Ridges",
    featureTypeStrings[3]="Valleys",
    featureTypeStrings[4]="Symmetry Planes",
    featureTypeStrings[5]="Symmetry Axes",
    featureTypeStrings[6]="Global XYZ",
    featureTypeStrings[7]="Global PCA",
    featureTypeStrings[8]="Perpendicular Support",
    featureTypeStrings[9]="Symmetric Support";

    for (int i=0; i<NUM_FEATURE_TYPES; ++i)
        m_bUseFeature[i]=true;


}

void PlaneMesh::SetDoTriangulation(bool b)
{
    m_bDoTriangulation = b;

    UpdatePlaneCuts();
}

void PlaneMesh::SetCheckForHoles(bool b)
{
    m_bCheckForHoles = b;
}

void PlaneMesh::OutputHTMLTable(std::ostream & os, const QList <QList <int> >  & fData1, const QList <QList <int> > & fData2, const QList <QString> & rowLabels, const QList <QString> & colLabels) {

    os.precision(3);

    os << "<html>" << std::endl;
    os << "<head>" << std::endl;    
    os << "</head>" << std::endl;
    os << "<body>" << std::endl;

    os << "<table border=1 cellpadding=5 cellspacing=0>" << std::endl;
    os << "<tr>" << std::endl;
    os << "<th width=80px></th>" << std::endl;

    for (int i=0; i<colLabels.size(); ++i) {

        os << "<th width=80px>" << colLabels[i].toStdString() << "</th>" << std::endl;

    }

    os << "</tr>" << std::endl;

    for (int i=0; i<fData1.size(); ++i) {

        os << "<tr>" << std::endl;

        if (i<rowLabels.size())
            os << "<td><center>" << rowLabels[i].toStdString() << "</center></td>" << std::endl;
        else
            os << "<td></td>" << std::endl;       

        for (int j=0; j<fData1[i].size(); ++j) {

            int hexValue=( (float)fData1[i][j] / (float)fData2[i][j] )*255.0f;
            os << "<td bgcolor=#FF" << std::hex << std::uppercase << 255-hexValue
                    << std::hex << std::uppercase << 255-hexValue
                    << ">" << std::dec << fData1[i][j] << " / " << fData2[i][j] << "</td>" << std::endl;

        }

        os << "</tr>" << std::endl;

    }

    os << "</table>" << std::endl;

    os << "</body>" << std::endl;
    os << "</html>" << std::endl;



}

void PlaneMesh::GetKeyPlanes_PCAForMeshSegments() { 

    //Clear pca line/axis feature set
    pcaPlaneFits.clear();
    pcaLineFits.clear();

    //Return if neither are enabled
     if (!m_bUseFeature[P_PCA] && !m_bUseFeature[A_PCA])
         return;

    //Otherwise proceed (with one or both)
    const float eigenValFactor=0.25f;
    //float eigenValFactor=0.0001f; //this makes us use 3 pca planes all the time

    const bool alwaysUse3Planes = false;

    for (int j=0; j<m_nSegments; ++j) {

        QList <Vector3> evec;
        QList <float> eval;
        Vector3 centroid;

        PCAPlanesForMeshSegment(j, evec, eval, centroid);

        //based on eigenvals, decided what to do
        //qDebug("Eigenvals: %f %f %f for segment %i.",eval[0],eval[1],eval[2],j);

        if (alwaysUse3Planes ||
            (eval[1] > eval[0] * eigenValFactor && eval[2] > eval[0] * eigenValFactor)) { //3 principal dirs -> makes 3 PCA planes

            //Go through mesh...  First get segment midpoint
            const QVector <QVector <int> > & faceList=inputMesh->getFaces();
            const int numFaces=faceList.size();
            const QVector <Vector3> & verts=inputMesh->getVerts();

            //compute segment boundrad
            float maxdist = -FLT_MAX;
            for (int i=0; i < numFaces; ++i) {

                if ( m_vMeshFaceSeg[i] != j )
                    continue;

                for (int k=0; k<3; ++k) {

                    const Vector3 & v = verts[faceList[i][k]];

                    float dist = (v - centroid).Length();
                    maxdist = max(maxdist, dist);

                }

            }

            for (int i=0; i<3; ++i) {

                Plane eachPlane;

                eachPlane.norm=evec[i];
                eachPlane.norm.Normalize();
                eachPlane.pt = eachPlane.norm.Dot(centroid);
                eachPlane.featcentroid = centroid;
                eachPlane.extent = maxdist * 0.95f;
                if (alwaysUse3Planes)
                    eachPlane.extent = 0.0f;

                SetPlaneParamsFromNormPt(eachPlane);

                if (m_bUseFeature[P_PCA])
                    pcaPlaneFits.push_back(eachPlane);

            }

        }
        else if (eval[1] > eval[0]*eigenValFactor) { //2 principal dirs -> makes 1 PCA plane

            //Go through mesh...  First get segment midpoint
            const QVector <QVector <int> > & faceList=inputMesh->getFaces();
            const int numFaces=faceList.size();
            const QVector <Vector3> & verts=inputMesh->getVerts();

            float maxdist=-FLT_MAX;
            float centroidDotEvec2=centroid.Dot(evec[2]);

            for (int i=0; i<numFaces; ++i) {

                if ( m_vMeshFaceSeg[i] != j )
                    continue;

                for (int k=0; k<3; ++k) {

                    const Vector3 & v = verts[faceList[i][k]];

                    float dotProdDiffAbs=fabs(v.Dot(evec[2])-centroidDotEvec2);
                    maxdist=max(maxdist,dotProdDiffAbs);

                }                

            }
            //qDebug("FOR SINGLE PLANE ADD maxdist: %f",maxdist);

            Plane eachPlane;

            eachPlane.norm=evec[2];
            eachPlane.norm.Normalize();
            eachPlane.pt = eachPlane.norm.Dot(centroid);
            eachPlane.featcentroid = centroid;
            eachPlane.extent = maxdist * 0.95f;

            SetPlaneParamsFromNormPt(eachPlane);

            if (m_bUseFeature[P_PCA])
                pcaPlaneFits.push_back(eachPlane);

        }
        else { // 1 principal dir -> makes 1 PCA line           

            Axis axis;

            axis.dir=evec[0];
            axis.dir.Normalize();
            axis.len=evec[0].Length();
            axis.p=centroid;

            if (m_bUseFeature[A_PCA])
                pcaLineFits.push_back(axis);
        }    

    }

}

void PlaneMesh::GetFeatures_MeshValleys() {

    const float valleyAxisCost = 0.001f;

    // Clear valley features
    valleycut.segments.clear();
    valleyAxes.clear();
    for (int i=0; i<valleyAxesChains.size(); ++i)
        valleyAxesChains[i].clear();
    valleyAxesChains.clear();
    valleyAxesChainsAvgNorm.clear();
    valleyAxesChainsAvgCurv.clear();
    valleyAxesChainsLength.clear();

    // Return if valleys aren't enabled
    if (!m_bUseFeature[A_VALLEY])
        return;

    //Otherwise, do valley feature processing...
    QList <Vector3> valleysegments;
    QList <Vector3> valleysegmentnorms;
    QList <float> valleyness;

    MeshRidges(false, m_fRidgeValleyThreshold, valleysegments, valleysegmentnorms, valleyness);

    for (int i=0; i<valleysegments.size(); i+=2) {

        MeshSegment eachsegment;
        eachsegment.pt1=valleysegments[i];
        eachsegment.pt2=valleysegments[i+1];
        eachsegment.curv = (valleyness[i] + valleyness[i+1]) / 2.0f;

        valleycut.segments.push_back(eachsegment);
    }

    ChainRidgeSegments(valleycut);

    //qDebug("Produced %i valleychains.",valleycut.chains.size());

    //Get piecewise planes from ridges/valleys
    if (!valleycut.chains.empty()) {

        for (int j=0;j<valleycut.chains.size();++j) {

            QList <Axis> eachaxes;
            QList <QList <MeshSegment> > eachchains;

            PiecewiseAxesFromCurve(valleycut.chains[j].segments, valleyAxisCost, eachaxes, eachchains);
            //qDebug("\tGenerated %i planes for valleychain %i.", eachaxes.size(), j);

            for (int i=0;i<eachaxes.size();++i)
                valleyAxes.append(eachaxes[i]);

            for (int i=0;i<eachchains.size();++i)
                valleyAxesChains.append(eachchains[i]);

        }
    }

    //Calculate average norm for each of the piecewise plane chains
    //Norm is calculated by summing norms across each segment, weighted by segment length
    //Result is normalized for avg norm across chain

    for (int i=0; i<valleyAxesChains.size(); ++i) {

        Vector3 eachn=Vector3(0,0,0);

        for (int j=0; j<valleyAxesChains[i].size(); ++j)
            eachn+=valleyAxesChains[i][j].n*(valleyAxesChains[i][j].pt2-valleyAxesChains[i][j].pt1).Length();

        eachn.Normalize();
        //flip normal (because valley)
        eachn = eachn * (-1.0f);
        valleyAxesChainsAvgNorm.append(eachn);

        //compute average curvature for subchain
        float eachcurv = 0.0f;
        float totallen = 0.0f;
        for (int j=0; j<valleyAxesChains[i].size(); ++j) {

            float eachlen = (valleyAxesChains[i][j].pt2-valleyAxesChains[i][j].pt1).Length();
            totallen += eachlen;

            eachcurv += valleyAxesChains[i][j].curv * eachlen;

        }
        eachcurv /= totallen;
        valleyAxesChainsAvgCurv.append(eachcurv);
        valleyAxesChainsLength.append(totallen);

    }

}

void PlaneMesh::GetFeatures_MeshRidges() {

    const float ridgeAxisCost = 0.001f;

    /*
     * Clear ridge feature set
     */
    ridgecut.segments.clear();
    ridgeAxes.clear();
    for (int i=0; i<ridgeAxesChains.size(); ++i)
        ridgeAxesChains[i].clear();
    ridgeAxesChains.clear();
    ridgeAxesChainsAvgNorm.clear();
    ridgeAxesChainsAvgCurv.clear();
    ridgeAxesChainsLength.clear();

    //Return if not to process ridges
    if (!m_bUseFeature[A_RIDGE])
        return;

    /*
     * Onward with processing ridges
     */
    QList <Vector3> ridgesegments;
    QList <Vector3> ridgesegmentnorms;
    QList <float> ridgeness;

    MeshRidges(true, m_fRidgeValleyThreshold, ridgesegments, ridgesegmentnorms, ridgeness);

    for (int i=0; i<ridgesegments.size(); i+=2) {

        MeshSegment eachsegment;
        eachsegment.pt1=ridgesegments[i];
        eachsegment.pt2=ridgesegments[i+1];
        eachsegment.n=ridgesegmentnorms[i];
        eachsegment.curv = (ridgeness[i] + ridgeness[i+1]) / 2.0f;

        ridgecut.segments.push_back(eachsegment);

    }

    ChainRidgeSegments(ridgecut);

    //qDebug("Produced %i ridgechains.",ridgecut.chains.size());

    //Get piecewise planes from ridges/valleys
    if (!ridgecut.chains.empty()) {

        for (int j=0; j<ridgecut.chains.size(); ++j) {

            //qDebug("Chain's length: %i",ridgecut.chains[j].segments.size());

            //QList <Plane> eachplanes;
            QList <Axis> eachaxes;
            QList <QList <MeshSegment> > eachchains;

            //PiecewisePlanesFromCurve(ridgecut.chains[j].segments, 1, eachplanes, eachchains);
            //we are now using piecewise axes to define curves (weighed by length)
            PiecewiseAxesFromCurve(ridgecut.chains[j].segments, ridgeAxisCost, eachaxes, eachchains);
            //qDebug("\tGenerated %i axes for ridgechain %i.",eachaxes.size(),j);

            //for (int i=0;i<eachplanes.size();++i)
            //    qDebug("Plane %i: n=%f,%f,%f  d=%f",i,eachplanes[i].norm.x,eachplanes[i].norm.y,eachplanes[i].norm.z,eachplanes[i].pt);

            for (int i=0; i<eachaxes.size(); ++i)
                ridgeAxes.append(eachaxes[i]);

            for (int i=0; i<eachchains.size(); ++i)
                ridgeAxesChains.append(eachchains[i]);

        }
    }

    //Calculate average norm for each of the piecewise plane chains
    //Norm is calculated by summing norms across each segment, weighted by segment length
    //Result is normalized for avg norm across chain

    for (int i=0; i<ridgeAxesChains.size(); ++i) {

        //compute average normal for subchain
        Vector3 eachn=Vector3(0,0,0);

        for (int j=0; j<ridgeAxesChains[i].size(); ++j)
            eachn+=ridgeAxesChains[i][j].n*(ridgeAxesChains[i][j].pt2-ridgeAxesChains[i][j].pt1).Length();

        eachn.Normalize();
        ridgeAxesChainsAvgNorm.append(eachn);
        //qDebug("\tRidgechain %i average norm: %f,%f,%f",i,eachn.x,eachn.y,eachn.z);

        //compute average curvature for subchain
        float eachcurv = 0.0f;
        float totallen = 0.0f;
        for (int j=0; j<ridgeAxesChains[i].size(); ++j) {

            float eachlen = (ridgeAxesChains[i][j].pt2-ridgeAxesChains[i][j].pt1).Length();
            totallen += eachlen;

            eachcurv += ridgeAxesChains[i][j].curv * eachlen;

        }
        eachcurv /= totallen;
        ridgeAxesChainsAvgCurv.append(eachcurv);
        ridgeAxesChainsLength.append(totallen);


    }

}

QString PlaneMesh::GetModelFilename(const int mNum)
{

    QString fName("../objs/");
    fName += QString::number(mNum);
    fName += ".off";

    return fName;
}

QString PlaneMesh::GetUserModelFilename(const int uNum, const int mNum)
{

    QString fName("../../userstudy/user");
    fName += QString::number(uNum);
    fName += "/";
    fName += QString::number(mNum);
    fName += ".off.obj";

    return fName;
}

bool PlaneMesh::LoadInputMesh(char * filename, char * modelName, char * segName, bool loadHumanData) {

    strcpy(meshfilename, filename);

    if (themesh != NULL)
        delete themesh;

    /*
     * Clear all mesh features
     */
    pcaPlaneFits.clear();
    pcaLineFits.clear();

    for (int i=0;i<ridgeAxesChains.size();++i)
        ridgeAxesChains[i].clear();
    ridgeAxesChains.clear();
    ridgeAxes.clear();

    for (int i=0;i<valleyAxesChains.size();++i)
        valleyAxesChains[i].clear();
    valleyAxesChains.clear();
    valleyAxes.clear();

    symPlanes.clear();
    symLines.clear();

    //Global XYZ/PCA planes
    globalxyzPlanes.clear();
    globalpcaPlanes.clear();

    perpSupportPlanes.clear();
    symSupportPlanes.clear();

    /*
     * Mesh normalization step (create temp file with normalized mesh for other libs to use),
     * as well as loading model into our own mesh structure
     */

    QString qFilename=QString(filename);
    QString tempFilename;

    if (QString::compare(qFilename.right(4), QString(".off"), Qt::CaseInsensitive) == 0) {

        tempFilename = QString("temp.off");

        OFFFile tempoff;
        tempoff.Read(filename);
        tempoff.Normalize();
        tempoff.Write(tempFilename.toAscii().data());

        inputMesh->SetVertsAndFaces(tempoff.GetVerts(), tempoff.GetFaces());
        inputMesh->FlipFaceNorms();

    }
    else if (QString::compare(qFilename.right(4),QString(".obj"), Qt::CaseInsensitive) == 0) { //otherwise assume obj

        tempFilename = QString("temp.obj");

        ObjFileSimple tempobj;
        tempobj.SimpleRead(filename);
        tempobj.Normalize();
        tempobj.Write(tempFilename.toAscii().data());

        QVector <Vector3> verts;
        tempobj.GetVerts(verts);

        QVector <QVector <int> > faces;
        tempobj.GetFaces(faces);

        inputMesh->SetVertsAndFaces(verts, faces);
        //inputMesh->FlipFaceNorms();

    }

    /*
     * Load mesh in trimesh2 (compute ridges and valleys) using the normalized temp model we wrote
     */

    themesh = TriMesh::read(tempFilename.toAscii().data());

    if (themesh == NULL) {
        qDebug() << "TriMesh::read returned NULL";
    }

    themesh->need_tstrips();
    themesh->need_bsphere();

    themesh->need_normals();
    themesh->need_curvatures();
    themesh->need_dcurv();   

    // Diffuse the curvatures across the mesh
    diffuse_curv(themesh, m_fCurvatureSmoothing);
    // Diffuse the curvature derivatives across the mesh
    diffuse_dcurv(themesh, m_fCurvatureSmoothing);

    //PIECEWISE PLANES FIGURE STUFF
    /*
    inputMesh->MakeSpiralMesh(0.1);        

    QList <Plane> piecewisePlanes;    

    //used 1.0 and 4.0 for plane cost
    PiecewisePlanesFromCurve(inputMesh->GetSpiralMeshSpine(), 0.25f, piecewisePlanes, piecewiseFigureChains);
    qDebug("Made %i planes.",piecewisePlanes.size());

    for (int i=0;i<piecewisePlanes.size();++i) {

        PlanarCut eachcut;
        eachcut.params=piecewisePlanes[i];

        //eachcut.params.rotAxis1=eachcut.params.rotAxis1*0.5;
        //eachcut.params.rotAxis2=eachcut.params.rotAxis2*0.5;

        eachcut.params.norm.Normalize();
        eachcut.params.rotAxis1.Normalize();
        eachcut.params.rotAxis2.Normalize();

        piecewiseFigureCuts.push_back(eachcut);

        ConstructPlanePiece(piecewiseFigureCuts[i]);
    }

    return true;
    */


    /*
     * On with the normal file load stuff...
     */   

    if (QString::compare(tempFilename.right(4), QString(".off"), Qt::CaseInsensitive) == 0) {

        /*
         * Load segmentation file (if it exists)
         */
        QString segFilename;

        if (segName==NULL) {
            segFilename=qFilename.left(qFilename.length() - 4);
            segFilename.append("_0.seg");
        }
        else
            segFilename = QString(segName);

        FILE *inf=fopen(segFilename.toAscii().data(),"r");

        m_vMeshFaceSeg.clear();

        if (inf!=NULL) {

            qDebug("Found segmentation file %s, loading segments...",segFilename.toAscii().data());

            m_vMeshFaceSeg.reserve(inputMesh->getFaces().size());

            char *eachLine=new char[100];

            while (!feof(inf)) {
                if (fgets(eachLine,100,inf))
                    m_vMeshFaceSeg.push_back(atoi(eachLine));
            }

            fclose(inf);

            delete [] eachLine;

        }
        else {

            //if no seg file, make all mesh a single segment
            m_vMeshFaceSeg = QVector <int> (inputMesh->getFaces().size(), 0);

        }

    }
    else if (QString::compare(tempFilename.right(4),QString(".obj"), Qt::CaseInsensitive) == 0) { //otherwise assume obj

        //hacky way of segmenting .obj's specifically using face groups
        m_vMeshFaceSeg.clear();

        QFile file(qFilename);

        if (!file.open( QIODevice::ReadOnly | QIODevice::Text )) {
            qDebug() << "File " << qFilename << " can't be opened for read.";
        }

        QTextStream ifs( &file );

        int curSegment = 0;

        while (!ifs.atEnd()) {

            QStringList line = ifs.readLine().split(" ");
            if (line[0].compare("g") == 0) {
                ++curSegment;
            }
            else if (line[0].compare("f") == 0) {
                m_vMeshFaceSeg.push_back(curSegment);
            }

        }

        file.close();

    }
    else {
        return false;
    }

    ClearPlanes();   


    //get the number of distinct mesh segments (using a set)

    QSet <int> segset;

    for (int i=0; i<m_vMeshFaceSeg.size(); ++i)
        segset.insert(m_vMeshFaceSeg[i]);

    m_nSegments=segset.size();

    /*
     * Load symmetry planes content, if it exists
     */
    QString symFilename=qFilename.left(qFilename.length()-4);
    symFilename.append(".sym");

    FILE *inf=fopen(symFilename.toAscii().data(),"r");

    symPlanes.clear();
    symLines.clear();

    if (inf!=NULL) {

        qDebug("Found symmetry file %s, loading symmetry planes/axes...",symFilename.toAscii().data());

        char *eachline=new char[500];

        while (fgets(eachline,500,inf)) {

            QString eachqline(eachline);            
            QStringList eachtoken=eachqline.split(" ");

            if (eachtoken[0].contains("P")) {

                Plane each;
                each.norm.x=eachtoken[1].toFloat();
                each.norm.y=eachtoken[2].toFloat();
                each.norm.z=eachtoken[3].toFloat();
                each.pt=eachtoken[4].toFloat();

                SetPlaneParamsFromNormPt(each);

                symPlanes.push_back(each);

            }
            else if (eachtoken[0].contains("A")) {

                Axis each;

                each.dir.x=eachtoken[1].toFloat();
                each.dir.y=eachtoken[2].toFloat();
                each.dir.z=eachtoken[3].toFloat();
                each.p.x=eachtoken[4].toFloat();
                each.p.y=eachtoken[5].toFloat();
                each.p.z=eachtoken[6].toFloat();

                each.len=each.dir.Length();
                each.dir.Normalize();

                symLines.push_back(each);

            }                             

        }

        fclose(inf);

        delete [] eachline;
    }

    /*
     * Load human authored planes, if checkbox enabled, and if they exist
     */

    figureCuts.clear();
    figureColours.clear();

    if (loadHumanData) {

        qDebug("Loading human cuts...");

        QList <QString> objFilenames;

        //QString basefolder=QString("/home/james/Desktop/projects/planarmesh/userstudy");

        QString basefolder=QString("../userstudy");
        QString eachfilename=qFilename.right(qFilename.length()-qFilename.lastIndexOf("/"))+".obj";

        //const float userNum = 17;
        //for (int i=userNum; i<=userNum; ++i) //test particular user
        for (int i=1; i<=NUM_HUMAN_DATASETS; ++i)
            objFilenames.push_back(QString(basefolder+QString("/user")+QString::number(i)+eachfilename));

        for (int i=0;i<objFilenames.size(); ++i) {

            char buffer[500];
            QString eachLine;
            QStringList eachSplitLine;

            ifstream ifs;
            ifs.open(objFilenames[i].toAscii().data(), ifstream::in);

            if (!ifs.is_open()) {
                qDebug("Failed to load file %s!",objFilenames[i].toAscii().data());
                continue;
            }

            ifs.getline(buffer, 500);

            eachLine=QString(buffer);
            eachSplitLine=eachLine.split(" ");

            int numPlanes=eachSplitLine[1].toInt();

            ifs.getline(buffer, 500);

            QList <PlanarCut> eachCuts;

            for (int j=0; j<numPlanes; ++j) {

                ifs.getline(buffer, 500);

                eachLine=QString(buffer);
                eachSplitLine=eachLine.split(" ");

                PlanarCut eachCut;

                eachCut.params.norm=Vector3(eachSplitLine[1].toFloat(),eachSplitLine[2].toFloat(),eachSplitLine[3].toFloat());
                eachCut.params.cent=Vector3(eachSplitLine[4].toFloat(),eachSplitLine[5].toFloat(),eachSplitLine[6].toFloat());                                
                eachCut.params.pt=eachCut.params.norm.Dot(eachCut.params.cent);
                eachCut.params.rotAxis1=Vector3(eachSplitLine[8].toFloat(),eachSplitLine[9].toFloat(),eachSplitLine[10].toFloat());
                eachCut.params.rotAxis2=Vector3(eachSplitLine[11].toFloat(),eachSplitLine[12].toFloat(),eachSplitLine[13].toFloat());

                GetSegments(eachCut);
                GetChains(eachCut);

                if (!eachCut.segments.empty()) //cut must have positive area intersection with mesh
                    eachCuts.push_back(eachCut);

            }

            ifs.close();

            figureCuts.push_back(eachCuts);
            figureColours.push_back(SetColorByIndex(i));

        }

        //for figure output        
        //figure 2/3 processing:

        //column headings
        if (colLabelsFigure2.empty()) {
            for (int i=0; i<objFilenames.size(); ++i) {                
                colLabelsFigure2.push_back(QString("User ")+QString(i+1));
            }
        }

        //row heading
        rowLabelsFigure2.push_back(modelName);

        //figure "2" data - number of planes used by each user on model
        QList <int> newRowDataFigure2;
        QList <int> numPlanesVec;
        int nAvgNumPlanes = 0;

        //use ceil of average # of planes
        for (int i=0; i < figureCuts.size(); ++i) {
            //qDebug() << figureCuts[i].size(); //OUTPUT EACH USER # OF PLANES
            nAvgNumPlanes+=figureCuts[i].size();
            newRowDataFigure2.push_back(figureCuts[i].size());
        }

        nAvgNumPlanes = ceil( ((float)nAvgNumPlanes) / NUM_HUMAN_DATASETS );        

        //use median number of planes.. first get list of # of planes used by each user        
        fFigure2Data.push_back(newRowDataFigure2);


        //instead of superproxy selection (above), we add all planes not covering a feature
        /*
        ComputeMeshFeatures();
        PopulateBinSpaceWithFeatures();

        for (int i=0; i<figureCuts.size(); ++i) {
            for (int j=0; j<figureCuts[i].size(); ++j) {

                int theta, phi, r;
                m_BinSpace.CartesianToPolar(figureCuts[i][j].params.norm, figureCuts[i][j].params.pt, theta, phi, r);

                if (m_BinSpace.GetBin(theta, phi, r).f.empty()) {

                    PlanarCut bestPcut;
                    bestPcut.params = figureCuts[i][j].params;

                    GetPlanarCut(bestPcut);
                    m_vPlanarCuts.push_back(bestPcut);

                }

            }
        }
        */



        //write these out to obj file for rendering
        /*
        QString filename("manbinspace.obj");
        QFile file(filename);

        if (!file.open( QIODevice::WriteOnly | QIODevice::Text )) {
            qDebug() << "File " << filename << " can't be opened for write.";
            return false;
        }

        QTextStream ofs( &file );
        int verts = 0;

        QVector <const PlaneSpaceBin *> bins = m_BinSpace.GetAllPopulatedBins();

        for (int j=0; j<8; ++j) {

            ofs << "g featuretype" << j << "\n";

            for (int theta=0; theta<m_BinSpace.m_nMaxTheta; theta+=m_BinSpace.m_nIncr) {
                for (int phi=0; phi<m_BinSpace.m_nMaxPhi; phi+=m_BinSpace.m_nIncr) {
                    for (int r=0; r<m_BinSpace.m_nMaxR ; r+=m_BinSpace.m_nIncr) {

                        PlaneSpaceBin bin = m_BinSpace.GetBin(theta, phi, r);

                        int maxind = -1;
                        float maxval = -FLT_MAX;
                        for (int k=0; k<bin.f.size(); ++k) {
                            if (bin.f[k].value > maxval) {
                                maxind = k;
                                maxval = bin.f[k].value;
                            }
                        }

                        if (bin.f[maxind].ftype == j && maxval > 0.0f) {
                            float sz = qMin(1.0f, maxval); // * float(m_BinSpace.GetIncr());
                            Vector3 p1 = Vector3(float(theta) - sz, float(phi) - sz, float(r) - sz);
                            Vector3 p2 = Vector3(float(theta) + sz, float(phi) + sz, float(r) + sz);
                            WriteBoxGeom(ofs, p1, p2, verts);
                        }
                    }
                }
            }

        }        

        file.close();     
        */

        //figure "3" data - number of "unique" planes selected by each user        

        //we bin each user's plane       

        m_BinSpace.Clear();

        int curFeatIndex=0;
        QMap <int, int> featToUserMap;
        QMap <int, int> featToUsersCutMap;

        for (int i=0; i < figureCuts.size(); ++i) {
            for (int j=0; j < figureCuts[i].size(); ++j) {

                const float fGroupDist = 0.1f;
                m_BinSpace.AddPointFeature(figureCuts[i][j].params.norm, figureCuts[i][j].params.pt, fGroupDist, curFeatIndex, i, 1.0f);

                featToUserMap[curFeatIndex]=i;
                featToUsersCutMap[curFeatIndex]=j;

                curFeatIndex++;

            }
        }

        QList <Plane> meanPlanes;
        PlaneSpaceBin eachMaxBin;

        //
        // This algo chooses amongst the user study planes ones that are
        // clustered together/well represented
        //
        while (true) {

            //eachMaxBin = m_BinSpace.GetMaxFeaturesBin();
            //if (eachMaxBin.f.size() < 4)
            eachMaxBin = m_BinSpace.GetMaxValueBin();
            if (eachMaxBin.totalvalue < 4.5f)
                break;

            qDebug() << "Plane " << meanPlanes.size()+1 << "value" << eachMaxBin.totalvalue;

            Plane p;
            p.norm=eachMaxBin.cartesianN;
            p.pt=0.0f;

            for (int j=0; j<eachMaxBin.f.size(); ++j) {

                int featInd = eachMaxBin.f[j].fi;
                int uInd = featToUserMap[featInd];
                int cInd = featToUsersCutMap[featInd];

                Q_ASSERT ( uInd < figureCuts.size() );
                Q_ASSERT ( cInd < figureCuts[uInd].size() );

                if ( p.norm.Dot(figureCuts[uInd][cInd].params.norm) > 0.0f )
                    p.pt+=figureCuts[uInd][cInd].params.pt;
                else
                    p.pt-=figureCuts[uInd][cInd].params.pt;

            }

            p.pt/=float(eachMaxBin.f.size());
            SetPlaneParamsFromNormPt(p);

            PlanarCut bestPcut;
            bestPcut.params=p;                                   

            //float fRad = float(m_BinSpace.GetIncr()) * 0.5f;
            //LocalSearchFindMaxAreaPlane(bestPcut, fRad, fRad/32.0f);

            GetPlanarCut(bestPcut);

            m_vPlanarCuts.push_back(bestPcut);            
            m_vPlanarCuts.last().nVotes = eachMaxBin.f.size();
            m_vPlanarCuts.last().fConfidence = eachMaxBin.totalvalue;

            m_BinSpace.RemoveFeaturesOfMaximalBin(eachMaxBin);

            meanPlanes.push_back(bestPcut.params);

        //}
        }

        //the binspace selected planes are for a "virtual ideal user", we then compare each user's planes to this set
        QList <int> newRowDataFigure3;
        figureCutsUnselected.clear();

        for (int i=0; i < figureCuts.size(); ++i) {

            int nUniquePlanes = 0;

            for (int j=0; j < figureCuts[i].size(); ++j) {

                bool planeUnique = true;

                for (int k=0; k < meanPlanes.size(); ++k) {

                    float eachDistance = GetPlaneDistance(figureCuts[i][j].params, meanPlanes[k]);

                    if (eachDistance < 0.1f)
                        planeUnique=false;

                }

                if (planeUnique) {
                    ++nUniquePlanes;
                    figureCutsUnselected.push_back(figureCuts[i][j].params);
                }

            }

            newRowDataFigure3.push_back( nUniquePlanes );

            //qDebug() << nUniquePlanes;

        }        

        fFigure3Data.push_back(newRowDataFigure3);

        ofstream outfile("usercuts-stats.html");
        OutputHTMLTable(outfile, fFigure3Data, fFigure2Data, rowLabelsFigure2, colLabelsFigure2);
        outfile.close();

        qDebug() << "Binning normal-deviation distribution...";
        QVector <float> normBin(91, 0.0f);
        for (int i=0; i<figureCuts.size(); ++i) {
            for (int j=0; j<figureCuts[i].size(); ++j) {

                Plane & p = figureCuts[i][j].params;

                for (int k=0; k<figureCuts[i][j].segments.size(); ++k) {

                    MeshSegment & s = figureCuts[i][j].segments[k];
                    float len = (s.pt2 - s.pt1).Length();
                    float n = acosf(fabs(s.n.Dot(p.norm))) * 180.0f / 3.14159f;

                    normBin[int(n)] += len;

                }

            }
        }



        //for (int i=0; i<normBin.size(); ++i)
        //    qDebug() << normBin[i];

        /*
        float p2pmindist = FLT_MAX;
        for (int i=0; i<figureCuts.size(); ++i) {
            for (int j=0; j<figureCuts[i].size(); ++j) {
                for (int k=j+1; k<figureCuts[i].size(); ++k) {
                    p2pmindist = qMin(p2pmindist, GetPlaneDistance(figureCuts[i][j].params, figureCuts[i][k].params));
                }
            }
        }
        qDebug() << "PLANE TO PLANE MINDIST!" << p2pmindist;
        */


    }

    return true;

}

float PlaneMesh::GetBoundRad() {

    if (inputMesh==NULL)
        return 1.0f;
    else
        return inputMesh->GetBoundRad();
}

Vector3 PlaneMesh::GetBoundLo() {
    if (inputMesh==NULL)
        return Vector3(-1,-1,-1);
    else
        return inputMesh->GetBoundLo();
}

Vector3 PlaneMesh::GetBoundHi() {
    if (inputMesh==NULL)
        return Vector3(1,1,1);
    else
        return inputMesh->GetBoundHi();

}

void PlaneMesh::GetPlanarCut(PlanarCut & planarCut) {

    // 1. Make a plane cut    
    GetSegments(planarCut);

    // 2. Construct chains from plane cut
    GetChains(planarCut);

    // 3. Create tesselation from chains
    if (m_bDoTriangulation)
        GetTriangles(planarCut);

}

void PlaneMesh::PerformPlaneCuts() {

    for (int i=0;i<m_vPlanarCuts.size();++i) {

        if (m_vPlanarCuts[i].chains.empty())
            continue;

        m_vPlanarCuts[i].tris.clear();
        GetTriangles( m_vPlanarCuts[i]);

    }

}

void PlaneMesh::UpdatePlaneCuts() {

    for (int i=0;i<m_vPlanarCuts.size();++i)
        GetPlanarCut(m_vPlanarCuts[i]);

}

Vector3 PlaneMesh::GetRotatedAxis(const float angle, const Vector3 & vec, const Vector3 & axis) {

    if(angle == 0.0f)
        return vec;

    Vector3 RotationRow1, RotationRow2, RotationRow3;

    const float sinAngle = sinf(angle);
    const float cosAngle = cosf(angle);
    const float oneSubCos = 1.0f - cosAngle;

    RotationRow1.x = (axis.x) * (axis.x) + cosAngle * (1.0f - (axis.x) * (axis.x));
    RotationRow1.y = (axis.x) * (axis.y) * (oneSubCos) - sinAngle * axis.z;
    RotationRow1.z = (axis.x) * (axis.z) * (oneSubCos) + sinAngle * axis.y;

    RotationRow2.x = (axis.x) * (axis.y) * (oneSubCos) + sinAngle * axis.z;
    RotationRow2.y = (axis.y) * (axis.y) + cosAngle * (1.0f - (axis.y) * (axis.y));
    RotationRow2.z = (axis.y) * (axis.z) * (oneSubCos) - sinAngle * axis.x;

    RotationRow3.x = (axis.x) * (axis.z) * (oneSubCos) - sinAngle * axis.y;
    RotationRow3.y = (axis.y) * (axis.z) * (oneSubCos) + sinAngle * axis.x;
    RotationRow3.z = (axis.z) * (axis.z) + cosAngle * (1.0f - (axis.z) * (axis.z));

    return Vector3(vec.Dot(RotationRow1), vec.Dot(RotationRow2), vec.Dot(RotationRow3));

}

void PlaneMesh::AddPlanarCut(PlanarCut & planarCut) {

    m_vPlanarCuts.push_back(planarCut);
    selectedPlaneIndex=m_vPlanarCuts.size()-1;

}

void PlaneMesh::GetSegments(PlanarCut & planarCut) {

    int numFaces=inputMesh->getFaces().size();    
    QVector <QVector <int> > & inputFaces=inputMesh->getFaces();
    QVector <Vector3> & inputVerts=inputMesh->getVerts();
    QVector <Vector3> & inputNorms=inputMesh->getFaceNorms();

    planarCut.segments.clear();

    /*
     * Determine segment of mesh nearest to centre of cutplane (if segmentation enabled)
     */

    planarCut.nearSeg=-1;
    if (m_bUseFaceSeg&&!m_vMeshFaceSeg.empty()) {
        double minDist=DBL_MAX;
        for (int i=0;i<numFaces;++i) {
            if ((planarCut.params.cent-inputVerts[inputFaces[i][0]]).LengthSquared()<minDist) {
                minDist=(planarCut.params.cent-inputVerts[inputFaces[i][0]]).LengthSquared();
                planarCut.nearSeg=m_vMeshFaceSeg[i];
            }
        }
    }


    /*
     * Cut plane through each face of the mesh
     */
    for (int i=0; i<numFaces; ++i) {

        if (m_bUseFaceSeg && !m_vMeshFaceSeg.empty() && m_vMeshFaceSeg[i] != planarCut.nearSeg)
            continue;

        Triangle eachTri;
        Vector3 eachP1, eachP2;
        int eachE1, eachE2;

        eachTri.p1=inputVerts[inputFaces[i][0]];
        eachTri.p2=inputVerts[inputFaces[i][1]];
        eachTri.p3=inputVerts[inputFaces[i][2]];        

        //JAMES EDIT
        //if ( GetPlaneTriIntersect(planarCut.params, eachTri, eachP1, eachP2, eachE1, eachE2) && (eachP1-eachP2).Length() > 0.00001f ) {
        if (GetPlaneTriIntersect(planarCut.params, eachTri, eachP1, eachP2, eachE1, eachE2)) {

            //if performing "aesthetic cuts", remove all segments whose
            //absolute value of dot product of surface and plane cut
            //is less than the specified threshold
            if (m_baesthetic) {

                Vector3 & norm = planarCut.params.norm;
                Vector3 & surfnorm = inputNorms[i];

                if (fabs(norm.Dot(surfnorm)) > m_baestheticthresh) {
                    continue;
                }
            }

            MeshSegment newSegment;
            newSegment.pt1=eachP1;
            newSegment.pt2=eachP2;
            newSegment.n=inputNorms[i];
            newSegment.face=i;

            //save edge (edge defined as pair of vert indexes)
            newSegment.edge1 = MeshEdge(inputFaces[i][eachE1], inputFaces[i][(eachE1+1)%3]);
            newSegment.edge2 = MeshEdge(inputFaces[i][eachE2], inputFaces[i][(eachE2+1)%3]);

            planarCut.segments.push_back(newSegment);

        }

    }  

    //remove redundant segments (possible if 2 cuts are made along an edge shared by two faces)
    //JAMES EDIT
    /*
    for (int i=0;i<planarCut.segments.size();++i) {

        MeshSegment & seg1 = planarCut.segments[i];

        for (int j=i+1;j<planarCut.segments.size();++j) {

            MeshSegment & seg2 = planarCut.segments[j];

            if ( (seg1.pt1==seg2.pt1 && seg1.pt2 == seg2.pt2) || (seg1.pt2 == seg2.pt1 && seg1.pt1 == seg2.pt2) ) {

                planarCut.segments.removeAt(j);
                --j;

            }

        }
    }   
    */

}

bool PlaneMesh::GetPlaneTriIntersect(const Plane & plane, const Triangle & tri, Vector3 & pt1, Vector3 & pt2, int & edge1, int & edge2) {

    float s1, s2, s3;
    float u1, u2, u3;
    Vector3 d1, d2, d3;	

    d1=tri.p2-tri.p1;
    d2=tri.p3-tri.p2;
    d3=tri.p1-tri.p3;

    u1=plane.norm.Dot(d1);
    u2=plane.norm.Dot(d2);
    u3=plane.norm.Dot(d3);

    s1=-(plane.norm.Dot(tri.p1-plane.cent))/u1;
    s2=-(plane.norm.Dot(tri.p2-plane.cent))/u2;
    s3=-(plane.norm.Dot(tri.p3-plane.cent))/u3;

    if (s1>=0.0f&&s1<=1.0f&&s2>=0.0f&&s2<=1.0f) {
        pt1=tri.p1+d1*s1;
        pt2=tri.p2+d2*s2;
        edge1 = 0;
        edge2 = 1;
        return true;
    }
    else if (s1>=0.0f&&s1<=1.0f&&s3>=0.0f&&s3<=1.0f) {
        pt1=tri.p1+d1*s1;
        pt2=tri.p3+d3*s3;
        edge1 = 0;
        edge2 = 2;
        return true;
    }
    else if (s2>=0.0f&&s2<=1.0f&&s3>=0.0f&&s3<=1.0f) {
        pt1=tri.p2+d2*s2;
        pt2=tri.p3+d3*s3;
        edge1 = 1;
        edge2 = 2;
        return true;
    }

    return false;


}

int PlaneMesh::GetNumChainsOpen(const PlanarCut & planarCut) {

    int numOpen=0;

    for (int i=0; i<planarCut.chains.size(); ++i)
        if (!planarCut.chains[i].isClosed)
            ++numOpen;

    return numOpen;

}

void PlaneMesh::ChainRidgeSegments(PlanarCut & planarCut)
{

    planarCut.chains.clear();

    while (planarCut.segments.size() > 0) {

        MeshChain newChain;
        newChain.segments.push_back(planarCut.segments.last());
        planarCut.segments.pop_back();

        bool otherFound = true;
        bool otherFlip;
        bool otherAddBack;
        int otherInd;

        while (otherFound) {

            otherFound = false;

            MeshSegment & fs = newChain.segments.first();
            MeshSegment & ls = newChain.segments.last();

            for (int i=0; i<planarCut.segments.size(); ++i) {

                MeshSegment & each = planarCut.segments[i];

                if ((each.pt1 - ls.pt2).Length() < 0.001f) {
                    otherFound = true;
                    otherFlip = false;
                    otherAddBack = true;
                    otherInd = i;
                    break;
                }
                else if ((each.pt2 - ls.pt2).Length() < 0.001f) {
                    otherFound = true;
                    otherFlip = true;
                    otherAddBack = true;
                    otherInd = i;
                    break;
                }
                else if ((each.pt2 - fs.pt1).Length() < 0.001f) {
                    otherFound = true;
                    otherFlip = false;
                    otherAddBack = false;
                    otherInd = i;
                    break;
                }
                else if ((each.pt1 - fs.pt1).Length() < 0.001f) {
                    otherFound = true;
                    otherFlip = true;
                    otherAddBack = false;
                    otherInd = i;
                    break;
                }

            }

            if (otherFound) {

                MeshSegment & seg = planarCut.segments[otherInd];

                if (otherFlip)
                    seg.flip();

                if (otherAddBack)
                    newChain.segments.push_back(seg);
                else
                    newChain.segments.push_front(seg);

                planarCut.segments.removeAt(otherInd);

            }

        }

        planarCut.chains.push_back(newChain);

    }

}

void PlaneMesh::GetChains(PlanarCut & planarCut) {

    //const float minDist=0.00001f;
    //unsigned int curChainIndex=0;

    //clear current chains
    planarCut.area=0.0f;
    for (int i=0; i<planarCut.chains.size(); ++i)
        planarCut.chains[i].segments.clear();
    planarCut.chains.clear();

    //return if no segments to chain
    if (planarCut.segments.empty())
        return;   

    QList <int> segIndexList;
    for (int i=0; i<planarCut.segments.size(); ++i)
        segIndexList.push_back(i);

    //begin first segment
    MeshChain newChain;

    newChain.segments.push_back(planarCut.segments[segIndexList.first()]);
    segIndexList.removeFirst();
    planarCut.chains.push_back(newChain);

    //construct chains until no segments remain (indexed from original list)
    //JAMES EDIT
    while (!segIndexList.empty()) {

        MeshChain & curchain = planarCut.chains.last();
        MeshSegment & firstseg = curchain.segments.first();
        MeshSegment & lastseg = curchain.segments.last();

        bool foundnext = false;

        for (int i=0; i<segIndexList.size(); ++i) {

            MeshSegment & seg = planarCut.segments[segIndexList[i]];

            if (seg.edge1 == lastseg.edge2) {

                foundnext = true;
                curchain.segments.push_back(seg);               

            }
            else if (seg.edge2 == lastseg.edge2) {

                foundnext = true;
                seg.flip();
                curchain.segments.push_back(seg);                

            }
            else if (seg.edge2 == firstseg.edge1) {

                foundnext = true;
                curchain.segments.push_front(seg);

            }
            else if (seg.edge1 == firstseg.edge1) {

                foundnext = true;
                seg.flip();
                curchain.segments.push_front(seg);

            }

            if (foundnext) {
                segIndexList.removeAt(i);
                break;
            }

        }

        //check if closed
        if (!foundnext || (curchain.segments.last().edge2 == firstseg.edge1)) {

            /*
            qDebug() << "Creating chain " << planarCut.chains.size() << "with" << curchain.segments.size() << "segments."
                        << "Foundnext:" << foundnext
                        << "Firstseg:" << "E1" << firstseg.edge1.v1i << firstseg.edge1.v2i << "E2" << firstseg.edge2.v1i << firstseg.edge2.v2i
                        << "Lastseg:" << "E1" << lastseg.edge1.v1i << lastseg.edge1.v2i << "E2" << lastseg.edge2.v1i << lastseg.edge2.v2i;
                        */

            curchain.isClosed = true;
            curchain.isHole = false;

            //remove segments with tiny length
            for (int j=0; j<curchain.segments.size(); ++j) {

                MeshSegment & s = curchain.segments[j];

                if (!((s.pt1 - s.pt2).Length() > 0.0001f) || (j > 0 && !(s.pt1 - curchain.segments[j-1].pt1).Length() > 0.0001f)) {

                    curchain.segments.removeAt(j);
                    //qDebug() << "Removing segment" << j << "from chain.";

                    --j;

                }

            }
            //potentially remove this chain if not of min length
            if (curchain.segments.size() < 3)
                planarCut.chains.pop_back();


            //begin first segment
            if (!segIndexList.empty()) {

                MeshChain newChain;
                newChain.segments.push_back(planarCut.segments[segIndexList.first()]);
                planarCut.chains.push_back(newChain);

                segIndexList.removeFirst();

            }

        }

    }   

    /*
    if (m_baesthetic) {   

        //chain together the longer ones
        while (true) {

            int c1i = -1;
            int c2i = -1;
            float bestdist = FLT_MAX;
            bool flip1;
            bool flip2;

            //find 2 open chains with mindist to connect
            for (int i=0; i<planarCut.chains.size(); ++i) {

                const MeshChain & c1 = planarCut.chains[i];

                if (c1.isClosed)
                    continue;

                for (int j=0; j<planarCut.chains.size(); ++j) {

                    const MeshChain & c2 = planarCut.chains[j];

                    if (c2.isClosed)
                        continue;

                    //endpoint distances
                    float dist[4] = {
                        (c1.segments.last().pt2 - c2.segments.first().pt1).Length(),
                        (c1.segments.last().pt2 - c2.segments.last().pt2).Length(),
                        (c1.segments.first().pt1 - c2.segments.first().pt1).Length(),
                        (c1.segments.first().pt1 - c2.segments.last().pt2).Length()
                    };

                    //is end of curve1 "facing" endpoint of curve2?
                    bool sees1[4] = {
                        c1.EndTan().Dot(c2.segments.first().pt1 - c1.segments.last().pt2) > 0.0f,
                        c1.EndTan().Dot(c2.segments.last().pt2 - c1.segments.last().pt2) > 0.0f,
                        c1.StartTan().Dot(c2.segments.first().pt1 - c1.segments.first().pt1) > 0.0f,
                        c1.StartTan().Dot(c2.segments.last().pt2 - c1.segments.first().pt1) > 0.0f
                    };

                    //is end of curve2 "facing" endpoint of curve1?
                    bool sees2[4] = {
                        c2.StartTan().Dot(c1.segments.last().pt2 - c2.segments.first().pt1) > 0.0f,
                        c2.EndTan().Dot(c1.segments.last().pt2 - c2.segments.last().pt2) > 0.0f,
                        c2.StartTan().Dot(c1.segments.first().pt1 - c2.segments.first().pt1) > 0.0f,
                        c2.EndTan().Dot(c1.segments.first().pt1 - c2.segments.last().pt2) > 0.0f
                    };

                    if (dist[0] < bestdist && sees1[0] && sees2[0]) {
                        bestdist = dist[0];
                        flip1 = false;
                        flip2 = false;
                        c1i = i;
                        c2i = j;
                    }
                    if (dist[1] < bestdist && sees1[1] && sees2[1] && c1i != c2i) {
                        bestdist = dist[1];
                        flip1 = false;
                        flip2 = true;
                        c1i = i;
                        c2i = j;
                    }
                    if (dist[2] < bestdist && sees1[2] && sees2[2] && c1i != c2i) {
                        bestdist = dist[2];
                        flip1 = true;
                        flip2 = false;
                        c1i = i;
                        c2i = j;
                    }
                    if (dist[3] < bestdist && sees1[3] && sees2[3]) {
                        bestdist = dist[3];
                        flip1 = true;
                        flip2 = true;
                        c1i = i;
                        c2i = j;
                    }

                }
            }

            //if not found, exit
            if (c1i == -1 || c2i == -1)
                break;

            //perform flipping
            if (flip2)
                planarCut.chains[c2i].Reverse();

            //bind these chains together
            if (c1i == c2i) { //chains are one and same, interp and close
                DoCubicHermiteInterpForChains(planarCut.chains[c1i], planarCut.chains[c2i]);
                planarCut.chains[c1i].isClosed = true;
            }
            else { //chains are separate, join but leave open for now
                DoCubicHermiteInterpForChains(planarCut.chains[c1i], planarCut.chains[c2i]);
                planarCut.chains[c1i].segments += planarCut.chains[c2i].segments;
                planarCut.chains.removeAt(c2i);
            }


        }

        //remove all un-closed chains
        for (int i=0; i<planarCut.chains.size(); ++i) {
            if (!planarCut.chains[i].isClosed) {
                planarCut.chains.removeAt(i);
                --i;
            }
        }

        if (planarCut.chains.empty())
            return;    

    }
    */

    /*
    if (m_bUseFaceSeg) {

        while ( GetNumChainsOpen(planarCut) > 0 ) {

            int openChain1=-1;

            for (int i=0; i<planarCut.chains.size(); ++i) {

                if (!planarCut.chains[i].isClosed) {
                    openChain1=i;
                    break;
                }

            }

            MeshChain & chain1 = planarCut.chains[openChain1];

            int openChain2=-1;
            float bestVal = 0.5f;
            bool reversalNeeded=false;

            for (int i=0; i<planarCut.chains.size(); ++i) {

                MeshChain & eachchain = planarCut.chains[i];

                if (eachchain.isClosed)
                    continue;

                if (i==openChain1)
                    continue;

                //find nearest endpoint p1 (of chain i) to p2 (of chain 0)
                float p2p1len=(chain1.segments.last().pt2 - eachchain.segments.first().pt1).Length();
                float p2p2len=(chain1.segments.last().pt2 - eachchain.segments.last().pt2).Length();

                if ( min(p2p1len,p2p2len) < bestVal) {
                    bestVal=min(p2p1len, p2p2len);
                    openChain2=i;
                    reversalNeeded = (p2p2len < p2p1len);
                }

            }           

            if (openChain2>=0) {

                MeshChain & chain2 = planarCut.chains[openChain2];
                //qDebug("Linking chain %i to %i, best val %f\n",openChain1, openChain2, bestVal);

                //reverse chain 2
                if (reversalNeeded) {

                    chain2.Reverse();

                }

                //append chain 2 to chain 1, and add intermediary link
                if (m_bCubicHermiteSegmentInterp) {

                    DoCubicHermiteInterpForChains(chain1, chain2);

                }
                else {

                    MeshSegment loopSegment;
                    loopSegment.pt1=chain1.segments.last().pt2;
                    loopSegment.pt2=chain2.segments.first().pt1;

                    chain1.segments.push_back(loopSegment);

                }

                for (int i=0; i<chain2.segments.size(); ++i)
                    chain1.segments.push_back(chain2.segments[i]);

                chain2.segments.clear();

                //remove chain 2
                planarCut.chains.erase(planarCut.chains.begin()+openChain2);
            }
            else {

                //close chain 1
                if (m_bCubicHermiteSegmentInterp) {

                    DoCubicHermiteInterpForChains(chain1, chain1);

                }
                else {

                    MeshSegment loopSegment;
                    loopSegment.pt1=chain1.segments.last().pt2;
                    loopSegment.pt2=chain1.segments.first().pt1;

                    chain1.segments.push_back(loopSegment);

                }

                chain1.isClosed=true;

            }

        }     

    }   
    */

    // Determine which chains are actually holes (given by inward facing normals on surface)    
    //JAMES EDIT

    if (m_bCheckForHoles) {

        for (int i=0; i<planarCut.chains.size(); ++i) {

            MeshChain & eachchain = planarCut.chains[i];

            if (!eachchain.isClosed) { //only care about closed chains
                eachchain.isHole=false;
                continue;
            }

            float normAway=0.0f;
            Vector3 eachCent=Vector3(0,0,0);

            for (int j=0; j<eachchain.segments.size(); ++j) {
                eachCent+=eachchain.segments[j].pt1;
            }

            eachCent = eachCent * ( 1.0f/float(eachchain.segments.size()) );

            for (int j=0; j<eachchain.segments.size(); ++j) {

                MeshSegment & seg = eachchain.segments[j];

                float segmentLen=(seg.pt2 - seg.pt1).Length();

                Vector3 midPt=(seg.pt2 + seg.pt1)*0.5f;

                if (seg.n.Dot(eachCent-midPt)<=0.0f)
                    normAway+=segmentLen;
                else
                    normAway-=segmentLen;
            }


            eachchain.isHole=(normAway<0.0f);

            //qDebug() << "Chain" << i << "hole" << planarCut.chains[i].isHole << "normaway" << normAway << "segsize" << planarCut.chains[i].segments.size();

        }

    }


    // Fill in "area" property of planarcut
    for (int j=0; j<planarCut.chains.size(); ++j) {

        if (!planarCut.chains[j].isClosed)
            continue;

        //note: area calculated without having to triangulate the chain
        if (planarCut.chains[j].isHole)
            planarCut.area-=fabs(GetPolygonArea(planarCut.params.rotAxis1,planarCut.params.rotAxis2,planarCut.chains[j].segments));
        else
            planarCut.area+=fabs(GetPolygonArea(planarCut.params.rotAxis1,planarCut.params.rotAxis2,planarCut.chains[j].segments));

    }

}

void PlaneMesh::GetChainsAs2DPoints(const PlanarCut & planarCut, QList <QList <Vector2> > & points, QVector <Vector2> & loBounds, QVector <Vector2> & hiBounds, Vector2 & loBoundAll, Vector2 & hiBoundAll) {

    // Compute bounding boxes
    points.clear();
    loBounds = QVector <Vector2> (planarCut.chains.size(), Vector2(FLT_MAX, FLT_MAX));
    hiBounds = QVector <Vector2> (planarCut.chains.size(), Vector2(-FLT_MAX, -FLT_MAX));
    loBoundAll = Vector2(FLT_MAX, FLT_MAX);
    hiBoundAll = Vector2(-FLT_MAX, -FLT_MAX);

    const Vector3 & norm = planarCut.params.norm;
    const Vector3 & axisDir1 = planarCut.params.rotAxis1;
    const Vector3 & axisDir2 = planarCut.params.rotAxis2;

    for (int i=0; i<planarCut.chains.size(); ++i) {

        QList <Vector2> projchain;

        for (int j=0; j<planarCut.chains[i].segments.size(); ++j) {

            const Vector3 & pt = planarCut.chains[i].segments[j].pt1;
            Vector3 ptNoNorm = pt - norm*(norm.Dot(pt));

            Vector2 eachVec2D;
            eachVec2D.x=axisDir1.Dot(ptNoNorm);
            eachVec2D.y=axisDir2.Dot(ptNoNorm);

            projchain.push_back(eachVec2D);

            loBounds[i].x = qMin(eachVec2D.x, loBounds[i].x);
            loBounds[i].y = qMin(eachVec2D.y, loBounds[i].y);
            hiBounds[i].x = qMax(eachVec2D.x, hiBounds[i].x);
            hiBounds[i].y = qMax(eachVec2D.y, hiBounds[i].y);

        }

        points.push_back(projchain);

        loBoundAll.x = qMin(loBoundAll.x, loBounds[i].x);
        loBoundAll.y = qMin(loBoundAll.y, loBounds[i].y);
        hiBoundAll.x = qMax(hiBoundAll.x, hiBounds[i].x);
        hiBoundAll.y = qMax(hiBoundAll.y, hiBounds[i].y);

    }

}

void PlaneMesh::GetTriangles(PlanarCut & planarCut) {

    planarCut.tris.clear();
    if (planarCut.chains.empty())
        return;

    Vector3 norm = planarCut.params.norm;
    float d = norm.Dot(planarCut.params.cent);
    Vector3 axisDir1=planarCut.params.rotAxis1;
    Vector3 axisDir2=planarCut.params.rotAxis2;

    norm.Normalize();
    axisDir1.Normalize();
    axisDir2.Normalize();

    // Get projected points and bounding boxes
    QVector <Vector2> lowerBounds;
    QVector <Vector2> upperBounds;
    Vector2 loBoundsAll, hiBoundsAll;
    QList <QList <Vector2> > projchains;

    GetChainsAs2DPoints(planarCut, projchains, lowerBounds, upperBounds, loBoundsAll, hiBoundsAll);

    //filter projected positions
    for (int i=0; i<projchains.size(); ++i) {

        for (int j=0; j<projchains[i].size(); ++j) {

            if ((j > 0 && !((projchains[i][j]-projchains[i][j-1]).Length() > 0.0001f)) ||
                    (j < projchains[i].size()-1 && !((projchains[i][j]-projchains[i][j+1]).Length() > 0.0001f))) {

                projchains[i].removeAt(j);
                //qDebug() << "Filtering out" << i << "chain" << j << "/" << projchains[i].size() << "at triangulation step!";

                --j;

            }

        }

        //remove chain (clear it) if now too short, but don't remove it from list
        if (projchains[i].size() < 4) {

            projchains[i].clear();

        }

    }

    //
    // Do 2D PSLG triangulation on curves formed by intersecting plane with mesh
    //
    triangulateio in, out;

    in.numberofpoints = 0;
    in.numberofsegments = 0;
    in.numberofholes = 0;
    int cIndex=0;

    for (int c=0; c<projchains.size(); ++c) {

        in.numberofsegments += projchains[c].size();

        if (planarCut.chains[c].isHole)
            ++in.numberofholes;
    }

    // Define input points.
    in.numberofpoints = in.numberofsegments;
    in.numberofpointattributes = 0;
    in.pointlist = (REAL *) malloc(in.numberofpoints * 2 * sizeof(REAL));
    in.pointattributelist = NULL;
    in.pointmarkerlist = NULL;

    for (int c=0; c<projchains.size(); ++c) {       

        for (int i=0; i<projchains[c].size(); ++i) {

            in.pointlist[cIndex*2]=projchains[c][i].x;
            in.pointlist[cIndex*2+1]=projchains[c][i].y;

            //qDebug() << "Chain" << c << "projected point" << i << "is" << projchains[c][i].x << projchains[c][i].y;

            cIndex++;

        }
    }


    //define PSLG line segments
    in.segmentlist=(int *)malloc(in.numberofsegments*2*sizeof(int));
    in.segmentmarkerlist=NULL;

    unsigned int origIndex;

    cIndex=0;

    for (int c=0; c<projchains.size(); ++c) {      

        origIndex=cIndex;       

        for (int i=0; i<projchains[c].size(); ++i) {

            in.segmentlist[cIndex*2]=cIndex;

            if ( i+1 < projchains[c].size())
                in.segmentlist[cIndex*2+1]=cIndex+1;
            else
                in.segmentlist[cIndex*2+1]=origIndex;

            ++cIndex;
        }
    }

    //define midpoints of those contours with holes, as being holes
    in.holelist = (REAL *) malloc(in.numberofholes * 2 * sizeof(REAL));

    cIndex=0;
    for (int c=0; c<planarCut.chains.size(); ++c) {

        MeshChain & chain = planarCut.chains[c];

        if (chain.isHole) {

            in.holelist[cIndex*2]=(lowerBounds[c].x+upperBounds[c].x)/2.0f;
            in.holelist[cIndex*2+1]=(lowerBounds[c].y+upperBounds[c].y)/2.0f;
            ++cIndex;

        }
    }

    in.numberofregions = 0;

    // Make necessary initializations so that Triangle can return a
    //   triangulation in `mid' and a voronoi diagram in `vorout'.

    out.pointlist = (REAL *) NULL;            // Not needed if -N switch used.
    // Not needed if -N switch used or number of point attributes is zero:
    out.pointattributelist = (REAL *) NULL;
    out.pointmarkerlist = (int *) NULL; // Not needed if -N or -B switch used.
    out.trianglelist = (int *) NULL;          // Not needed if -E switch used.
    //Not needed if -E switch used or number of triangle attributes is zero:
    out.triangleattributelist = (REAL *) NULL;
    out.neighborlist = (int *) NULL;         // Needed only if -n switch used.
    //Needed only if segments are output (-p or -c) and -P not used:
    out.segmentlist = (int *) NULL;
    // Needed only if segments are output (-p or -c) and -P and -B not used:
    out.segmentmarkerlist = (int *) NULL;
    out.edgelist = (int *) NULL;             // Needed only if -e switch used.
    out.edgemarkerlist = (int *) NULL;   // Needed if -e used and -B not used.

    if (in.numberofsegments>=3)
        triangulate((char *)"pzQY", &in, &out, NULL); // not delaunay, fewer tris
    else
        out.numberoftriangles=0;

    //
    // Map every 2D planar point back into 3-space
    //

    Vector3 vec2D, vec3D;

    for (int i=0; i<out.numberoftriangles; ++i) {

        Triangle eachTri;

        for (int j=0; j<3; ++j) {

            float x2d = out.pointlist[out.trianglelist[i*3+j]*2];
            float y2d = out.pointlist[out.trianglelist[i*3+j]*2+1];

            Vector3 vec3D = axisDir1*x2d + axisDir2*y2d + norm*d;

            switch (j) {

                case 0:
                    eachTri.p1=vec3D;
                    break;
                case 1:
                    eachTri.p2=vec3D;
                    break;
                default:
                    eachTri.p3=vec3D;
                    break;

            }
        }

        planarCut.tris.push_back(eachTri);

    }

    free(in.pointlist);
    free(in.segmentlist);
    free(in.holelist);

    if (out.numberoftriangles>0) {
        free(out.pointlist);
        free(out.trianglelist);
    }

}

void PlaneMesh::DrawInputVerts() {

    if (inputMesh==NULL)
        return;

    QVector <Vector3> &verts=inputMesh->getVerts();
    unsigned int nVerts=verts.size();

    glDisable(GL_LIGHTING);

    glPointSize(4.0f);
    glColor3f(0,0,0);

    glBegin(GL_POINTS);
    for (unsigned int i=0;i<nVerts;++i) {
        verts[i].Draw();        
    }
    glEnd();

    glColor3f(1,1,1);

    glPointSize(1.0f);


}

void PlaneMesh::DrawInputWireframe() {

    if (inputMesh==NULL)
        return;

    glDisable(GL_LIGHTING);

    QVector <QVector <int> > &inputFaces=inputMesh->getFaces();
    int numFaces=inputFaces.size();
    QVector <Vector3> &inputVerts=inputMesh->getVerts();

    glColor3f(0,0,0);
    glBegin(GL_LINES);
    for (int i=0;i<numFaces;++i) {

        for (int j=0;j<inputFaces[i].size();++j) {

            inputVerts[inputFaces[i][j]].Draw();
            inputVerts[inputFaces[i][(j+1)%inputFaces[i].size()]].Draw();                        

        }

    }
    glEnd();

}

void PlaneMesh::DrawInputMesh(double meshAlpha) {

    if (inputMesh==NULL)
        return;   

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);    

    const float ambient[4]={m_meshLightColor.redF(),
                            m_meshLightColor.greenF(),
                            m_meshLightColor.blueF(),
                            1.0f};
    const float diffuse[4]={.4f,.4f,.4f,1};
    const float specular[4]={0,0,0,1};

    const float position1[] = { camera->GetEye().x, camera->GetEye().y, camera->GetEye().z, 1.0f };

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
    glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f);
    glLightfv(GL_LIGHT0, GL_POSITION, position1);

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
    
    glColor3f(1,1,1);

    if (m_bUseFaceSeg && !m_vMeshFaceSeg.empty()) {

        QVector <QVector <int> > &faceList=inputMesh->getFaces();        
        QVector <Vector3> &verts=inputMesh->getVerts();
        QVector <Vector3> &faceNorms=inputMesh->getFaceNorms();

        const int numFaces=faceList.size();

        const float segambient[4]={.2f,.2f,.2f,1};
        glMaterialfv(GL_FRONT, GL_AMBIENT, segambient);

        glEnable(GL_COLOR_MATERIAL);
        glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

        glBegin(GL_TRIANGLES);
        for (int i=0; i<numFaces; ++i) {

            glNormal3f(faceNorms[i].x,faceNorms[i].y,faceNorms[i].z);

            if (selectedPlaneIndex<m_vPlanarCuts.size()&&m_vMeshFaceSeg[i]==m_vPlanarCuts[selectedPlaneIndex].nearSeg)
                glColor3f(0.8f,0.8f,0.8f);
            else {
                Vector3 col = SetColorByIndex(m_vMeshFaceSeg[i]) * 0.7f;
                glColor3f(col.x,col.y,col.z);
            }

            for (int j=0; j<3; ++j) {
                glVertex3f(verts[faceList[i][j]].x,
                           verts[faceList[i][j]].y,
                           verts[faceList[i][j]].z);
            }
        }
        glEnd();

        glDisable(GL_COLOR_MATERIAL);

    }
    else {

        glColor4f(1, 1, 1, float(meshAlpha));

        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

        inputMesh->DrawSimple();

        glDisable(GL_BLEND);


    }

    //inputMesh->DrawSimple();

    glDisable(GL_LIGHTING);

}

void PlaneMesh::DrawMeshFeatures() {

    glDisable(GL_LIGHTING);

    // DRAW PCA PLANES STUFF
    if (m_bUseFeature[P_PCA]) {

        for (int i=0;i<pcaPlaneFits.size();++i)
            DrawFeaturePlane(pcaPlaneFits[i], Vector3(0.5f,0.5f,0.5f));

    }

    // DRAW PCA LINE STUFF
    if (m_bUseFeature[A_PCA]) {

        for (int i=0; i<pcaLineFits.size(); ++i)
            DrawAxis(pcaLineFits[i],Vector3(0.5f,0.5f,0.5f));

    }


    //Draw ridges and valley lines on mesh    
    if (m_bUseFeature[A_RIDGE]) {

        glLineWidth(5.0f);

        for (int i=0;i<ridgeAxesChains.size();++i) {

            glColor3f(0.5f,0.5f,0.5f);

            glBegin(GL_LINE_STRIP);
            for (int j=0;j<ridgeAxesChains[i].size();++j) {
                glVertex3f(ridgeAxesChains[i][j].pt1.x,ridgeAxesChains[i][j].pt1.y,ridgeAxesChains[i][j].pt1.z);
                glVertex3f(ridgeAxesChains[i][j].pt2.x,ridgeAxesChains[i][j].pt2.y,ridgeAxesChains[i][j].pt2.z);
            }
            glEnd();           

        }

        glLineWidth(1.0f);

        for (int i=0; i<ridgeAxes.size(); ++i)
            DrawAxis(ridgeAxes[i], Vector3(1,0.5f,0));

    }


    if (m_bUseFeature[A_VALLEY]) {

        glLineStipple(1,0x0f0f);
        glEnable(GL_LINE_STIPPLE);

        glLineWidth(5.0f);

        for (int i=0;i<valleyAxesChains.size();++i) {

            glColor3f(0.5f,0.5f,0.5f);

            glBegin(GL_LINE_STRIP);
            for (int j=0;j<valleyAxesChains[i].size();++j) {
                glVertex3f(valleyAxesChains[i][j].pt1.x,valleyAxesChains[i][j].pt1.y,valleyAxesChains[i][j].pt1.z);
                glVertex3f(valleyAxesChains[i][j].pt2.x,valleyAxesChains[i][j].pt2.y,valleyAxesChains[i][j].pt2.z);
            }
            glEnd();

        }

        glLineWidth(1.0f);
        glDisable(GL_LINE_STIPPLE);

        for (int i=0; i<valleyAxes.size(); ++i)
            DrawAxis(valleyAxes[i],Vector3(1,0.5f,0));

    }   

    //Draw symmetry planes
    if (m_bUseFeature[P_SYM]) {

        for (int i=0; i<symPlanes.size(); ++i)
            DrawPlane(symPlanes[i], Vector3(0.5f,0.5f,0.5f));

    }

    //Draw symmetry axes
    if (m_bUseFeature[A_SYM]) {

        for (int i=0; i<symLines.size(); ++i)
            DrawAxis(symLines[i], Vector3(0.3f,0.3f,0.3f));

    }

    glLineWidth(1.0f);
    glColor3f(1,1,1);

    //Draw global xyz planes
    if (m_bUseFeature[P_GLOBALXYZ]) {

        for (int i=0; i<globalxyzPlanes.size(); ++i)
            DrawPlane(globalxyzPlanes[i], Vector3(0.5f,0.5f,0.5f));

    }

    //Draw global pca planes
    if (m_bUseFeature[P_GLOBALPCA]) {

        for (int i=0; i<globalpcaPlanes.size(); ++i)
            DrawPlane(globalpcaPlanes[i], Vector3(0.5f,0.5f,0.5f));

    }

}

void PlaneMesh::DrawAxis(const Axis &axis, const Vector3 col) {

    glLineWidth(5.0f);
    glColor3f(col.x,col.y,col.z);

    glBegin(GL_LINES);
    glVertex3f(axis.p.x - axis.dir.x * axis.len,
               axis.p.y - axis.dir.y * axis.len,
               axis.p.z - axis.dir.z * axis.len);
    glVertex3f(axis.p.x + axis.dir.x * axis.len,
               axis.p.y + axis.dir.y * axis.len,
               axis.p.z + axis.dir.z * axis.len);
    glEnd();

    glLineWidth(7.0f);
    glColor3f(0,0,0);

    glBegin(GL_LINES);
    glVertex3f(axis.p.x - axis.dir.x * axis.len,
               axis.p.y - axis.dir.y * axis.len,
               axis.p.z - axis.dir.z * axis.len);
    glVertex3f(axis.p.x + axis.dir.x * axis.len,
               axis.p.y + axis.dir.y * axis.len,
               axis.p.z + axis.dir.z * axis.len);
    glEnd();

    glLineWidth(1.0f);
    glColor3f(1,1,1);

}

void PlaneMesh::DrawFeaturePlane(const Plane & cursorPlane, const Vector3 col)
{

    glPushMatrix();

    glTranslatef(cursorPlane.featcentroid.x,
                 cursorPlane.featcentroid.y,
                 cursorPlane.featcentroid.z);

    glScaled(cursorPlane.extent, cursorPlane.extent, cursorPlane.extent);

    Vector3 r1=cursorPlane.rotAxis1;
    Vector3 r2=cursorPlane.rotAxis2;

    for (int i=0; i<2; ++i) {

        if (i == 0) {
            glLineWidth(5.0f);
            glColor3f(col.x,col.y,col.z);
        }
        else {
            glLineWidth(7.0f);
            glColor3f(0,0,0);
        }

        glBegin(GL_LINES);
        glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
        glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
        glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
        glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
        glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
        glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
        glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
        glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
        glEnd();

    }

    glPopMatrix();

    glLineWidth(1.0f);
    glColor3f(1,1,1);

}

void PlaneMesh::DrawPlane(const Plane & cursorPlane, const Vector3 col) {

    glPushMatrix();

    glTranslatef(cursorPlane.cent.x,
                 cursorPlane.cent.y,
                 cursorPlane.cent.z);

    glScaled(inputMesh->GetBoundRad(),inputMesh->GetBoundRad(),inputMesh->GetBoundRad());

    Vector3 r1=cursorPlane.rotAxis1;
    Vector3 r2=cursorPlane.rotAxis2;

    for (int i=0; i<2; ++i) {

        if (i == 0) {
            glLineWidth(5.0f);
            glColor3f(col.x,col.y,col.z);
        }
        else {
            glLineWidth(7.0f);
            glColor3f(0,0,0);
        }

        glBegin(GL_LINES);
        glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
        glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
        glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
        glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
        glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
        glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
        glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
        glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
        glEnd();

    }

    glPopMatrix();

    glLineWidth(1.0f);
    glColor3f(1,1,1);

}

void PlaneMesh::DrawPlaneCoordFrame(const Plane & cursorPlane) {

    glDisable(GL_LIGHTING);

    glPushMatrix();

    glTranslatef(cursorPlane.cent.x,
                 cursorPlane.cent.y,
                 cursorPlane.cent.z);

    glScaled(inputMesh->GetBoundRad(),inputMesh->GetBoundRad(),inputMesh->GetBoundRad());

    Vector3 r1=cursorPlane.rotAxis1;
    Vector3 r2=cursorPlane.rotAxis2;

    //draw black outlines    
    glLineWidth(5.0f);

    int rot1col=254+(254<<8)+(254<<16);
    glColor3ub((unsigned char)rot1col,(unsigned char)(rot1col>>8),(unsigned char)(rot1col>>16));
    glBegin(GL_LINES);
    glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
    glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
    glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
    glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
    glEnd();

    int rot2col=253+(253<<8)+(253<<16);
    glColor3ub((unsigned char)rot2col,(unsigned char)(rot2col>>8),(unsigned char)(rot2col>>16));
    glBegin(GL_LINES);
    glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
    glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
    glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
    glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
    glEnd();

    int trans1col=255+(0<<8)+(0<<16);
    glColor3ub((unsigned char)trans1col,(unsigned char)(trans1col>>8),(unsigned char)(trans1col>>16));
    glBegin(GL_LINES);
    glVertex3f(0,0,0);
    glVertex3f(r1.x,r1.y,r1.z);
    glEnd();

    int trans2col=0+(255<<8)+(0<<16);
    glColor3ub((unsigned char)trans2col,(unsigned char)(trans2col>>8),(unsigned char)(trans2col>>16));
    glBegin(GL_LINES);
    glVertex3f(0,0,0);
    glVertex3f(r2.x,r2.y,r2.z);
    glEnd();

    glLineWidth(7.0f);

    glColor3f(0,0,0);
    glBegin(GL_LINES);
    glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
    glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
    glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
    glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
    glVertex3f(r1.x+r2.x,r1.y+r2.y,r1.z+r2.z);
    glVertex3f(r1.x-r2.x,r1.y-r2.y,r1.z-r2.z);
    glVertex3f(-r1.x-r2.x,-r1.y-r2.y,-r1.z-r2.z);
    glVertex3f(-r1.x+r2.x,-r1.y+r2.y,-r1.z+r2.z);
    glEnd();

    glPopMatrix();

    if (planeManip==NONEBALL||planeManip==MOVEBALL1||planeManip==MOVEBALL2||planeManip==MOVEBALL3) {

        const int ball1col=255+(20<<8)+(20<<16);
        const int ball2col=254+(20<<8)+(20<<16);
        const int ball3col=253+(20<<8)+(20<<16);

        float sphereRad=inputMesh->GetBoundRad()/40.0f;

        glColor3ub((unsigned char)ball1col,(unsigned char)(ball1col>>8),(unsigned char)(ball1col>>16));
        if (planeManip==MOVEBALL1)
            glColor3ub(255,120,120);
        glPushMatrix();
        glTranslated(cursorPlane.posB1.x,cursorPlane.posB1.y,cursorPlane.posB1.z);
        gluSphere(gluNewQuadric(),sphereRad,20,20);
        glPopMatrix();

        glColor3ub((unsigned char)ball2col,(unsigned char)(ball2col>>8),(unsigned char)(ball2col>>16));
        if (planeManip==MOVEBALL2)
            glColor3ub(255,120,120);
        glPushMatrix();
        glTranslated(cursorPlane.posB2.x,cursorPlane.posB2.y,cursorPlane.posB2.z);
        gluSphere(gluNewQuadric(),sphereRad,20,20);
        glPopMatrix();

        glColor3ub((unsigned char)ball3col,(unsigned char)(ball3col>>8),(unsigned char)(ball3col>>16));
        if (planeManip==MOVEBALL3)
            glColor3ub(255,120,120);
        glPushMatrix();
        glTranslated(cursorPlane.posB3.x,cursorPlane.posB3.y,cursorPlane.posB3.z);
        gluSphere(gluNewQuadric(),sphereRad,20,20);
        glPopMatrix();

    }

    glLineWidth(1.0f);

}

void PlaneMesh::DrawHumanCuts() {


    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glDepthFunc(GL_LEQUAL);    
    glDepthMask(GL_FALSE);        

    glDisable(GL_LIGHTING);

    glColor4f(0,0,0,1.0f/(float)NUM_HUMAN_DATASETS); //draw black

    //glColor4f(0,0,0,1);

    glPointSize(3.0f);
    glLineWidth(5.0f);
    for (int i=0; i<figureCuts.size(); ++i) {
        //draw cuts coloured by user      
        for (int j=0; j<figureCuts[i].size(); ++j)
            figureCuts[i][j].DrawChains();

    }      
    glLineWidth(1.0f);


    // for RAINBOW figure (i believe)
    /*
    if (figureCuts.size()>=NUM_HUMAN_DATASETS) {

        //glEnable(GL_BLEND);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

        //glEnable(GL_LINE_STIPPLE);

        glLineWidth(3.0f);

        for (int i=0;i<4;++i) {

            QList <PlanarCut> * cutlist;

            if (i==0) {
                cutlist=&m_vPlanarCuts;
                SetColorByIndex(0);
                glLineStipple(2, 0xf000);
            }
            //else if (i==1) {
            //    cutlist=&figureCuts[7]; //user 8 22 unique planes
            //    SetColorByIndex(1, 0.5f);
            //}
            else if (i==1) {
                cutlist=&figureCuts[8]; //user 9 26 unique planes
                SetColorByIndex(2);
                glLineStipple(2, 0x0f00);
            }
            else if (i==2) {
                cutlist=&figureCuts[0]; //user 1 29 unique planes
                SetColorByIndex(3);
                glLineStipple(2, 0x00f0);
            }
            else if (i==3) {
                cutlist=&figureCuts[2]; //user 3 has 47 (*user 3 had 55) user 6 had 55
                SetColorByIndex(7);
                glLineStipple(2, 0x000f);
            }

            for (int j=0;j<cutlist->size();++j)
                (*cutlist)[j].DrawChains(0.002f * (i+1));

        }

        glLineWidth(1.0f);
        //glDisable(GL_BLEND);
        //glDisable(GL_LINE_STIPPLE);

    }
    */


    glDepthFunc(GL_LESS);
    glDepthMask(GL_TRUE);

    glDisable(GL_BLEND);    

    glColor3f(1,1,1);

}

void PlaneMesh::DrawOutputMesh(bool drawInteriorTriangulation, bool drawContoursInColour) {

    glDisable(GL_LIGHTING);
    glDisable(GL_CULL_FACE);

    if (drawInteriorTriangulation) {
        RenderPlanePieces(m_vPlanarCuts, 2);
    }

    for (int i=0;i<m_vPlanarCuts.size();++i) {       

        glLineWidth(5.0f);
        glPointSize(5.0f);

        for (int k=0;k<m_vPlanarCuts[i].chains.size();k++) {

            if (drawContoursInColour) {
                SetColorByIndex(i);
            }
            else
                glColor3f(0,0,0);

            //draw the chain
            m_vPlanarCuts[i].chains[k].Draw();

            //draw normals associated to each chain segment
            /*
                glBegin(GL_LINES);
                for (int j=0;j<m_vPlanarCuts[i].chains[k].segments.size();++j) {

                    glVertex3d(m_vPlanarCuts[i].chains[k].segments[j].pt1.x,
                               m_vPlanarCuts[i].chains[k].segments[j].pt1.y,
                               m_vPlanarCuts[i].chains[k].segments[j].pt1.z);
                    glVertex3d(m_vPlanarCuts[i].chains[k].segments[j].pt1.x+m_vPlanarCuts[i].chains[k].segments[j].n.x*boundingSphereRad/30.0,
                               m_vPlanarCuts[i].chains[k].segments[j].pt1.y+m_vPlanarCuts[i].chains[k].segments[j].n.y*boundingSphereRad/30.0,
                               m_vPlanarCuts[i].chains[k].segments[j].pt1.z+m_vPlanarCuts[i].chains[k].segments[j].n.z*boundingSphereRad/30.0);
                }
                glEnd();
                */

        }

        glLineWidth(1.0f);
        glPointSize(1.0f);

        if (i == (int)selectedPlaneIndex)
            DrawPlaneCoordFrame(m_vPlanarCuts[i].params);

    }

    glDisable(GL_LIGHTING);


    if (drawInteriorTriangulation) {
        for (int i=0;i<piecewiseFigureCuts.size();++i) {

            glColor3f(0,0,0);
            glLineWidth(3.0f);
            glBegin(GL_LINES);

            for (int j=0; j<piecewiseFigureCuts[i].segments.size(); ++j) {

                MeshSegment & eachseg = piecewiseFigureCuts[i].segments[j];

                glVertex3f(eachseg.pt1.x, eachseg.pt1.y, eachseg.pt1.z);
                glVertex3f(eachseg.pt2.x, eachseg.pt2.y, eachseg.pt2.z);

            }

            glEnd();
            glLineWidth(1.0f);


            SetColorByIndex(i);
            glPushMatrix();

            Plane & params = piecewiseFigureCuts[i].params;

            glTranslatef(-params.norm.x * params.pt * 2.0f,
                         -params.norm.y * params.pt * 2.0f,
                         -params.norm.z * params.pt * 2.0f);

            glBegin(GL_TRIANGLES);

            for (int j=0; j<piecewiseFigureCuts[i].tris.size(); ++j) {

                Triangle & tri = piecewiseFigureCuts[i].tris[j];

                glVertex3f(tri.p1.x, tri.p1.y, tri.p1.z);
                glVertex3f(tri.p2.x, tri.p2.y, tri.p2.z);
                glVertex3f(tri.p3.x, tri.p3.y, tri.p3.z);

            }

            glEnd();

            glPopMatrix();


        }

    }

    glEnable(GL_LIGHTING);

}

Vector3 PlaneMesh::SetColorByIndex(const unsigned int index) {

    if (distinctColours.empty()) {      

        for (int i=1; i<=3; ++i) {
            for (float hue=0.0;hue<360.0f;hue+=30.0f) {

                float v=1.0f;
                float shv=1.0f/float(i);
                float c=v*shv;

                float hprime=hue/(60.0f);
                float x=c*(1.0f-fabs(fmod(hprime,2.0f)-1.0f));

                Vector3 eachcol;
                if (hprime<1)
                    eachcol=Vector3(c,x,0);
                else if (hprime<2)
                    eachcol=Vector3(x,c,0);
                else if (hprime<3)
                    eachcol=Vector3(0,c,x);
                else if (hprime<4)
                    eachcol=Vector3(0,x,c);
                else if (hprime<5)
                    eachcol=Vector3(x,0,c);
                else if (hprime<6)
                    eachcol=Vector3(c,0,x);

                float m=v-c;
                eachcol.x+=m;
                eachcol.y+=m;
                eachcol.z+=m;

                distinctColours.push_back(eachcol);

            }
        }

    }

    int i = index % distinctColours.size();    

    Vector3 col = distinctColours[i];
    glColor3f(col.x, col.y, col.z);
    return col;

}

void PlaneMesh::RenderPlanePieces(const QList <PlanarCut> &planarCuts, int renderMode) {

    switch (renderMode) {

        case 0:

        for (int i=0; i<planarCuts.size(); ++i) {

            if (selectedPlaneIndex==i)
                glColor3f(.5f,1,.5f);
            else
                glColor3f(0,0,0);

            planarCuts[i].DrawTrisWireframe();            

        }

        break;

        case 1:

        for (int i=0; i<planarCuts.size(); ++i) {

            planarCuts[i].DrawTrisFilled();

        }

        break;

        case 2:

        //glEnable(GL_LIGHTING);

        for (int i=0; i<planarCuts.size(); ++i) {

            Vector3 indexcol = SetColorByIndex(i);
            Vector3 campos = camera->GetEye();
            campos.Normalize();

            //float colfactor = 0.5f + 0.5f * fabs(campos.Dot(planarCuts[i].params.norm));
            float colfactor = 1.0f;
            glColor3f(indexcol.x * colfactor, indexcol.y * colfactor, indexcol.z * colfactor);

            glBegin(GL_TRIANGLES);
            for (int j=0; j<planarCuts[i].tris.size(); ++j) {

                const Triangle & tri = planarCuts[i].tris[j];

                glNormal3d(planarCuts[i].params.norm.x,
                           planarCuts[i].params.norm.y,
                           planarCuts[i].params.norm.z);

                glVertex3f(tri.p1.x, tri.p1.y, tri.p1.z);
                glVertex3f(tri.p2.x, tri.p2.y, tri.p2.z);
                glVertex3f(tri.p3.x, tri.p3.y, tri.p3.z);

            }
            glEnd();

        }

        //glDisable(GL_LIGHTING);

        break;

        case 3:

        glBegin(GL_TRIANGLES);
        for (int i=0; i<planarCuts.size(); ++i) {

            glColor3ub((unsigned char)i,(unsigned char)(i>>8),(unsigned char)(i>>16));

            for (int j=0; j<planarCuts[i].tris.size(); ++j) {

                const Triangle & tri = planarCuts[i].tris[j];

                glVertex3f(tri.p1.x, tri.p1.y, tri.p1.z);
                glVertex3f(tri.p2.x, tri.p2.y, tri.p2.z);
                glVertex3f(tri.p3.x, tri.p3.y, tri.p3.z);

            }

        }
        glEnd();

        break;

    }

}

unsigned int PlaneMesh::GetNumPlanes() {

    return m_vPlanarCuts.size();

}

void PlaneMesh::ClearPlanes() {

    m_vPlanarCuts.clear();

    selectedPlaneIndex=INT_MAX;

}

bool PlaneMesh::ExportOutputMeshAsGeom_OBJ(char *filename, float ext) {

    QString strfilename(filename);
    QFile file(strfilename);

    if (!file.open( QIODevice::WriteOnly | QIODevice::Text )) {
        qDebug() << "File " << strfilename << " can't be opened for write.";
        return false;
    }

    QTextStream ofs( &file );
    int verts = 0;

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        ofs << "g groupbadcut" << i << "\n";

        for (int k=0; k<m_vPlanarCuts[i].chains.size(); ++k) {
            for (int l=0; l<m_vPlanarCuts[i].chains[k].segments.size()-1; ++l) {

                MeshSegment & s1 = m_vPlanarCuts[i].chains[k].segments[l];
                MeshSegment & s2 = m_vPlanarCuts[i].chains[k].segments[l+1];

                WriteFeatureGeom(ofs, s1.pt1, s2.pt1, verts, ext);
            }
        }

    }

    file.close();

    return true;

}

bool PlaneMesh::ExportOutputMeshAsSlabs_OBJ(char *filename, float slabThick) {

    ofstream ofs(filename, ios_base::out);

    if (ofs.fail())
        return false;

    ofs.setf(ios_base::fixed, ios_base::floatfield);

    ofs << "# " << m_vPlanarCuts.size() << " Plane Parameters:" << std::endl;
    ofs << "# nx, ny, nz, px, py, pz, d, r1x, r1y, r1z, r2x, r2y, r2z, area" << std::endl;

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        Plane & pp = m_vPlanarCuts[i].params;

        ofs << "# " << pp.norm.x << " " << pp.norm.y << " " << pp.norm.z
                << " " << pp.cent.x << " " << " " << pp.cent.y << " " << pp.cent.z
                << " " << pp.pt
                << " " << pp.rotAxis1.x << " " << pp.rotAxis1.y << " " << pp.rotAxis1.z
                << " " << pp.rotAxis2.x << " " << pp.rotAxis2.y << " " << pp.rotAxis2.z
                << " " << m_vPlanarCuts[i].area << std::endl;

    }

    ofs << "#" << std::endl;

    ofs << "# Vertex/normal list" << std::endl;   

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        QList <Triangle> & eachPieces = m_vPlanarCuts[i].tris;
        Vector3 vExtrude = m_vPlanarCuts[i].params.norm * slabThick;

        for (int j=0; j<eachPieces.size(); ++j) {

            ofs << "v " << eachPieces[j].p1.x
                    << " " << eachPieces[j].p1.y
                    << " " << eachPieces[j].p1.z
                    << std::endl;
            ofs << "v " << eachPieces[j].p2.x
                    << " " << eachPieces[j].p2.y
                    << " " << eachPieces[j].p2.z
                    << std::endl;
            ofs << "v " << eachPieces[j].p3.x
                    << " " << eachPieces[j].p3.y
                    << " " << eachPieces[j].p3.z
                    << std::endl;

            ofs << "v " << eachPieces[j].p1.x + vExtrude.x
                    << " " << eachPieces[j].p1.y + vExtrude.y
                    << " " << eachPieces[j].p1.z + vExtrude.z
                    << std::endl;
            ofs << "v " << eachPieces[j].p2.x + vExtrude.x
                    << " " << eachPieces[j].p2.y + vExtrude.y
                    << " " << eachPieces[j].p2.z + vExtrude.z
                    << std::endl;
            ofs << "v " << eachPieces[j].p3.x + vExtrude.x
                    << " " << eachPieces[j].p3.y + vExtrude.y
                    << " " << eachPieces[j].p3.z + vExtrude.z
                    << std::endl;

        }

    }

    ofs << "# Face list" << std::endl;
    
    unsigned int curBaseIndex=0;

    for (int i=0;i<m_vPlanarCuts.size();++i) {

        ofs << "g plane" << i << std::endl;

        for (int j=0;j<m_vPlanarCuts[i].tris.size();++j) {

            ofs << "f " << (curBaseIndex+j)*6 + 1 << " "
                    << (curBaseIndex+j)*6 + 2 << " "
                    << (curBaseIndex+j)*6 + 3 << " "
                    << std::endl;

            ofs << "f " << (curBaseIndex+j)*6 + 6 << " "
                    << (curBaseIndex+j)*6 + 5 << " "
                    << (curBaseIndex+j)*6 + 4 << " "
                    << std::endl;

            ofs << "f " << (curBaseIndex+j)*6 + 1 << " "
                    << (curBaseIndex+j)*6 + 3 << " "
                    << (curBaseIndex+j)*6 + 6 << " "
                    << (curBaseIndex+j)*6 + 4 << " "
                    << std::endl;

            ofs << "f " << (curBaseIndex+j)*6 + 3 << " "
                    << (curBaseIndex+j)*6 + 2 << " "
                    << (curBaseIndex+j)*6 + 5 << " "
                    << (curBaseIndex+j)*6 + 6 << " "
                    << std::endl;

            ofs << "f " << (curBaseIndex+j)*6 + 2 << " "
                    << (curBaseIndex+j)*6 + 1 << " "
                    << (curBaseIndex+j)*6 + 4 << " "
                    << (curBaseIndex+j)*6 + 5 << " "
                    << std::endl;

        }

        curBaseIndex+=m_vPlanarCuts[i].tris.size();

    }

    ofs.close();

    qDebug("Successfully wrote %s.\n",filename);

    return true;


}

bool PlaneMesh::ExportInputMesh_OBJ(char * filename) {

    FILE *outFile;
    outFile=fopen(filename,"w");

    if (outFile==NULL) {
        qDebug("Unable to write file %s.\n",filename);
        return false;
    }

    const QVector <Vector3> & verts = inputMesh->getVerts();
    const QVector <QVector <int> > & faces = inputMesh->getFaces();

    //fprintf(outFile,"# Vertex/normal list\n");
    for (int i=0;i<verts.size();++i) {

        fprintf(outFile,"v %f %f %f\n",
                verts[i].x,
                verts[i].y,
                verts[i].z);

    }

    // Then dump face info
    //fprintf(outFile,"# Face list\n");

    for (int j=0; j<m_nSegments; ++j) {

        fprintf(outFile, "g segment%i\n",j);

        for (int i=0;i<faces.size();++i) {

            if (m_vMeshFaceSeg[i] == j) {
                fprintf(outFile, "f ");

                for (int j=0;j<faces[i].size();++j)
                    fprintf(outFile, "%i ", faces[i][j] + 1);

                fprintf(outFile,"\n");
            }

        }

    }

    fclose(outFile);

    qDebug("Successfully wrote %s.\n",filename);

    return true;

}

void PlaneMesh::WriteBoxGeom(QTextStream & ofs, const Vector3 & p1, const Vector3 & p2, int & vertcount)
{

    Vector3 basis1(0,0,0);
    basis1.x = p2.x - p1.x;
    Vector3 basis2(0,0,0);
    basis2.y = p2.y - p1.y;
    Vector3 basis3(0,0,0);
    basis3.z = p2.z - p1.z;

    Vector3 verts[8];
    verts[0] = p1;
    verts[1] = p1 + basis1;
    verts[2] = p1 + basis1 + basis2;
    verts[3] = p1 + basis2;
    verts[4] = p1 + basis3;
    verts[5] = p1 + basis1 + basis3;
    verts[6] = p1 + basis1 + basis2 + basis3;
    verts[7] = p1 + basis2 + basis3;

    //first verts
    for (int i=0; i<8; ++i) {
        ofs << "v " << verts[i].x << " " << verts[i].y << " " << verts[i].z << "\n";
    }

    //now faces
    int v = vertcount + 1;

    //bottom
    ofs << "f " << v+0 << " " << v+1 << " " << v+2 << " " << v+3 << "\n";
    //top
    ofs << "f " << v+4 << " " << v+5 << " " << v+6 << " " << v+7 << "\n";
    //left
    ofs << "f " << v+0 << " " << v+3 << " " << v+7 << " " << v+4 << "\n";
    //right
    ofs << "f " << v+1 << " " << v+2 << " " << v+6 << " " << v+5 << "\n";
    //front
    ofs << "f " << v+0 << " " << v+1 << " " << v+5 << " " << v+4 << "\n";
    //back
    ofs << "f " << v+2 << " " << v+3 << " " << v+7 << " " << v+6 << "\n";

    vertcount += 8;

}

void PlaneMesh::WriteFeatureGeom(QTextStream & ofs, const Vector3 & p1, const Vector3 & p2, int & vertcount, float ext)
{

    //put our fixed # of verts in (10?)

    Vector3 basis1 = p2 - p1;
    basis1.Normalize();

    Vector3 basis2;
    if (fabs(basis1.Dot(Vector3(1,0,0))) < 0.9)
        basis2 = basis1.Cross(Vector3(1,0,0));
    else
        basis2 = basis1.Cross(Vector3(0,1,0));
    basis2.Normalize();

    Vector3 basis3;
    basis3 = basis1.Cross(basis2);
    basis3.Normalize();

    basis1 = basis1 * ext;
    basis2 = basis2 * ext;
    basis3 = basis3 * ext;

    Vector3 verts[10];
    verts[0] = p1 - basis1;
    verts[1] = p2 + basis1;
    verts[2] = p1 - basis2 - basis3;
    verts[3] = p1 - basis2 + basis3;
    verts[4] = p1 + basis2 + basis3;
    verts[5] = p1 + basis2 - basis3;
    verts[6] = p2 - basis2 - basis3;
    verts[7] = p2 - basis2 + basis3;
    verts[8] = p2 + basis2 + basis3;
    verts[9] = p2 + basis2 - basis3;

    //first verts
    for (int i=0; i<10; ++i) {
        ofs << "v " << verts[i].x << " " << verts[i].y << " " << verts[i].z << "\n";
    }

    //now faces
    int v = vertcount + 1;
    //cap1
    ofs << "f " << v+0 << " " << v+2 << " " << v+3 << "\n";
    ofs << "f " << v+0 << " " << v+3 << " " << v+4 << "\n";
    ofs << "f " << v+0 << " " << v+4 << " " << v+5 << "\n";
    ofs << "f " << v+0 << " " << v+5 << " " << v+2 << "\n";

    //cap2
    ofs << "f " << v+1 << " " << v+6 << " " << v+7 << "\n";
    ofs << "f " << v+1 << " " << v+7 << " " << v+8 << "\n";
    ofs << "f " << v+1 << " " << v+8 << " " << v+9 << "\n";
    ofs << "f " << v+1 << " " << v+9 << " " << v+6 << "\n";

    //shaft
    ofs << "f " << v+2 << " " << v+3 << " " << v+7 << " " << v+6 << "\n";
    ofs << "f " << v+3 << " " << v+4 << " " << v+8 << " " << v+7 << "\n";
    ofs << "f " << v+4 << " " << v+5 << " " << v+9 << " " << v+8 << "\n";
    ofs << "f " << v+5 << " " << v+2 << " " << v+6 << " " << v+9 << "\n";

    vertcount += 10;

}

bool PlaneMesh::ExportFeatures_OBJ(char * filename)
{

    QFile file(filename);

    if (!file.open( QIODevice::WriteOnly | QIODevice::Text )) {
        qDebug() << "File " << filename << " can't be opened for write.";
        return false;
    }

    QTextStream ofs( &file );
    int verts = 0;

    for (int i=0; i<8; ++i) {

        QString group;
        QVector <Vector3> pts;

        switch (i) {

        case 0:
            group = "P_PCA";

            for (int j=0; j<pcaPlaneFits.size(); ++j) {

                const Plane & p = pcaPlaneFits[j];
                Vector3 v1 = p.featcentroid + p.rotAxis1 * p.extent + p.rotAxis2 * p.extent;
                Vector3 v2 = p.featcentroid + p.rotAxis1 * p.extent - p.rotAxis2 * p.extent;
                Vector3 v3 = p.featcentroid - p.rotAxis1 * p.extent - p.rotAxis2 * p.extent;
                Vector3 v4 = p.featcentroid - p.rotAxis1 * p.extent + p.rotAxis2 * p.extent;

                pts.push_back(v1);
                pts.push_back(v2);

                pts.push_back(v2);
                pts.push_back(v3);

                pts.push_back(v3);
                pts.push_back(v4);

                pts.push_back(v4);
                pts.push_back(v1);

            }

            break;

        case 1:
            group = "A_PCA";

            for (int j=0; j<pcaLineFits.size(); ++j) {

                const Axis & a = pcaLineFits[j];

                pts.push_back(a.p - a.dir * a.len);
                pts.push_back(a.p + a.dir * a.len);

            }

            break;

        case 2:
            group = "A_RIDGE";

            for (int j=0; j<ridgeAxes.size(); ++j) {

                const Axis & a = ridgeAxes[j];

                pts.push_back(a.p - a.dir * a.len);
                pts.push_back(a.p + a.dir * a.len);

            }

            break;

        case 3:
            group = "A_VALLEY";

            for (int j=0; j<valleyAxes.size(); ++j) {

                const Axis & a = valleyAxes[j];

                pts.push_back(a.p - a.dir * a.len);
                pts.push_back(a.p + a.dir * a.len);

            }

            break;

        case 4:
            group = "P_SYM";

            for (int j=0; j<symPlanes.size(); ++j) {

                const Plane & p = symPlanes[j];
                Vector3 v1 = p.cent + p.rotAxis1 + p.rotAxis2;
                Vector3 v2 = p.cent + p.rotAxis1 - p.rotAxis2;
                Vector3 v3 = p.cent - p.rotAxis1 - p.rotAxis2;
                Vector3 v4 = p.cent - p.rotAxis1 + p.rotAxis2;

                pts.push_back(v1);
                pts.push_back(v2);

                pts.push_back(v2);
                pts.push_back(v3);

                pts.push_back(v3);
                pts.push_back(v4);

                pts.push_back(v4);
                pts.push_back(v1);

            }

            break;

        case 5:
            group = "A_SYM";

            for (int j=0; j<symLines.size(); ++j) {

                const Axis & a = symLines[j];

                pts.push_back(a.p - a.dir * a.len);
                pts.push_back(a.p + a.dir * a.len);

            }


            break;

        case 6:
            group = "P_GLOBALXYZ";

            for (int j=0; j<globalxyzPlanes.size(); ++j) {

                const Plane & p = globalxyzPlanes[j];
                Vector3 v1 = p.cent + p.rotAxis1 + p.rotAxis2;
                Vector3 v2 = p.cent + p.rotAxis1 - p.rotAxis2;
                Vector3 v3 = p.cent - p.rotAxis1 - p.rotAxis2;
                Vector3 v4 = p.cent - p.rotAxis1 + p.rotAxis2;

                pts.push_back(v1);
                pts.push_back(v2);

                pts.push_back(v2);
                pts.push_back(v3);

                pts.push_back(v3);
                pts.push_back(v4);

                pts.push_back(v4);
                pts.push_back(v1);

            }

            break;

        case 7:
            group = "P_GLOBALPCA";

            for (int j=0; j<globalpcaPlanes.size(); ++j) {

                const Plane & p = globalpcaPlanes[j];
                Vector3 v1 = p.cent + p.rotAxis1 + p.rotAxis2;
                Vector3 v2 = p.cent + p.rotAxis1 - p.rotAxis2;
                Vector3 v3 = p.cent - p.rotAxis1 - p.rotAxis2;
                Vector3 v4 = p.cent - p.rotAxis1 + p.rotAxis2;

                pts.push_back(v1);
                pts.push_back(v2);

                pts.push_back(v2);
                pts.push_back(v3);

                pts.push_back(v3);
                pts.push_back(v4);

                pts.push_back(v4);
                pts.push_back(v1);

            }

            break;

        default:
            break;
        }

        ofs << "g " << group << "\n";
        for (int j=0; j<pts.size(); j+=2) {
            WriteFeatureGeom(ofs, pts[j], pts[j+1], verts);
        }

    }

    file.close();
    return true;

}

bool PlaneMesh::ExportOutputMesh_OBJ(char *filename) {

    FILE *outFile;
    outFile=fopen(filename,"w");

    if (outFile==NULL) {
        qDebug("Unable to write file %s.\n",filename);
        return false;
    }

    // First dump vertex/normal info
    fprintf(outFile,"# %i Plane Parameters:\n",m_vPlanarCuts.size());
    fprintf(outFile,"# nx, ny, nz, px, py, pz, d, r1x, r1y, r1z, r2x, r2y, r2z, area\n");
    for (int i=0;i<m_vPlanarCuts.size();++i) {
        Plane pp=m_vPlanarCuts[i].params;
        fprintf(outFile,"# %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
                pp.norm.x,pp.norm.y,pp.norm.z,
                pp.cent.x,pp.cent.y,pp.cent.z,
                pp.pt,
                pp.rotAxis1.x,pp.rotAxis1.y,pp.rotAxis1.z,
                pp.rotAxis2.x,pp.rotAxis2.y,pp.rotAxis2.z,
                m_vPlanarCuts[i].area);
    }
    fprintf(outFile,"#\n");

    fprintf(outFile,"# Vertex/normal list\n");
    for (int i=0;i<m_vPlanarCuts.size();++i) {

        //fprintf(outFile,"vn %f %f %f\n",
        //        m_vPlanarCuts[i].params.norm.x*-1.0f,m_vPlanarCuts[i].params.norm.y*-1.0f,m_vPlanarCuts[i].params.norm.z*-1.0f);

        for (int j=0;j<m_vPlanarCuts[i].tris.size();++j) {

            fprintf(outFile,"v %f %f %f\n",m_vPlanarCuts[i].tris[j].p1.x,
                    m_vPlanarCuts[i].tris[j].p1.y,
                    m_vPlanarCuts[i].tris[j].p1.z);
            fprintf(outFile,"v %f %f %f\n",m_vPlanarCuts[i].tris[j].p2.x,
                    m_vPlanarCuts[i].tris[j].p2.y,
                    m_vPlanarCuts[i].tris[j].p2.z);
            fprintf(outFile,"v %f %f %f\n",m_vPlanarCuts[i].tris[j].p3.x,
                    m_vPlanarCuts[i].tris[j].p3.y,
                    m_vPlanarCuts[i].tris[j].p3.z);

        }

    }

    // Then dump face info
    fprintf(outFile,"# Face list\n");
    unsigned int curBaseIndex=0;

    for (int i=0;i<m_vPlanarCuts.size();++i) {

        fprintf(outFile,"g plane%i\n",i);
        for (int j=0;j<m_vPlanarCuts[i].tris.size();++j) {

            /*
            fprintf(outFile,"f %i//%i %i//%i %i//%i\n",
                    (curBaseIndex+j)*3+1,
                    i+1,
                    (curBaseIndex+j)*3+2,
                    i+1,
                    (curBaseIndex+j)*3+3,
                    i+1);
                    */

            fprintf(outFile,"f %i %i %i\n",
                    (curBaseIndex+j)*3+1,
                    (curBaseIndex+j)*3+2,
                    (curBaseIndex+j)*3+3);


        }

        curBaseIndex+=m_vPlanarCuts[i].tris.size();

    }

    fclose(outFile);

    qDebug("Successfully wrote %s.\n",filename);

    return true;

}

void PlaneMesh::SelectPlane(unsigned int mouse_x, unsigned int mouse_y){ 

    glClearColor(255,255,255,1.0f);
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
    glClearColor(0.8f,0.8f,0.8f,1.0f);

    RenderPlanePieces(m_vPlanarCuts,3);

    unsigned char r, g, b;
    glReadPixels(mouse_x,mouse_y,1,1,GL_RED,GL_UNSIGNED_BYTE,&r);
    glReadPixels(mouse_x,mouse_y,1,1,GL_GREEN,GL_UNSIGNED_BYTE,&g);
    glReadPixels(mouse_x,mouse_y,1,1,GL_BLUE,GL_UNSIGNED_BYTE,&b);

    int newIndex=(unsigned int)r+((unsigned int)g<<8)+((unsigned int)b<<16);

    if (newIndex<m_vPlanarCuts.size())
        selectedPlaneIndex=newIndex;

    //qDebug("Selected plane %i/%i.\n",selectedPlaneIndex,m_vPlanarCuts.size());

}

void PlaneMesh::ErasePlane(int planeIndex) {

    if (planeIndex < m_vPlanarCuts.size() && planeIndex >= 0)
        m_vPlanarCuts.removeAt(planeIndex);

    if (m_vPlanarCuts.size()>0)
        selectedPlaneIndex=m_vPlanarCuts.size()-1;
    else
        selectedPlaneIndex=INT_MAX;

}

unsigned int PlaneMesh::GetSelectedPlaneIndex() {
    return selectedPlaneIndex;
}

void PlaneMesh::SetSelectedPlaneIndex(int i) {

    if (i<0)
        selectedPlaneIndex=INT_MAX;
    else
        selectedPlaneIndex=i;

    if (selectedPlaneIndex >= m_vSelectedBins.size())
        return;

    PlaneSpaceBin &bin=m_vSelectedBins[i];

    qDebug("Features covered by selected plane %i (%i in total, weight total %f):",i,bin.f.size(),bin.totalvalue);
    for (int j=0; j < bin.f.size(); ++j) {

        int featindex=bin.f[j].fi;
        int type=bin.f[j].ftype;
        const char *&typeName=featureTypeStrings[type];

        qDebug("\tFeature %i (type %i: %s) (weight: %f)",featindex,type,typeName,bin.f[j].value);
    }

}

bool PlaneMesh::GetRayPlaneIntersect(const Plane & p, const Vector3 & rayP, const Vector3 & rayD, Vector3 & intersectPt) {

    const float Vd = p.norm.Dot(rayD);

    if (Vd == 0.0f)
        return false;

    const float V0 = -(p.norm.Dot(rayP) - p.pt);

    const float t = V0 / Vd;

    intersectPt = rayP + rayD * t;

    return t >= 0.0f;

}

bool PlaneMesh::GetRayTriIntersect(const Vector3 & rayp, const Vector3 & rayd, const Triangle & tri, Vector3 & ipt)
{

    Vector3 e1 = tri.p2 - tri.p1;
    Vector3 e2 = tri.p3 - tri.p1;

    // Find the cross product of edge2 and the ray direction
    Vector3 s1 = rayd.Cross(e2);

    // Find the divisor, if its zero, return false as the triangle is degenerate
    float divisor = s1.Dot(e1);
    if (divisor == 0.0f) {
        return false;
    }

    // A inverted divisor, as multipling is faster then division
    float invDivisor = 1.0f / divisor;

    // Calculate the first barycentic coordinate. Barycentic coordinates
    // are between 0.0 and 1.0
    Vector3 distance = rayp - tri.p1;
    float barycCoord_1 = distance.Dot(s1) * invDivisor;

    if (barycCoord_1 < 0.0f || barycCoord_1 > 1.0f) {
        return false;
    }

    // Calculate the second barycentic coordinate
    Vector3 s2 = distance.Cross(e1);
    float barycCoord_2 = rayd.Dot(s2) * invDivisor;
    if (barycCoord_2 < 0.0f || (barycCoord_1 + barycCoord_2) > 1.0f) {
        return false;
    }

    // After doing the barycentic coordinate test we know if the ray hits or
    // not. If we got this far the ray hits.
    // Calculate the distance to the intersection point
    float intdist = e2.Dot(s2) * invDivisor;
    ipt = rayp + rayd * intdist;
    rayp.Debug("rayp");
    rayd.Debug("rayd");
    qDebug() << "intdist" << intdist;
    return true;

}

bool PlaneMesh::GetPlanePlaneIntersect(const Plane & p1, const Plane & p2, Vector3 & ipt, Vector3 & idir)
{

    //if norms parallel, planes dont have line intersection
    if (fabs(p1.norm.Dot(p2.norm)) > 0.999f)
        return false;

    const float n1dn1 = p1.norm.Dot(p1.norm);
    const float n2dn2 = p2.norm.Dot(p2.norm);
    const float n1dn2 = p1.norm.Dot(p2.norm);

    //equation of line = p = c1n1 + c2n2 + u n1 cross n2
    const float det = n1dn1 * n2dn2 - n1dn2 * n1dn2;
    const float c1 = (p1.pt * n2dn2 - p2.pt * n1dn2) / det;
    const float c2 = (p2.pt * n1dn1 - p1.pt * n1dn2) / det;

    //line intersection dir
    idir = p1.norm.Cross(p2.norm);
    idir.Normalize();

    //point on line
    ipt = p1.norm * c1 + p2.norm * c2;

    return true;

}

bool PlaneMesh::GetLinePlaneIntersect(const Triangle & tri, const Vector3 & pt1, const Vector3 & pt2, Vector3 &intersectPoint) {

    Vector3    u, v, n;             // triangle vectors
    Vector3    dir, w0, w;          // ray vectors
    float     r, a, b;             // params to calc ray-plane intersect

    // get triangle edge vectors and plane normal
    u = tri.p2-tri.p1;
    v = tri.p3-tri.p1;
    n=u.Cross(v);             // cross product
    if (n.Length()==0.0f)            // triangle is degenerate
        return false;                 // do not deal with this case

    dir = pt2-pt1;             // ray direction vector
    w0 = pt1 - tri.p1;
    a = -n.Dot(w0);
    b = n.Dot(dir);
    if (fabs(b) < 0.0001f) {     // ray is parallel to triangle plane
        return false;             // ray disjoint from plane
    }

    // get intersect point of ray with triangle plane
    r = a / b;
    //if (r < 0.0)                   // ray goes away from triangle
    //     return NULL;                  // => no intersect
    // for a segment, also test if (r > 1.0) => no intersect

    intersectPoint = pt1 + dir*r;           // intersect point of ray and plane	
    return true;

}

Vector3 PlaneMesh::GetBoundCentre() {

    if (inputMesh==NULL)
        return Vector3(0,0,0);
    else
        return inputMesh->GetBoundCentre();
}

float PlaneMesh::GetTriangleArea(const Triangle & t) {

    float a=(t.p1-t.p2).Length();
    float b=(t.p2-t.p3).Length();
    float c=(t.p3-t.p1).Length();

    QList <float> sv;
    sv.push_back(a);
    sv.push_back(b);
    sv.push_back(c);

    qSort(sv.begin(), sv.end(), qGreater<float>());

    return 0.25f*sqrtf( (sv[0]+(sv[1]+sv[2]))*(sv[2]-(sv[0]-sv[1]))*(sv[2]+(sv[0]-sv[1]))*(sv[0]+(sv[1]-sv[2])) );

}

void PlaneMesh::LocalSearchFindMaxAreaPlane(PlanarCut & cut, float fRad, const float fRadThreshold) {

    //qDebug("Recursing for maxarea plane: rad: %f, threshold: %f", fRad, fRadThreshold);

    if (cut.tris.empty())
        GetPlanarCut(cut);

    int theta, phi, r;
    m_BinSpace.CartesianToPolar(cut.params.norm, cut.params.pt, theta, phi, r);

    float ftheta = float(theta);
    float fphi = float(phi);
    float fr = float(r);

    PlanarCut bestcut = cut;

    //take cuts at corners of cube        
    PlanarCut cuts[27];        

    while (fRad > fRadThreshold) {      

        for (int x=0; x<3; ++x) {
            for (int y=0; y<3; ++y) {
                for (int z=0; z<3; ++z) {

                    PlanarCut & eachcut = cuts[x + y*3 + z*9];

                    float etheta = ftheta + fRad * float(x-1);
                    float ephi = fphi + fRad * float(y-1);
                    float er = fr + fRad * float(z-1);

                    m_BinSpace.PolarToCartesianF(etheta, ephi, er, eachcut.params.norm, eachcut.params.pt);
                    SetPlaneParamsFromNormPt(eachcut.params);

                    GetPlanarCut(eachcut);

                }
            }
        }

        //branch in direction of maxarea
        float maxarea = -FLT_MAX;        
        int bestind = -1;

        for (int ind=0; ind<27; ++ind) {

            //if cut has fewest chains we take it... otherwise, we take cut with fewest chains that maximizes area
            /*
            if ( cuts[ind].area > maxarea && cuts[ind].area > cut.area && cuts[ind].area > 0.0f &&
                 (cuts[ind].chains.size() <= cut.chains.size() || cut.chains.size() == 0) ) {
                 */
            if ( cuts[ind].area > maxarea) {

                maxarea=cuts[ind].area;                
                bestind=ind;

            }

        }

        if ( bestind >= 0 ) {

            //this re-centers search space
            m_BinSpace.CartesianToPolarF(cuts[bestind].params.norm, cuts[bestind].params.pt, ftheta, fphi, fr);

            bestcut = cuts[bestind];

        }

        fRad/=2.0f;

    }

    cut = bestcut;

}

float PlaneMesh::GetPolygonArea(Vector3 v1, Vector3 v2, const QList <MeshSegment> & meshSegments) {

    float areaValue=0.0f;

    for (int i=0; i<meshSegments.size(); ++i) {

        areaValue+=meshSegments[i].pt1.Dot(v1)*meshSegments[i].pt2.Dot(v2);
        areaValue-=meshSegments[i].pt2.Dot(v1)*meshSegments[i].pt1.Dot(v2);

    }

    areaValue/=2.0f;

    return areaValue;

}

float PlaneMesh::GetPolygonPerimeter(const QList <MeshSegment> & meshSegments) {

    float perim=0.0f;

    for (int i=0; i<meshSegments.size()-1; ++i)
        perim+=(meshSegments[i+1].pt1-meshSegments[i].pt1).Length();    

    return perim;

}

float PlaneMesh::GetPolygonCurvatureVariation(Vector3 v1, Vector3 v2, const QList <MeshSegment> & meshSegments) {

    //project chain onto plane
    QList <Vector3> edgePointSet;

    for (int i=0; i<meshSegments.size(); ++i)
        edgePointSet.push_back(Vector3(meshSegments[i].pt1.Dot(v1),0.0,meshSegments[i].pt1.Dot(v2))); //projection to XZ, using basis v1/v2

    //calculate curvature there
    QList <float> curvVec;

    for (int i=0;i<edgePointSet.size();++i) {

        //curve is closed... so indices must loop around
        int ind=i;
        int indm1=i-1;
        if (indm1<0)
            indm1=edgePointSet.size()-1;
        int indp1=i+1;
        if (indp1>=edgePointSet.size())
            indp1=0;

        //this curvature definition from schneider/kobbelt paper
        Vector3 vec1=edgePointSet[ind]-edgePointSet[indm1];
        Vector3 vec2=edgePointSet[indp1]-edgePointSet[ind];

        float det=vec1.x*vec2.z-vec2.x*vec1.z;

        float curvVal=2.0f*det/(vec1.Length()*vec2.Length()*(vec1+vec2).Length());
        curvVec.push_back(curvVal);      

    }

    //estimate variation of curvature
    //curvature variation is estimated as integrating abs(dCurve/dt)
    float curvVar=0.0f;

    for (int i=0; i<curvVec.size()-1; ++i)
        curvVar+=(fabs(curvVec[i+1]-curvVec[i])*(meshSegments[i+1].pt1-meshSegments[i].pt1).Length());    

    return curvVar;

}

float PlaneMesh::GetPlaneCutArea(int planeIndex) {
    return m_vPlanarCuts[planeIndex].area;
}

bool PlaneMesh::AddNewPlane() {

    if (inputMesh==NULL)
        return false;

    PlanarCut newCut;
    newCut.params.norm=Vector3(1,0,0);
    newCut.params.rotAxis1=Vector3(0,1,0);
    newCut.params.rotAxis2=Vector3(0,0,1);
    newCut.params.cent=Vector3(inputMesh->GetBoundRad(),0,0)+inputMesh->GetBoundCentre();
    newCut.params.pt=newCut.params.cent.Dot(newCut.params.norm);

    newCut.params.posB1=newCut.params.cent;
    newCut.params.posB2=newCut.params.cent+newCut.params.rotAxis1*inputMesh->GetBoundRad()*0.5;
    newCut.params.posB3=newCut.params.cent+newCut.params.rotAxis2*inputMesh->GetBoundRad()*0.5;

    GetPlanarCut(newCut);
    AddPlanarCut(newCut);

    SetSelectedPlaneIndex(m_vPlanarCuts.size()-1);

    return true;

}

bool PlaneMesh::mousePressEvent(QMouseEvent *event, int, int winHeight) {

    if (selectedPlaneIndex>=m_vPlanarCuts.size())
        return false;

    glClearColor(0.0f,1.0f,1.0f,1.0f);
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
    glClearColor(0.8f,0.8f,0.8f,1.0f);

    //draw
    glLineWidth(7.0f);
    DrawPlaneCoordFrame(m_vPlanarCuts[selectedPlaneIndex].params);
    glLineWidth(1.0f);

    unsigned char r, g, b;
    glReadPixels(event->x(),winHeight-event->y()-1,1,1,GL_RED,GL_UNSIGNED_BYTE,&r);
    glReadPixels(event->x(),winHeight-event->y()-1,1,1,GL_GREEN,GL_UNSIGNED_BYTE,&g);
    glReadPixels(event->x(),winHeight-event->y()-1,1,1,GL_BLUE,GL_UNSIGNED_BYTE,&b);

    int newIndex=(unsigned int)r+((unsigned int)g<<8)+((unsigned int)b<<16);

    //qDebug("Read: %i %i %i = %i",r,g,b,newIndex);

    const int rot1col=254+(254<<8)+(254<<16);
    const int rot2col=253+(253<<8)+(253<<16);
    const int trans1col=255+(0<<8)+(0<<16);
    const int trans2col=0+(255<<8)+(0<<16);

    const int ball1col=255+(20<<8)+(20<<16);
    const int ball2col=254+(20<<8)+(20<<16);
    const int ball3col=253+(20<<8)+(20<<16);

    lastPos=event->pos();

    if (planeManip==NONE) {

        switch (newIndex) {

        case rot1col:

            planeManip=ROT1;
            break;

        case rot2col:

            planeManip=ROT2;
            break;

        case trans1col:

            planeManip=TRANS;
            break;

        case trans2col:

            planeManip=TRANS;
            break;

        default:

            planeManip=NONE;
            return false;

        }

    }
    else if (planeManip==NONEBALL) {

        switch (newIndex) {
        case ball1col:
            planeManip=MOVEBALL1;
            break;

        case ball2col:
            planeManip=MOVEBALL2;
            break;

        case ball3col:
            planeManip=MOVEBALL3;
            break;

        default:
            return false;
        }

    }

    return true;
}

bool PlaneMesh::mouseReleaseEvent(QMouseEvent *, int, int) {

    if (planeManip==NONEBALL ||
        planeManip==MOVEBALL1 ||
        planeManip==MOVEBALL2 ||
        planeManip==MOVEBALL3 )
        planeManip=NONEBALL;
    else
        planeManip=NONE;

    return false;

}

bool PlaneMesh::mouseMoveEvent(QMouseEvent *event, int winWidth, int winHeight) {

    if (planeManip==NONE||planeManip==NONEBALL)
        return false;

    if (selectedPlaneIndex>=m_vPlanarCuts.size())
        return false;

    Plane & pp = m_vPlanarCuts[selectedPlaneIndex].params;

    Vector3 udDisplace=camera->GetUp();
    Vector3 lrDisplace=udDisplace.Cross(camera->GetCent()-camera->GetEye());
    udDisplace.Normalize();
    lrDisplace.Normalize();
    float rotAngle;

    switch (planeManip) {  

    case ROT1:

        rotAngle=4.0f/winHeight*(event->y()-lastPos.y())+
                 4.0f/winWidth*(event->x()-lastPos.x());
        rotAngle=-rotAngle;

        pp.norm=GetRotatedAxis(rotAngle,pp.norm,pp.rotAxis1);
        pp.rotAxis2=GetRotatedAxis(rotAngle,pp.rotAxis2,pp.rotAxis1);

        break;

    case ROT2:

        rotAngle=4.0f/winHeight*(event->y()-lastPos.y())+
                 4.0f/winWidth*(event->x()-lastPos.x());
        rotAngle=-rotAngle;

        pp.norm=GetRotatedAxis(rotAngle,pp.norm,pp.rotAxis2);
        pp.rotAxis1=GetRotatedAxis(rotAngle,pp.rotAxis1,pp.rotAxis2);
        break;

    case TRANS:

        {
            pp.cent=pp.cent+pp.norm*(-2.0f/winHeight*(event->x()-lastPos.x())*pp.norm.Dot(lrDisplace))*inputMesh->GetBoundRad();
            pp.cent=pp.cent+pp.norm*(-2.0f/winHeight*(event->y()-lastPos.y())*pp.norm.Dot(udDisplace))*inputMesh->GetBoundRad();
            pp.pt=pp.cent.Dot(pp.norm);
        }
        break;

    case MOVEBALL1:

        UnprojectPosOntoInputMesh(event->pos(),pp.posB1);
        break;

    case MOVEBALL2:

        UnprojectPosOntoInputMesh(event->pos(),pp.posB2);
        break;

    case MOVEBALL3:

        UnprojectPosOntoInputMesh(event->pos(),pp.posB3);
        break;

    default:

        break;

    }

    if (planeManip==MOVEBALL1||planeManip==MOVEBALL2||planeManip==MOVEBALL3) {
        pp.cent=pp.posB1;
        pp.rotAxis1=pp.posB2-pp.posB1;
        pp.rotAxis1.Normalize();
        pp.rotAxis2=pp.posB3-pp.posB1;
        pp.rotAxis2=pp.rotAxis2-pp.rotAxis1*(pp.rotAxis1.Dot(pp.rotAxis2));
        pp.rotAxis2.Normalize();

        pp.norm=pp.rotAxis1.Cross(pp.rotAxis2);
    }

    GetPlanarCut(m_vPlanarCuts[selectedPlaneIndex]);

    lastPos=event->pos();

    return true;

}

bool PlaneMesh::UnprojectPos(QPoint pos, Vector3 & v) {

    glGetIntegerv(GL_VIEWPORT, origViewport);
    glGetDoublev( GL_PROJECTION_MATRIX, origProj );
    glGetDoublev( GL_MODELVIEW_MATRIX, origModl );

    float zVal;
    glReadPixels( pos.x(),origViewport[3]-pos.y(), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &zVal );

    if (zVal>=1.0f)
        return false;  

    double px, py, pz;
    gluUnProject(pos.x(),origViewport[3]-pos.y(),zVal,origModl,origProj,origViewport,&px,&py,&pz);

    v.x=(float)px;
    v.y=(float)py;
    v.z=(float)pz;

    return true;

}

void PlaneMesh::UnprojectPosOntoInputMesh(QPoint pos, Vector3 &v) {

    glGetIntegerv(GL_VIEWPORT, origViewport);
    glGetDoublev( GL_PROJECTION_MATRIX, origProj );
    glGetDoublev( GL_MODELVIEW_MATRIX, origModl );

    float zVal;
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    inputMesh->DrawSimple();
    glReadPixels( pos.x(),origViewport[3]-pos.y(), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &zVal );

    if (zVal>=1.0f)
        return;

    zVal+=0.00025f;

    double px, py, pz;
    gluUnProject(pos.x(),origViewport[3]-pos.y(),zVal,origModl,origProj,origViewport,&px,&py,&pz);

    v.x=(float)px;
    v.y=(float)py;
    v.z=(float)pz;

}

void PlaneMesh::SetPlaneManipMethod(PlaneManip m) {
    planeManip=m;
}

void PlaneMesh::SnapPlaneNormalToXAxis() {

    if (selectedPlaneIndex>=m_vPlanarCuts.size())
        return;

    Plane & pp = m_vPlanarCuts[selectedPlaneIndex].params;

    pp.cent=inputMesh->GetBoundCentre();
    pp.norm=Vector3(1,0,0);

    SetPlaneParamsFromNormCent(pp);

    pp.posB1=pp.cent;
    pp.posB2=pp.cent+pp.rotAxis1*inputMesh->GetBoundRad()*0.5;
    pp.posB3=pp.cent+pp.rotAxis2*inputMesh->GetBoundRad()*0.5;

}

void PlaneMesh::SnapPlaneNormalToYAxis() {

    if (selectedPlaneIndex>=m_vPlanarCuts.size())
        return;

    Plane & pp = m_vPlanarCuts[selectedPlaneIndex].params;

    pp.cent=inputMesh->GetBoundCentre();
    pp.norm=Vector3(0,1,0);

    SetPlaneParamsFromNormCent(pp);

    pp.posB1=pp.cent;
    pp.posB2=pp.cent+pp.rotAxis1*inputMesh->GetBoundRad()*0.5;
    pp.posB3=pp.cent+pp.rotAxis2*inputMesh->GetBoundRad()*0.5;   

}

void PlaneMesh::SnapPlaneNormalToZAxis() {

    if (selectedPlaneIndex>=m_vPlanarCuts.size())
        return;

    Plane & pp = m_vPlanarCuts[selectedPlaneIndex].params;

    pp.cent=inputMesh->GetBoundCentre();
    pp.norm=Vector3(0,0,1);

    SetPlaneParamsFromNormCent(pp);

    pp.posB1=pp.cent;
    pp.posB2=pp.cent+pp.rotAxis1*inputMesh->GetBoundRad()*0.5f;
    pp.posB3=pp.cent+pp.rotAxis2*inputMesh->GetBoundRad()*0.5f;

}

void PlaneMesh::SetUseSegmentations(bool s) {

    m_bUseFaceSeg=s;

}

float PlaneMesh::GetPointLineDistance(const Vector3 & p, const Axis & a) {

    Vector3 x0 = p;
    Vector3 x1 = a.p;
    Vector3 x2 = a.p + a.dir;

    return (x0-x1).Cross(x0-x2).Length() / (x2-x1).Length() ;

}

/*
 * New version - order n... Greedy, higher error, but way faster for longer chains
 */
void PlaneMesh::PiecewiseAxesFromCurve(const QList <MeshSegment> & curvepts, const float axiscost, QList <Axis> &bestaxes, QList <QList <MeshSegment> > &pointchains) {

    const int npts=curvepts.size();

    for (int i=0; i<npts; ++i) {

        QList <MeshSegment> eachPts;

        eachPts.push_back(curvepts[i]);

        for (int j=i+1; j<npts; ++j) {

            eachPts.push_back(curvepts[j]);

            Axis axis;
            GetBestFitAxis(eachPts, axis);

            float fiterror = 0.0f;

            for (int k=0; k<eachPts.size(); ++k)
                fiterror+=GetPointLineDistance(eachPts[k].pt1, axis) * (eachPts[k].pt2-eachPts[k].pt1).Length(); //scale point error by line segment length

            if (fiterror > axiscost) {

                eachPts.pop_back();
                GetBestFitAxis(eachPts, axis);

                bestaxes.push_back(axis);
                pointchains.push_back(eachPts);

                i = j-1;
                break;

            }

        }

    }

}

/*
 * Old version: n^2 complexity... lower total error, but way slower
 */
void PlaneMesh::PiecewiseAxesFromCurve2(const QList <MeshSegment> & curvepts, const float axiscost, QList <Axis> &bestaxes, QList <QList <MeshSegment> > &pointchains) {

    //allocate cost matrix and initialize
    int npts=curvepts.size();
    float **errormatrix=new float*[npts];
    int **walkmatrix=new int*[npts];

    for (int i=0; i<npts; ++i) {

        errormatrix[i]=new float[npts];
        walkmatrix[i]=new int[npts];

        for (int j=0; j<npts; ++j) {

            errormatrix[i][j]=axiscost;
            walkmatrix[i][j]=-1;

        }
    }

    //iterate through remaining diagonals, doing dynamic programming stuff...
    for (int j=3; j<npts; ++j) {
        for (int i=0; i+j<npts; ++i) {

            QList <MeshSegment> ptstofit=curvepts.mid(i,j+1);

            //fit best plane to these points
            Axis axis;
            GetBestFitAxis(ptstofit,axis);

            //get fit error of this single axis
            float fiterror=0.0f;

            for (int k=0; k<ptstofit.size(); ++k)
                fiterror+=GetPointLineDistance(ptstofit[k].pt1, axis) * (ptstofit[k].pt2-ptstofit[k].pt1).Length(); //scale point error by line segment length

            float minerror=fiterror+axiscost;
            int minindex=-1;

            for (int each=i+1; each<i+j; ++each) {
                if (errormatrix[i][each]+errormatrix[each][i+j]<minerror) {
                    minindex=each;
                    minerror=errormatrix[i][each]+errormatrix[each][i+j];
                }
            }

            //qDebug("(%i,%i): %f: split? %i",i,i+j,minerror,minindex);

            walkmatrix[i][i+j]=minindex;
            errormatrix[i][i+j]=minerror;

        }
    }

    //go through error/walk matrices and take best planes
    bool planeend[npts];
    for (int i=0; i<npts; ++i)
        planeend[i]=false;

    //qDebug("Recursing through %ix%i matrix",npts,npts);
    RecurseThroughWalkMatrix(walkmatrix,0,npts-1,planeend);

    for (int i=0; i<npts-1; ++i) {

        if (planeend[i]) {

            for (int j=i+1; j<npts; ++j) {

                if (planeend[j]) {

                    QList <MeshSegment> ptstofit=curvepts.mid(i,j-i+1);

                    Axis axis;
                    GetBestFitAxis(ptstofit, axis);

                    bestaxes.push_back(axis);
                    pointchains.push_back(ptstofit);

                    i=j-1; //skip ahead

                    break;

                }

            }

        }
    }

    //deallocate
    for (int i=0; i<npts; ++i) {
        delete [] errormatrix[i];
        delete [] walkmatrix[i];
    }

    delete [] errormatrix;
    delete [] walkmatrix;



}

void PlaneMesh::PiecewisePlanesFromCurve(const QList <MeshSegment> & curvepts, const float planecost, QList <Plane> &bestplanes, QList <QList <MeshSegment> > &pointchains) {

    //allocate cost matrix and initialize
    int npts=curvepts.size();
    float **errormatrix=new float*[npts];
    int **walkmatrix=new int*[npts];

    for (int i=0;i<npts;++i) {

        errormatrix[i]=new float[npts];
        walkmatrix[i]=new int[npts];

        for (int j=0;j<npts;++j) {
            errormatrix[i][j]=planecost;
            walkmatrix[i][j]=-1;
        }
    }

    //iterate through remaining diagonals, doing dynamic programming stuff...
    for (int j=3; j<npts; ++j) {
        for (int i=0; i+j<npts; ++i) {

            QList <MeshSegment> ptstofit=curvepts.mid(i,j+1);

            //fit best plane to these points
            Plane plane;
            GetBestFitPlane(ptstofit,plane);

            //get fit error of this single plane            
            float fiterror=0.0f;

            for (int k=0;k<ptstofit.size();k++)
                fiterror+=fabs(ptstofit[k].pt1.Dot(plane.norm)+plane.pt);

            float minerror=fiterror+planecost;
            int minindex=-1;

            for (int each=i+1;each<i+j;each++) {
                if (errormatrix[i][each]+errormatrix[each][i+j]<minerror) {
                    minindex=each;
                    minerror=errormatrix[i][each]+errormatrix[each][i+j];
                }
            }

            //qDebug("(%i,%i): %f: split? %i",i,i+j,minerror,minindex);

            walkmatrix[i][i+j]=minindex;
            errormatrix[i][i+j]=minerror;

        }
    }

    //go through error/walk matrices and take best planes
    bool planeend[npts];
    for (int i=0; i<npts; ++i)
        planeend[i]=false;

    //qDebug("Recursing through %ix%i matrix",npts,npts);
    RecurseThroughWalkMatrix(walkmatrix,0,npts-1,planeend);

    for (int i=0; i<npts-1; ++i) {

        if (planeend[i]) {

            for (int j=i+1; j<npts; ++j) {

                if (planeend[j]) {

                    QList <MeshSegment> ptstofit=curvepts.mid(i,j-i+1);

                    Plane plane;
                    GetBestFitPlane(ptstofit,plane);

                    bestplanes.push_back(plane);
                    pointchains.push_back(ptstofit);

                    float fiterror=0.0f;

                    for (int k=0; k<ptstofit.size(); ++k)
                        fiterror+=fabs(ptstofit[k].pt1.Dot(plane.norm)+plane.pt);                    

                    i=j-1; //skip ahead

                    break;

                }

            }

        }
    }

    //deallocate
    for (int i=0; i<npts; ++i) {
        delete [] errormatrix[i];
        delete [] walkmatrix[i];
    }

    delete [] errormatrix;
    delete [] walkmatrix;    

}

void PlaneMesh::RecurseThroughWalkMatrix(int ** walkmatrix, int begin, int end, bool *planeend) {

    if (begin+2>=end) {
        planeend[begin]=true;
        planeend[end]=true;
    }   
    else {

        if (walkmatrix[begin][end]==-1) {
            planeend[begin]=true;
            planeend[end]=true;
        }
        else {
            RecurseThroughWalkMatrix(walkmatrix, begin, walkmatrix[begin][end], planeend);
            RecurseThroughWalkMatrix(walkmatrix, walkmatrix[begin][end], end, planeend);
        }
    }

}

void PlaneMesh::GetBestFitPlane(const QList <MeshSegment> & points, Plane &plane) {

    QVector <Vector3> pts;
    QList <Vector3> eigenvecs;
    QList <float> eigenvals;
    Vector3 centroid;
    float maxdist;

    for (int i=0; i<points.size(); ++i)
        pts.push_back(points[i].pt1);

    EigenSolver::PCAForPoints(pts, eigenvecs, eigenvals, centroid, maxdist);

    plane.norm = eigenvecs[2];
    plane.norm.Normalize();
    plane.cent = centroid;    

    SetPlaneParamsFromNormCent(plane);

}

void PlaneMesh::GetBestFitAxis(const QList <MeshSegment> & points, Axis &axis) {

    QVector <Vector3> pts;
    QList <Vector3> eigenvecs;
    QList <float> eigenvals;
    Vector3 centroid;
    float maxdist;

    pts.push_back(points[0].pt1);
    for (int i=0; i<points.size(); ++i)
        pts.push_back(points[i].pt2);

    EigenSolver::PCAForPoints(pts, eigenvecs, eigenvals, centroid, maxdist);

    axis.dir = eigenvecs[0];
    axis.dir.Normalize();

    axis.p = Vector3(0,0,0);
    axis.len = 0.0f;

    for (int i=0; i < points.size(); ++i) {

        float eachlen = (points[i].pt2-points[i].pt1).Length();

        axis.len+= eachlen;
        axis.p+= (points[i].pt1 + points[i].pt2) * 0.5f * eachlen;

    }

    axis.p = axis.p * (1.0f/axis.len);
    axis.len/=2.0f;


}

void PlaneMesh::PCAPlanesForMeshSegment(const int segmentIndex, QList <Vector3> &eigenvecs, QList <float> &eigenvals, Vector3 &centroid) {

    //Go through mesh...  First get segment midpoint
    QVector <QVector <int> > &faceList=inputMesh->getFaces();    
    QVector <Vector3> &verts=inputMesh->getVerts();
    int numFaces=faceList.size();

    QVector <Vector3> pts;
    float maxdist;

    for (int i=0; i<numFaces; ++i) {

        if (m_vMeshFaceSeg[i]!=segmentIndex)
            continue;

        for (int j=0; j<3; ++j) {
            pts.push_back(Vector3(verts[faceList[i][j]].x,
                              verts[faceList[i][j]].y,
                              verts[faceList[i][j]].z));
        }

    }

    EigenSolver::PCAForPoints(pts, eigenvecs, eigenvals, centroid, maxdist);

}

void PlaneMesh::MeshRidges(bool do_ridge, float thresh, QList <Vector3> & ridgesegments, QList <Vector3> & ridgenorms, QList <float> & ridgeness) {

    const int *t = &themesh->tstrips[0];
    const int *stripend = t;
    const int *end = t + themesh->tstrips.size();

    // Walk through triangle strips
    while (true) {

        if (unlikely(t >= stripend)) {

            if (unlikely(t >= end))
                return;
            // New strip: each strip is stored as
            // length followed by indices
            stripend = t + 1 + *t;
            // Skip over length plus first two indices of
            // first face
            t += 3;
        }

        GetFaceRidges(*(t-2), *(t-1), *t, do_ridge, thresh, ridgesegments, ridgenorms, ridgeness);

        t++;
    }   

}

void PlaneMesh::GetFaceRidges(int v0, int v1, int v2, bool do_ridge, float thresh, QList <Vector3> & ridgesegments, QList <Vector3> & ridgenorms, QList <float> & ridgeness) {

    // Check if ridge possible at vertices just based on curvatures
    if (do_ridge) {
            if ((themesh->curv1[v0] <= 0.0f) ||
                (themesh->curv1[v1] <= 0.0f) ||
                (themesh->curv1[v2] <= 0.0f))
                    return;
    } else {
            if ((themesh->curv1[v0] >= 0.0f) ||
                (themesh->curv1[v1] >= 0.0f) ||
                (themesh->curv1[v2] >= 0.0f))
                    return;
    }

    // Sign of curvature on ridge/valley
    float rv_sign = do_ridge ? 1.0f : -1.0f;

    // The "tmax" are the principal directions of maximal curvature,
    // flipped to point in the direction in which the curvature
    // is increasing (decreasing for valleys).  Note that this
    // is a bit different from the notation in Ohtake et al.,
    // but the tests below are equivalent.
    const float &emax0 = themesh->dcurv[v0][0];
    const float &emax1 = themesh->dcurv[v1][0];
    const float &emax2 = themesh->dcurv[v2][0];
    vec tmax0 = rv_sign * themesh->dcurv[v0][0] * themesh->pdir1[v0];
    vec tmax1 = rv_sign * themesh->dcurv[v1][0] * themesh->pdir1[v1];
    vec tmax2 = rv_sign * themesh->dcurv[v2][0] * themesh->pdir1[v2];

    // We have a "zero crossing" if the tmaxes along an edge
    // point in opposite directions
    bool z01 = ((tmax0 DOT tmax1) <= 0.0f);
    bool z12 = ((tmax1 DOT tmax2) <= 0.0f);
    bool z20 = ((tmax2 DOT tmax0) <= 0.0f);

    if (z01 + z12 + z20 < 2)
            return;

    // Draw line segment
    const float &kmax0 = themesh->curv1[v0];
    const float &kmax1 = themesh->curv1[v1];
    const float &kmax2 = themesh->curv1[v2];
    if (!z01) {
            GetSegmentRidge(v1, v2, v0,
                               emax1, emax2, emax0,
                               kmax1, kmax2, kmax0,
                               thresh, false,
                               ridgesegments, ridgenorms, ridgeness);
    }
    else if (!z12) {
            GetSegmentRidge(v2, v0, v1,
                               emax2, emax0, emax1,
                               kmax2, kmax0, kmax1,
                               thresh, false,
                               ridgesegments, ridgenorms, ridgeness);
    }
    else if (!z20) {
            GetSegmentRidge(v0, v1, v2,
                               emax0, emax1, emax2,
                               kmax0, kmax1, kmax2,
                               thresh, false,
                               ridgesegments, ridgenorms, ridgeness);
    }
    else {
        // All three edges have crossings -- connect all to center
        //James: since dealing with just chains, verts with degree > 2 are not too useful
        /*
            return GetSegmentRidge(v1, v2, v0,
                               emax1, emax2, emax0,
                               kmax1, kmax2, kmax0,
                               thresh, true);
            return GetSegmentRidge(v2, v0, v1,
                               emax2, emax0, emax1,
                               kmax2, kmax0, kmax1,
                               thresh, true);
            return GetSegmentRidge(v0, v1, v2,
                               emax0, emax1, emax2,
                               kmax0, kmax1, kmax2,
                               thresh, true);
            */
    }
}

void PlaneMesh::GetSegmentRidge(int v0, int v1, int v2, float emax0, float emax1, float emax2, float kmax0, float kmax1, float kmax2, float thresh, bool to_center,
                                QList <Vector3> &ridgesegments, QList <Vector3> &ridgenorms, QList <float> & ridgeness)
{

    // Interpolate to find ridge/valley line segment endpoints
    // in this triangle and the curvatures there
    float w10 = fabs(emax0) / (fabs(emax0) + fabs(emax1));
    float w01 = 1.0f - w10;
    point p01 = w01 * themesh->vertices[v0] + w10 * themesh->vertices[v1];
    float k01 = fabs(w01 * kmax0 + w10 * kmax1);

    point p12;
    float k12;
    if (to_center) {
            // Connect first point to center of triangle
            p12 = (themesh->vertices[v0] +
                   themesh->vertices[v1] +
                   themesh->vertices[v2]) / 3.0f;
            k12 = fabs(kmax0 + kmax1 + kmax2) / 3.0f;
    } else {
            // Connect first point to second one (on next edge)
            float w21 = fabs(emax1) / (fabs(emax1) + fabs(emax2));
            float w12 = 1.0f - w21;
            p12 = w12 * themesh->vertices[v1] + w21 * themesh->vertices[v2];
            k12 = fabs(w12 * kmax1 + w21 * kmax2);
    }

    // Don't draw below threshold
    k01 -= thresh;
    if (k01 < 0.0f)
            k01 = 0.0f;
    k12 -= thresh;
    if (k12 < 0.0f)
            k12 = 0.0f;

    // Skip lines that you can't see...
    if (k01 == 0.0f && k12 == 0.0f)
            return;

    // Fade lines    
    //k01 = k12 = 1.0f;

    // Average per-vert norms together for facenorm
    Vector3 n[3], navg;
    n[0]=Vector3(themesh->normals[v0][0],themesh->normals[v0][1],themesh->normals[v0][2]);
    n[1]=Vector3(themesh->normals[v1][0],themesh->normals[v1][1],themesh->normals[v1][2]);
    n[2]=Vector3(themesh->normals[v2][0],themesh->normals[v2][1],themesh->normals[v2][2]);
    navg=n[0]+n[1]+n[2];
    navg.Normalize();

    // Append the line segment
    ridgesegments.push_back(Vector3(p01[0],p01[1],p01[2]));
    ridgenorms.push_back(navg);
    ridgeness.push_back(k01);

    ridgesegments.push_back(Vector3(p12[0],p12[1],p12[2]));
    ridgenorms.push_back(navg);
    ridgeness.push_back(k12);

}

void PlaneMesh::ComputeMeshFeatures() {   

    /*
     * Extract feature planes (key planes) from this mesh
     */    
    GetFeatures_MeshRidges();
    emit percentComplete(50);

    GetFeatures_MeshValleys();
    emit percentComplete(100);

    GetKeyPlanes_PCAForMeshSegments();    
    emit percentComplete(150);

    GetKeyPlanes_GlobalXYZ();    
    emit percentComplete(200);

    GetKeyPlanes_GlobalPCA();
    emit percentComplete(250);

    //symmetry planes already loaded upon loading mesh...

    /*
     * Populate unified feature list
     */

    meshFeatures.clear();

    if (m_bUseFeature[P_PCA])        
        for (int i=0; i<pcaPlaneFits.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=P_PCA;
            eachFeature.typeindex=i;
            eachFeature.p=pcaPlaneFits[i];

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[A_PCA])
        for (int i=0; i<pcaLineFits.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=A_PCA;
            eachFeature.typeindex=i;
            eachFeature.a=pcaLineFits[i];
            eachFeature.usedPerpSupport = false;

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[A_RIDGE])
        for (int i=0; i<ridgeAxes.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=A_RIDGE;
            eachFeature.typeindex=i;
            eachFeature.a=ridgeAxes[i];

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[A_VALLEY])
        for (int i=0; i<valleyAxes.size(); ++i) {

            MeshFeature eachFeature;
            eachFeature.type=A_VALLEY;
            eachFeature.typeindex=i;
            eachFeature.a=valleyAxes[i];

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[P_SYM])
        for (int i=0; i<symPlanes.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=P_SYM;
            eachFeature.typeindex=i;
            eachFeature.p=symPlanes[i];

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[A_SYM])
        for (int i=0; i<symLines.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=A_SYM;
            eachFeature.typeindex=i;
            eachFeature.a=symLines[i];
            eachFeature.usedPerpSupport = false;

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[P_GLOBALXYZ])
        for (int i=0; i<globalxyzPlanes.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=P_GLOBALXYZ;
            eachFeature.typeindex=i;
            eachFeature.p=globalxyzPlanes[i];

            meshFeatures.push_back(eachFeature);
        }

    if (m_bUseFeature[P_GLOBALPCA])
        for (int i=0; i<globalpcaPlanes.size(); ++i) {
            MeshFeature eachFeature;
            eachFeature.type=P_GLOBALPCA;
            eachFeature.typeindex=i;
            eachFeature.p=globalpcaPlanes[i];

            meshFeatures.push_back(eachFeature);
        }

}

void PlaneMesh::SetPlaneParamsFromNormPt(Plane & p) {

    //ensure d positive
    if (p.pt < 0.0f) {
        p.pt = -p.pt;
        p.norm = p.norm * (-1.0f);
    }

    //generate centrepoint
    p.cent=p.norm*p.pt;

    //create basis vectors for plane (rotation axes)
    Vector3 crossvec;

    if (fabs(p.norm.Dot(Vector3(0,1,0)))>0.9f)
        crossvec=Vector3(1,0,0);
    else
        crossvec=Vector3(0,1,0);

    p.rotAxis1=p.norm.Cross(crossvec);
    p.rotAxis1.Normalize();
    p.rotAxis2=p.norm.Cross(p.rotAxis1);
    p.rotAxis2.Normalize();

}

void PlaneMesh::SetPlaneParamsFromNormCent(Plane &p) {

    p.pt = - p.norm.Dot(p.cent);

    //ensure d positive
    if (p.pt < 0.0f) {
        p.pt = -p.pt;
        p.norm = p.norm * (-1.0f);
    }

    //create basis vectors for plane (rotation axes)
    Vector3 crossvec;

    if (fabs(p.norm.Dot(Vector3(0,1,0)))>0.9f)
        crossvec=Vector3(1,0,0);
    else
        crossvec=Vector3(0,1,0);

    p.rotAxis1=p.norm.Cross(crossvec);
    p.rotAxis1.Normalize();
    p.rotAxis2=p.norm.Cross(p.rotAxis1);
    p.rotAxis2.Normalize();

}

float PlaneMesh::GetPlaneDistance(const Plane & p1, const Plane & p2) {

    float deltan1=acosf(p1.norm.Dot(p2.norm));
    float deltad1=fabs(p1.pt - p2.pt);

    float deltan2=acosf(p1.norm.Dot(p2.norm * -1.0f));
    float deltad2=fabs(p1.pt + p2.pt);

    return min(deltan1 + deltad1, deltan2 + deltad2);

}

float PlaneMesh::GetPlaneDistance(const Plane & p1, const QVector <Plane> & p2vec) {

    float minDist = FLT_MAX;

    for (int i=0; i<p2vec.size(); ++i) {
        float eachDist = GetPlaneDistance(p1, p2vec[i]);
        if (eachDist < minDist)
            minDist = eachDist;
    }

    if (minDist == FLT_MAX)
        qDebug() << "Distance comparison problem! " << p1.norm.x << p1.norm.y << p1.norm.z << p1.pt;

    return minDist;

}

Vector3 PlaneMesh::CubicHermiteInterp(const Vector3 & p1, const Vector3 & p2, const Vector3 & t1, const Vector3 & t2, const float t) {

    float tp2=t*t;
    float tp3=tp2*t;

    return p1*(2.0f*tp3-3.0f*tp2+1) + t1*(tp3-2.0f*tp2+t) + p2*(-2.0f*tp3+3*tp2) + t2*(tp3-tp2);

}

void PlaneMesh::SetCubicHermiteInterpSegmentChains(bool b) {
    m_bCubicHermiteSegmentInterp=b;
}

void PlaneMesh::DoCubicHermiteInterpForChains(MeshChain &chain1, MeshChain &chain2) {
    
    if (chain1.segments.empty())
        return;

    if (chain2.segments.empty())
        return;

    //add series of intermediate points based on cubic hermite interpolation
    MeshSegment & beginSeg=chain1.segments.last();
    MeshSegment & lastSeg=chain2.segments.first();

    Vector3 p1=beginSeg.pt2;
    Vector3 p2=lastSeg.pt1;
    Vector3 m1=beginSeg.pt2-beginSeg.pt1;    
    Vector3 m2=lastSeg.pt2-lastSeg.pt1;

    //don't normalize the tangent vectors... this gives the guy wolverine claws...
    //in fact it should probably be scaled based to be some factor of distance between beginseg.pt2 and lastseg.pt1

    float connectdist=(beginSeg.pt2-lastSeg.pt1).Length();

    m1.Normalize();
    m2.Normalize();
    m1=m1*(connectdist*m_fHermiteTanScaleFactor);
    m2=m2*(connectdist*m_fHermiteTanScaleFactor);

    /*
     * Interpolate along cubic hermite spline segment adding in intermediary segments between chains 1 and 2
     */

    MeshSegment eachsegment;
    eachsegment.n=Vector3(0,0,0);
    eachsegment.pt1=beginSeg.pt2;    

    float interpincr=0.1f;
    for (float interp = interpincr/2.0f; interp < 1.0f - interpincr; interp+=interpincr) {

        eachsegment.pt2=CubicHermiteInterp(p1,p2,m1,m2,interp);
        chain1.segments.push_back(eachsegment);
        eachsegment.pt1=eachsegment.pt2;

    }

    eachsegment.pt2=lastSeg.pt1;
    chain1.segments.push_back(eachsegment);
    
}

void PlaneMesh::SetCubicHermiteTanScaleFactor(float d) {

    m_fHermiteTanScaleFactor=d;

}

bool PlaneMesh::ExportToIllustratorAI(char * filename) {

    FILE * fp = fopen(filename,"w");

    if (!fp) {
        qDebug() << "Couldn't open " << filename << " to export contours as Illustrator AI.";
        return false;
    }

    fprintf(fp,"%%!PS-Adobe-3.0\n");
    fprintf(fp,"%%%%Creator: Rhinoceros\n");
    fprintf(fp,"%%%%Title: (%s)\n", filename);
    fprintf(fp,"%%%%BoundingBox: -23 -10 26 32\n");
    fprintf(fp,"%%%%DocumentProcessColors: Black\n");
    fprintf(fp,"%%%%DocumentNeededResources: procset Adobe_packedarray 2.0 0\n");
    fprintf(fp,"%%%%+ procset Adobe_cmykcolor 1.1 0\n");
    fprintf(fp,"%%%%+ procset Adobe_cshow 1.1 0\n");
    fprintf(fp,"%%%%+ procset Adobe_customcolor 1.0 0\n");
    fprintf(fp,"%%%%+ procset Adobe_typography_AI3 1.0 0\n");
    fprintf(fp,"%%%%+ procset Adobe_IllustratorA_AI3 1.0 0\n");
    fprintf(fp,"%%AI3_ColorUsage: Color\n");
    fprintf(fp,"%%AI3_TemplateBox: 288 384 288 384\n");
    fprintf(fp,"%%AI3_TileBox: 0 0 576 768\n");
    fprintf(fp,"%%AI3_DocumentPreview: None\n");
    fprintf(fp,"%%%%Template:\n");
    fprintf(fp,"%%%%PageOrigin:0 0\n");
    fprintf(fp,"%%%%EndComments\n");
    fprintf(fp,"%%%%BeginProlog\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_packedarray 2.0 0\n");
    fprintf(fp,"Adobe_packedarray /initialize get exec\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_cmykcolor 1.1 0\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_cshow 1.1 0\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_customcolor 1.0 0\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_typography_AI3 1.0 0\n");
    fprintf(fp,"%%%%IncludeResource: procset Adobe_IllustratorA_AI3 1.0 0\n");
    fprintf(fp,"%%%%EndProlog\n");
    fprintf(fp,"%%%%BeginSetup\n");
    fprintf(fp,"Adobe_cmykcolor /initialize get exec\n");
    fprintf(fp,"Adobe_cshow /initialize get exec\n");
    fprintf(fp,"Adobe_customcolor /initialize get exec\n");
    fprintf(fp,"Adobe_typography_AI3 /initialize get exec\n");
    fprintf(fp,"Adobe_IllustratorA_AI3 /initialize get exec\n");
    fprintf(fp,"%%%%EndSetup\n");
    fprintf(fp,"0 A\n");
    fprintf(fp,"0 R\n");
    fprintf(fp,"0 0 0 1 K\n");
    fprintf(fp,"0 i 0 J 0 j 1 w 4 M []0 d\n");
    fprintf(fp,"0 D\n");
    fprintf(fp,"1 1 1 1 0 0 -1 0 0 0 Lb\n");


    //Get all the projected chains, and their lo/hi bounds

    float curpagey = 0.0f;
    int curline = 0;

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        QVector <Vector2> lowerBounds;
        QVector <Vector2> upperBounds;
        Vector2 loBoundsAll, hiBoundsAll;
        QList <QList <Vector2> > projchains;

        GetChainsAs2DPoints(m_vPlanarCuts[i], projchains, lowerBounds, upperBounds, loBoundsAll, hiBoundsAll);

        float sizex = hiBoundsAll.x - loBoundsAll.x;
        float sizey = hiBoundsAll.y - loBoundsAll.y;

        for (int j=0; j<projchains.size(); ++j) {

            if (projchains[j].size() > 1) {

                fprintf(fp,"(%i) Ln\n", curline);
                fprintf(fp,"%f %f m\n", projchains[j].first().x - loBoundsAll.x, projchains[j].first().y - loBoundsAll.y + curpagey);

                for (int k=1; k<projchains[j].size(); ++k) {
                    fprintf(fp,"%f %f L\n", projchains[j][k].x - loBoundsAll.x, projchains[j][k].y - loBoundsAll.y + curpagey);
                }

                fprintf(fp,"S\n");

                ++curline;

            }

        }

        curpagey += sizey;

    }

    fprintf(fp,"LB\n");
    fprintf(fp,"%%%%PageTrailer\n");
    fprintf(fp,"gsave annotatepage grestore showpage\n");
    fprintf(fp,"%%%%Trailer\n");
    fprintf(fp,"Adobe_IllustratorA_AI3 /terminate get exec\n");
    fprintf(fp,"Adobe_typography_AI3 /terminate get exec\n");
    fprintf(fp,"Adobe_customcolor /terminate get exec\n");
    fprintf(fp,"Adobe_cshow /terminate get exec\n");
    fprintf(fp,"Adobe_cmykcolor /terminate get exec\n");
    fprintf(fp,"Adobe_packedarray /terminate get exec\n");
    fprintf(fp,"%%%%EOF\n");

    fclose(fp);

    return true;

}

bool PlaneMesh::ExportToSVG(char * filename) {

    FILE * fp = fopen(filename, "w");

    if (!fp) {
        qDebug() << "Couldn't open " << filename << " to export contours as SVG.";
        return false;
    }

    fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
    fprintf(fp, "<svg\n");
    fprintf(fp, "xmlns:svg=\"http://www.w3.org/2000/svg\"\n");
    fprintf(fp, "xmlns=\"http://www.w3.org/2000/svg\"\n");
    fprintf(fp, "version=\"1.0\"\n");
    fprintf(fp, "width=\"744.09448\"\n");
    fprintf(fp, "height=\"1052.3622\"\n");
    fprintf(fp, "id=\"svg2\">\n");
    fprintf(fp, "<defs\n");
    fprintf(fp, "id=\"defs4\" />\n");

    float curpagey = 0.0f;
    int curline = 0;

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        QVector <Vector2> lowerBounds;
        QVector <Vector2> upperBounds;
        Vector2 loBoundsAll, hiBoundsAll;
        QList <QList <Vector2> > projchains;

        GetChainsAs2DPoints(m_vPlanarCuts[i], projchains, lowerBounds, upperBounds, loBoundsAll, hiBoundsAll);

        float sizex = hiBoundsAll.x - loBoundsAll.x;
        float sizey = hiBoundsAll.y - loBoundsAll.y;

        Vector3 outColour = SetColorByIndex(i);

        for (int j=0; j<projchains.size(); ++j) {

            if (projchains[j].size() > 1) {

                fprintf(fp, "<g\n");
                fprintf(fp, "id=\"layer%i\">\n", curline);
                fprintf(fp, "<path\n");
                fprintf(fp, "d=\"");
                fprintf(fp, "M %f %f\n", (projchains[j].first().x - loBoundsAll.x) * 100.0f, (projchains[j].first().y - loBoundsAll.y + curpagey) * 100.0f);

                for (int k=1; k<projchains[j].size(); ++k) {
                    fprintf(fp, "L %f %f\n", (projchains[j][k].x - loBoundsAll.x) * 100.0f, (projchains[j][k].y - loBoundsAll.y + curpagey) * 100.0f);
                }

                if (m_vPlanarCuts[i].chains[j].isClosed) {
                    fprintf(fp, "Z \n"); //closes the chain
                }

                fprintf(fp, "\" id=\"path%i\"\n ");
                fprintf(fp,"style=\"fill:none;fill-rule:evenodd;stroke:#%02x%02x%02x;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1\" />\n",
                    (unsigned int)(outColour.x*255.0),(unsigned int)(outColour.y*255.0),(unsigned int)(outColour.z*255.0));

                fprintf(fp, "/>\n </g>\n", curline);

                ++curline;

            }

        }

        curpagey += sizey + 0.1f;

    }

    fprintf(fp, "</svg>\n");

    fclose(fp);

    return true;

}

bool PlaneMesh::ExportContours(char *filename) {

    FILE *f=fopen(filename,"w");

    if (f==NULL) {
        qDebug("Unable to write file %s.\n",filename);
        return false;
    }
    else {
        qDebug("Trying to write %s\n",filename);
    }

    QList <PlanarCut> &planecuts=m_vPlanarCuts;

    const float fSpaceFactor = 1000.0f; //scale vertex pos of model for ctr2suf

    fprintf(f,"%i\n",planecuts.size());

    for (int i=0;i<planecuts.size();++i) {

        fprintf(f,"%f %f %f %f\n",planecuts[i].params.norm.x,planecuts[i].params.norm.y,planecuts[i].params.norm.z,planecuts[i].params.pt);

        int totalverts=0;
        int totaledges=0;

        for (int j=0; j<planecuts[i].chains.size(); ++j) {

            MeshChain & chain = planecuts[i].chains[j];

            totalverts+=chain.segments.size();
            totaledges+=chain.segments.size();

        }

        fprintf(f,"%i %i\n",totalverts,totaledges);       

        for (int j=0; j<planecuts[i].chains.size(); ++j) {

            MeshChain & chain = planecuts[i].chains[j];

            for (int k=0; k<chain.segments.size(); ++k) {

                fprintf(f,"%f %f %f\n",
                        chain.segments[k].pt1.x * fSpaceFactor,
                        chain.segments[k].pt1.y * fSpaceFactor,
                        chain.segments[k].pt1.z * fSpaceFactor);
            }

        }

        int baseindex=0;
        for (int j=0; j<planecuts[i].chains.size(); ++j) {

            MeshChain & chain = planecuts[i].chains[j];

            //Calculate centroid
            Vector3 centroid=Vector3(0,0,0);
            float centroidnorm=0.0f;

            for (int k=0;k<chain.segments.size();k++) {

                Vector3 pt1=chain.segments[k].pt1;
                Vector3 pt2=chain.segments[k].pt2;

                float eachlen=(pt2-pt1).Length();
                centroid+=pt1*eachlen;
                centroidnorm+=eachlen;

            }
            centroid=centroid*(1.0f/centroidnorm);

            //Traverse along, doing inside/outside testing
            float rightisinside = 0.0f;

            for (int k=0; k<chain.segments.size()-1; ++k) {

                Vector3 pt1=chain.segments[k+1].pt1;
                Vector3 pt2=chain.segments[k].pt1;

                Vector3 rightVec=planecuts[i].params.norm.Cross(pt2-pt1);

                if ((centroid-pt1).Dot(rightVec)>0.0f)
                    rightisinside+=(pt2-pt1).Length();
                else
                    rightisinside-=(pt2-pt1).Length();
            }

            if (chain.isHole) //if hole, centroid is OUTSIDE of mesh, actually
                rightisinside=-rightisinside;

            //ok, begin writing chain!
            for (int k=0; k<chain.segments.size()-1; ++k) {

                if (rightisinside>0.0f)
                    fprintf(f,"%i %i 0 1\n",baseindex+k,baseindex+k+1);
                else
                    fprintf(f,"%i %i 1 0\n",baseindex+k,baseindex+k+1);
            }

            if (rightisinside>0.0f)
                fprintf(f,"%i %i 0 1\n",baseindex+chain.segments.size()-1,baseindex);
            else
                fprintf(f,"%i %i 1 0\n",baseindex+chain.segments.size()-1,baseindex);

            baseindex+=chain.segments.size();

        }

    }

    fclose(f);

    return true;

}

void PlaneMesh::SetCurvatureSmoothing(double d) {
    m_fCurvatureSmoothing=d;
}

void PlaneMesh::SetCurvatureThreshold(double d) {
    m_fRidgeValleyThreshold=d;
}

void PlaneMesh::SetUseMeshFeatures(bool *use) {

    for (int i=0;i<NUM_FEATURE_TYPES;++i)
        m_bUseFeature[i]=use[i];

}

void PlaneMesh::SetWeightMeshFeatures(float *weights) {

    for (int i=0;i<NUM_FEATURE_TYPES;++i)
        m_fWeights[i]=weights[i];

}

void PlaneMesh::SetRadiiMeshFeatures(float *radii) {

    for (int i=0;i<NUM_FEATURE_TYPES;++i)
        m_fRadii[i]=radii[i];    

}

void PlaneMesh::GetKeyPlanes_GlobalXYZ() {

    //Clear feature set of global xyz planes
    globalxyzPlanes.clear();

    //Return if global xyz planes aren't enabled
    if (!m_bUseFeature[P_GLOBALXYZ])
        return;

    //Otherwise, proceed

    //Go through mesh...  Get centroid
    QVector <Vector3> &verts=inputMesh->getVerts();

    Vector3 centroid=Vector3(0,0,0);

    for (int i=0;i<verts.size();++i)
        centroid+=Vector3(verts[i].x, verts[i].y, verts[i].z);

    float recip=1.0f/(float)verts.size();
    centroid=centroid*recip;   

    Plane xp, yp, zp;

    xp.norm=Vector3(1,0,0);
    xp.cent=centroid;
    xp.pt=xp.norm.Dot(xp.cent);
    xp.rotAxis1=Vector3(0,1,0);
    xp.rotAxis2=Vector3(0,0,1);

    yp.norm=Vector3(0,1,0);
    yp.cent=centroid;
    yp.pt=yp.norm.Dot(yp.cent);
    yp.rotAxis1=Vector3(0,0,1);
    yp.rotAxis2=Vector3(1,0,0);

    zp.norm=Vector3(0,0,1);
    zp.cent=centroid;
    zp.pt=zp.norm.Dot(zp.cent);
    zp.rotAxis1=Vector3(1,0,0);
    zp.rotAxis2=Vector3(0,1,0);

    globalxyzPlanes.push_back(xp);
    globalxyzPlanes.push_back(yp);
    globalxyzPlanes.push_back(zp);

}


void PlaneMesh::GetKeyPlanes_GlobalPCA() {

    //Clear feature set for global pca planes
    globalpcaPlanes.clear();

    //Return if feature not enabled
    if (!m_bUseFeature[P_GLOBALPCA])
        return;

    //Otherwise, proceed

    //Go through mesh...  Get centroid
    const QVector <Vector3> & verts=inputMesh->getVerts();

    Vector3 centroid=Vector3(0,0,0);

    for (int i=0; i<verts.size(); ++i)
        centroid+=Vector3(verts[i].x, verts[i].y, verts[i].z);

    float recip = 1.0f/(float)verts.size();
    centroid=centroid*recip;

    //Go through mesh again constructing covariance matrix
    float fSumXX=0;
    float fSumXY=0;
    float fSumXZ=0;

    float fSumYY=0;
    float fSumYZ=0;
    float fSumZZ=0;

    float maxdist=0.0f;

    for (int i=0; i<verts.size(); ++i) {

        Vector3 kDiff = Vector3(verts[i].x, verts[i].y, verts[i].z)-centroid;

        maxdist = max(kDiff.Length(), maxdist);

        fSumXX+= kDiff.x * kDiff.x; // sum of the squares of the differences.
        fSumXY+= kDiff.x * kDiff.y; // sum of the squares of the differences.
        fSumXZ+= kDiff.x * kDiff.z; // sum of the squares of the differences.

        fSumYY+= kDiff.y * kDiff.y;
        fSumYZ+= kDiff.y * kDiff.z;
        fSumZZ+= kDiff.z * kDiff.z;

    }

    fSumXX *= recip;
    fSumXY *= recip;
    fSumXZ *= recip;
    fSumYY *= recip;
    fSumYZ *= recip;
    fSumZZ *= recip;

    // setup the eigensolver
    EigenSolver kES;

    kES.mElement[0][0] = fSumXX;
    kES.mElement[0][1] = fSumXY;
    kES.mElement[0][2] = fSumXZ;

    kES.mElement[1][0] = fSumXY;
    kES.mElement[1][1] = fSumYY;
    kES.mElement[1][2] = fSumYZ;

    kES.mElement[2][0] = fSumXZ;
    kES.mElement[2][1] = fSumYZ;
    kES.mElement[2][2] = fSumZZ;

    // compute eigenstuff... we will return all 3 eigenvectors
    kES.DecrSortEigenStuff();   

    const float boundrad = inputMesh->GetBoundRad();

    for (int i=0; i<3; ++i) {

        Plane each;

        each.norm=Vector3(kES.mElement[0][i]*maxdist/boundrad,
                         kES.mElement[1][i]*maxdist/boundrad,
                         kES.mElement[2][i]*maxdist/boundrad);
        each.cent=centroid;
        each.pt=each.norm.Dot(each.cent);
        each.rotAxis1=Vector3(kES.mElement[0][(i+1)%3]*maxdist/boundrad,
                              kES.mElement[1][(i+1)%3]*maxdist/boundrad,
                              kES.mElement[2][(i+1)%3]*maxdist/boundrad);
        each.rotAxis2=Vector3(kES.mElement[0][(i+2)%3]*maxdist/boundrad,
                              kES.mElement[1][(i+2)%3]*maxdist/boundrad,
                              kES.mElement[2][(i+2)%3]*maxdist/boundrad);

        globalpcaPlanes.push_back(each);

    }

}

void PlaneMesh::SetSelectBinThreshold(float f) {

    m_BinSpace.SetSelectThreshold(f);

}

void PlaneMesh::PopulateBinSpaceWithFeatures() {

    // Populate bins with each feature
    qDebug("Populating binspace...");

    //for feature volumes figure:
    //m_BinSpace.AddCapsuleFeature(Vector3(-1,0,0),Vector3(-0.5,0,0),0.2,m_fRadii[P_PCA],0,P_PCA,m_fWeights[P_PCA]);
    //m_BinSpace.AddAxisFeature(Vector3(0,1,0),Vector3(-0.5,0,0),m_fRadii[A_PCA],0,A_PCA,m_fWeights[A_PCA]);
    //m_BinSpace.AddPointFeature(Vector3(-1,0,0),0.5,m_fRadii[P_GLOBALXYZ],0,P_GLOBALXYZ,m_fWeights[P_GLOBALXYZ]);
    //Vector3 anisNorm=Vector3(1,0,0); anisNorm.Normalize();
    //m_BinSpace.AddAnisotropicPointFeature(Vector3(-1,0,0),0.5,m_fRadii[P_VALLEY],0,P_VALLEY,m_fWeights[P_VALLEY],anisNorm);

    m_BinSpace.Clear();

    QVector <int> numFeatures = QVector <int> (10, 0);

    for (int i=0; i<meshFeatures.size(); ++i)
        ++numFeatures[meshFeatures[i].type];

    for (int i=0; i<numFeatures.size(); ++i)
        qDebug("Num of feature #%i: %i",i,numFeatures[i]);

    for (int i=0; i<meshFeatures.size(); ++i) {

        emit percentComplete(250 + int(float(i)/float(meshFeatures.size()) * 500.0f));

        if (i % 10 == 0) {            
            qDebug("Processing feature %i of %i...",i,meshFeatures.size());
        }

        Vector3 norm=meshFeatures[i].p.norm;
        norm.Normalize();
        float pt=norm.Dot(meshFeatures[i].p.cent);

        switch (meshFeatures[i].type) {

        case P_PCA:

            m_BinSpace.AddCapsuleFeature(meshFeatures[i].p.norm,
                                         meshFeatures[i].p.cent,
                                         meshFeatures[i].p.extent,
                                         m_fRadii[P_PCA],
                                         i,
                                         P_PCA,
                                         m_fWeights[P_PCA]);

            /*
            m_BinSpace.AddPointFeature(norm,
                                       pt,
                                       m_fRadii[P_PCA],
                                       i,
                                       P_PCA,
                                       m_fWeights[P_PCA]);
                                       */


            break;

        case A_PCA:

            m_BinSpace.AddAxisFeature(meshFeatures[i].a.dir,
                                      meshFeatures[i].a.p,
                                      m_fRadii[A_PCA],
                                      i,
                                      A_PCA,
                                      m_fWeights[A_PCA]);
            break;

        case A_RIDGE:
        case A_VALLEY:

            {

                QList <MeshSegment> * chain;
                Vector3 avgNorm;
                float avgCurv;
                float fLength;

                if (meshFeatures[i].type == A_RIDGE) {
                    chain = &ridgeAxesChains[meshFeatures[i].typeindex];
                    avgNorm = ridgeAxesChainsAvgNorm[meshFeatures[i].typeindex];
                    avgCurv = ridgeAxesChainsAvgCurv[meshFeatures[i].typeindex];
                    fLength = ridgeAxesChainsLength[meshFeatures[i].typeindex];
                }
                else {
                    chain = &valleyAxesChains[meshFeatures[i].typeindex];
                    avgNorm = valleyAxesChainsAvgNorm[meshFeatures[i].typeindex];
                    avgCurv = valleyAxesChainsAvgCurv[meshFeatures[i].typeindex];
                    fLength = valleyAxesChainsLength[meshFeatures[i].typeindex];
                }

                m_BinSpace.AddAxisFeature(meshFeatures[i].a.dir,
                                          meshFeatures[i].a.p,
                                          m_fRadii[meshFeatures[i].type],
                                          i,
                                          meshFeatures[i].type,
                                          m_fWeights[meshFeatures[i].type] * fLength);

                //qDebug() << "Adding ridge/valley with " << m_fWeights[meshFeatures[i].type] << fLength << avgCurv << m_fWeights[meshFeatures[i].type] * fLength * avgCurv;

                //feature should be anisotropic (i.e. care about avg normal direction)
                /*
                m_BinSpace.AddAnisotropicAxisFeature(meshFeatures[i].a.dir,
                                          meshFeatures[i].a.p,
                                          m_fRadii[meshFeatures[i].type],
                                          i,
                                          meshFeatures[i].type,
                                          m_fWeights[meshFeatures[i].type] * fLength,
                                          avgNorm);
                                          */

            }

            break;

        case P_SYM:

            m_BinSpace.AddPointFeature(norm,
                                       pt,
                                       m_fRadii[P_SYM],
                                       i,
                                       P_SYM,
                                       m_fWeights[P_SYM]);
            break;

        case A_SYM:

            m_BinSpace.AddAxisFeature(meshFeatures[i].a.dir,
                                      meshFeatures[i].a.p,
                                      m_fRadii[A_SYM],
                                      i,
                                      A_SYM,
                                      m_fWeights[A_SYM]);
            break;

        case P_GLOBALXYZ:

            /*
            m_BinSpace.AddPointFeature(norm,
                                       pt,
                                       m_fRadii[P_GLOBALXYZ],
                                       i,
                                       P_GLOBALXYZ,
                                       m_fWeights[P_GLOBALXYZ]);
                                       */
            m_BinSpace.AddCapsuleFeature(norm,
                                         Vector3(0,0,0),
                                         1.0f, // this changes a lot! was 0.1
                                         m_fRadii[P_GLOBALXYZ],
                                         i,
                                         P_GLOBALXYZ,
                                         m_fWeights[P_GLOBALXYZ]);

            break;

        case P_GLOBALPCA:

            m_BinSpace.AddCapsuleFeature(meshFeatures[i].p.norm,
                                         meshFeatures[i].p.cent,
                                         meshFeatures[i].p.norm.Length()*0.9f, // this changes a lot! was 0.1
                                         m_fRadii[P_GLOBALPCA],
                                         i,
                                         P_GLOBALPCA,
                                         m_fWeights[P_GLOBALPCA]);
            break;

        default:

            break;

        }

    }    

}

void PlaneMesh::AddPerpendicularSupport(const Plane & p, const PlaneSpaceBin & maxBin) {

    /*
    * If the cutplane introduced covered a SPINDLE-like feature (a PCA or symmetry axis), we introduce a new feature volume:
    *   PERPENDICULAR SUPPORT PLANE
    */
    if (m_bUseFeature[P_PERPSUPPORT]) {

        /*
        * Important note: we must ensure that the plane added is not ALREADY COVERING a perpendicular support, otherwise
        * we can potentially end up in an infinite loop where one support adds the other, which gets selected and introduces
        * the other, on and on...
        *
        * James Jan 2, 2011
        * Improving on the above, we make it so that an axis feature can only introduce one perp support, that makes the most sense.
        * that way if a plane goes up that's a perpendicular support, but covers some new uncovered axis, it won't get a support, and
        * that is not fair...
        */

        for (int i=0;i<maxBin.f.size();++i) {

            MeshFeatureType type=static_cast<MeshFeatureType>(maxBin.f[i].ftype);

            if ( type == A_PCA || type == A_SYM ) {

                if (meshFeatures[maxBin.f[i].fi].usedPerpSupport)
                    continue;

                // Compute parameters of a plane orthogonal to the one added, and that (roughly) intersects the axis
                Axis featPlane=meshFeatures[maxBin.f[i].fi].a;
                Vector3 dir = featPlane.dir;
                dir.Normalize();

                if ( fabs( dir.Dot(p.norm) ) > 0.1f )
                    continue;

                Plane newFeaturePlane;
                newFeaturePlane.norm=dir.Cross(p.norm);
                newFeaturePlane.norm.Normalize();
                newFeaturePlane.pt=newFeaturePlane.norm.Dot(featPlane.p);

                SetPlaneParamsFromNormPt(newFeaturePlane);

                // Add this feature to binspace
                m_BinSpace.AddPointFeature(newFeaturePlane.norm,
                                           newFeaturePlane.pt,
                                           m_fRadii[P_PERPSUPPORT],
                                           meshFeatures.size(),
                                           P_PERPSUPPORT,
                                           m_fWeights[P_PERPSUPPORT]);

                // Add this feature to the global list of features for the mesh
                MeshFeature newFeature;
                newFeature.p=newFeaturePlane;
                newFeature.type=P_PERPSUPPORT;
                newFeature.typeindex=perpSupportPlanes.size();

                perpSupportPlanes.push_back(newFeaturePlane);
                meshFeatures.push_back(newFeature);

                // This axis feature only gets to introduce a perp support ONE TIME
                meshFeatures[maxBin.f[i].fi].usedPerpSupport = true;

                //qDebug("Axis feature %i introduces perpendicular support to binspace at %f,%f,%f,%f",maxBin.f[i].fi,newFeaturePlane.norm.x,newFeaturePlane.norm.y,newFeaturePlane.norm.z,newFeaturePlane.pt);

            }

        }


    }

}

void PlaneMesh::AddSymmetricSupport(const Plane & p, const PlaneSpaceBin & maxBin) {

    /*
    * If the plane is not orthogonal to an existing symmetry plane of the mesh, we introduce a new feature volume:
    *   SYMMETRY SUPPORT PLANE
    */
    if (m_bUseFeature[P_SYMSUPPORT]) {

        /*
        * Important note: we must ensure that the plane added is not ALREADY COVERING a symmetry support, otherwise
        * we can potentially end up in an infinite loop where one support adds the other, which gets selected and introduces
        * the other, on and on...
        */

        for (int i=0; i<maxBin.f.size(); ++i) {

            if (maxBin.f[i].ftype == P_SYM || maxBin.f[i].ftype == P_SYMSUPPORT)
                return;

        }


        // We may introduce a symmetry support for ALL global symmetry plane features of the mesh
        for (int i = 0; i < meshFeatures.size(); ++i) {

            MeshFeatureType type = meshFeatures[i].type;

            if ( type == P_SYM ) {

                Plane symPlane=meshFeatures[i].p;

                // Skip if just added plane is "nearly" orthogonal (angle between norms > 80)
                const float angleThreshold = 85.0f/180.0f*M_PI;
                const float normDotProd = symPlane.norm.Dot(p.norm);
                const float normAngle = acosf( fabs(normDotProd) );
                if (normAngle > angleThreshold)
                    continue;

                // Compute parameters of plane that is mirrored across the symmetry plane              
                Plane newPlane;

                const Vector3 normProj = symPlane.norm * normDotProd;
                newPlane.norm = p.norm - normProj * 2.0f;
                newPlane.norm.Normalize();

                float centPlaneDist = symPlane.norm.Dot(p.norm * p.pt) - symPlane.pt;
                newPlane.cent = p.cent - symPlane.norm * centPlaneDist * 2.0f;
                newPlane.pt = newPlane.norm.Dot(newPlane.cent);

                SetPlaneParamsFromNormPt(newPlane);

                // Add this feature to binspace
                m_BinSpace.AddPointFeature(newPlane.norm,
                                           newPlane.pt,
                                           m_fRadii[P_SYMSUPPORT],
                                           meshFeatures.size(),
                                           P_SYMSUPPORT,
                                           m_fWeights[P_SYMSUPPORT]);

                // Add this feature to the global list of features for the mesh
                MeshFeature newFeature;
                newFeature.p=newPlane;
                newFeature.type=P_SYMSUPPORT;
                newFeature.typeindex=symSupportPlanes.size();

                symSupportPlanes.push_back(newPlane);
                meshFeatures.push_back(newFeature);


                qDebug("Adding symmetry support to binspace. Symmetry plane (%i): %f,%f,%f,%f Pairplane: %f,%f,%f, %f New plane: %f,%f,%f,%f",
                       i,
                       symPlane.norm.x,symPlane.norm.y,symPlane.norm.z,symPlane.pt,
                       p.norm.x,p.norm.y,p.norm.z,p.pt,
                       newPlane.norm.x,newPlane.norm.y,newPlane.norm.z,newPlane.pt);



            }

        }


    }

}

void PlaneMesh::GetSegmentsCovered(const PlanarCut & planarcut, QSet <int> & coveredSegments) {

    for (int i=0; i<planarcut.segments.size(); ++i)
        coveredSegments.insert(m_vMeshFaceSeg[planarcut.segments[i].face]);

}

void PlaneMesh::run() {

    DoBinningAlgorithm();

}

void PlaneMesh::DoBinningAlgorithm() {

    // Clear planes and bins
    ClearPlanes();
    m_vSelectedBins.clear();
    m_BinSpace.Clear();

    // Compute features for the mesh, incorporating GUI params
    ComputeMeshFeatures();   //25%

    // Populate the binspace
    PopulateBinSpaceWithFeatures(); //75%

    // Analyze how many human cuts fall into the feature volumes
    int nFeaturesIn = 0, nFeaturesOut = 0, nOutsideBinspace = 0;

    for (int i=0; i<figureCuts.size(); ++i) {

        for (int j=0; j<figureCuts[i].size(); ++j) {

            int theta, phi, r;

            m_BinSpace.CartesianToPolar(figureCuts[i][j].params.norm,figureCuts[i][j].params.pt,theta,phi,r);

            if (m_BinSpace.Empty(theta,phi,r))
                ++nFeaturesOut;
            else
                ++nFeaturesIn;

        }

    }

    qDebug("Human authored planes in feature volume: %i out: %i off mesh: %i total: %i",nFeaturesIn,nFeaturesOut,nOutsideBinspace,
           nFeaturesIn+nFeaturesOut+nOutsideBinspace);

    // Actually get around to choosing the planes from the binspace (choose bins above the threshold)

    const bool bDoLocalSearch = true;

    while ( true ) {

        // Get the currently maximal bin
        PlaneSpaceBin eachMaxBin = m_BinSpace.GetMaxValueBin(); //old algo
        //PlaneSpaceBin eachMaxBin = m_BinSpace.GetMaxValueBin_IncludeSelected(); //new algo

        // Add cutplane corresponding to this maximal bin
        //qDebug("\tMaximal bin: %i %i %i value %f, adding cutplane!",maxt,maxp,maxr,maxval);       

        PlanarCut bestPcut;

        bestPcut.params.norm=eachMaxBin.cartesianN;
        bestPcut.params.pt=eachMaxBin.cartesianD;       

        SetPlaneParamsFromNormPt(bestPcut.params);

        GetPlanarCut(bestPcut);

        // Before adding to cutset, find local optimal plane (but only when plane is not symmetry plane)

        if (bDoLocalSearch) {

            float searchSize = float(m_BinSpace.GetIncr()) * 0.5f;
            const float searchThresh = searchSize/32.0f;

            //max area algo
            //float searchSize = float(m_BinSpace.GetIncr()) * 50.0f;
            //const float searchThresh = float(m_BinSpace.GetIncr()) / 4.0f;

            for (int i=0; i<eachMaxBin.f.size(); ++i) {

                /*
                if ( eachMaxBin.f[i].ftype == P_SYM)
                    searchSize = 0.0f;
                else if (eachMaxBin.f[i].ftype == P_PERPSUPPORT  || eachMaxBin.f[i].ftype == P_SYMSUPPORT)
                    searchSize = qMin(searchSize, float(m_BinSpace.GetIncr()) * 0.55f);
                    */

                /*
                if ( eachMaxBin.f[i].ftype == P_SYM || eachMaxBin.f[i].ftype == A_SYM || eachMaxBin.f[i].ftype == P_SYMSUPPORT)
                    searchSize = 0.0f;
                else if (eachMaxBin.f[i].ftype == P_PERPSUPPORT)
                    searchSize = min(searchSize, float(m_BinSpace.GetIncr()) * 0.25f);
                    */

                if ( eachMaxBin.f[i].ftype == P_SYM || eachMaxBin.f[i].ftype == A_SYM)
                    searchSize = 0.0f;
                else if ( eachMaxBin.f[i].ftype == P_SYMSUPPORT || eachMaxBin.f[i].ftype == P_PERPSUPPORT)
                    searchSize = min(searchSize, float(m_BinSpace.GetIncr()) * 0.25f);

            }

            qDebug("Doing local search...");
            LocalSearchFindMaxAreaPlane(bestPcut, searchSize, searchThresh);
            qDebug("Done.");

        }


        //if the bin is not above our greedy threshold and...
        //  we (either have covered all segments or the maximal bin does nothing)
        if ( eachMaxBin.totalvalue < m_BinSpace.GetSelectThreshold() - 0.00001f) {

            if (m_bDoCoverAllSegments) {

                QSet <int> segmentsCovered;

                for (int i=0; i<m_vPlanarCuts.size(); ++i)
                    GetSegmentsCovered(m_vPlanarCuts[i], segmentsCovered);

                if ( segmentsCovered.size() == m_nSegments || eachMaxBin.totalvalue <= 0.0f )
                    break; // sole loop exit condition (but what about symmetric supports???)

                int numSegmentsBefore = segmentsCovered.size();
                GetSegmentsCovered(bestPcut, segmentsCovered);

                bool containsSymSupport = false;
                for (int i=0; i<eachMaxBin.f.size(); ++i) {
                    if (eachMaxBin.f[i].ftype == P_SYMSUPPORT)
                        containsSymSupport = true;
                }


                if (numSegmentsBefore == segmentsCovered.size() && !containsSymSupport) {

                    // Remove the currently maximal bin (don't want to keep choosing it)
                    m_BinSpace.RemoveFeaturesOfMaximalBin(eachMaxBin);

                    qDebug("Throwing away.");
                    continue;
                }
                else
                    qDebug("It covers one or more new segments, using anyway!!");

            }
            else
                break;

        }

        // use the cut
        AddPlanarCut(bestPcut);
        m_vSelectedBins.push_back(eachMaxBin);

        // Remove the currently maximal bin
        m_BinSpace.RemoveFeaturesOfMaximalBin(eachMaxBin);

        // Potentially add supports
        AddPerpendicularSupport(bestPcut.params, eachMaxBin);
        AddSymmetricSupport(bestPcut.params, eachMaxBin);

        emit percentComplete( qMin(990, 750 + m_vPlanarCuts.size() * 25));

    }   

    //have no plane selected by default
    selectedPlaneIndex=INT_MAX;

    emit percentComplete(1000);
    emit algoFinished();

}

void PlaneMesh::DrawBinSpace() {

    if ( selectedPlaneIndex >= m_vPlanarCuts.size() ) {

        m_BinSpace.SetDrawCursor(false);

    }
    else {

        int t, p, r;
        m_BinSpace.CartesianToPolar(m_vPlanarCuts[selectedPlaneIndex].params.norm,
                                    m_vPlanarCuts[selectedPlaneIndex].params.pt,
                                    t, p, r);
        m_BinSpace.SetCursorPos(t, p, r);
        m_BinSpace.SetDrawCursor(true);

    }

    m_BinSpace.Draw();

}

void PlaneMesh::ComputeMatrixA(QString matfilename) {

    m_BinSpace.Clear();
    m_vSelectedBins.clear();

    //binspace params
    for (int i=0; i<NUM_FEATURE_TYPES; ++i) {
        m_fWeights[i] = 1.0f;
        m_fRadii[i] = 0.1f;
        m_bUseFeature[i] = true;
    }

    // Compute features for the mesh, incorporating GUI params
    ComputeMeshFeatures();

    // Populate the binspace
    PopulateBinSpaceWithFeatures();

    ofstream ofs;
    ofs.open(matfilename.toAscii().data());

    ofs << "A = [ A; ";

    qDebug("'Positive' inequalities going out...");

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        QVector <float> rowOfA(NUM_FEATURE_TYPES, 0.0f);

        const Plane & p = m_vPlanarCuts[i].params;

        int theta, phi, r;
        m_BinSpace.CartesianToPolar(p.norm, p.pt, theta, phi, r);

        PlaneSpaceBin bin = m_BinSpace.GetBin(theta, phi, r);
        m_vSelectedBins.push_back(bin);

        for (int j=0; j<bin.f.size(); ++j)
            rowOfA[bin.f[j].ftype]+= bin.f[j].value;

        QString meshfile = QString(meshfilename);
        meshfile = meshfile.right(meshfile.size() - meshfile.lastIndexOf("/") - 1);
        meshfile = meshfile.left(meshfile.indexOf("."));

        ofs.setf(ios::fixed, ios::floatfield);
        ofs.precision(6);
        ofs << meshfile.toAscii().data() << " ";
        ofs << m_vPlanarCuts[i].fConfidence << " ";
        ofs << m_vPlanarCuts[i].nVotes << " ";
        ofs << bin.totalvalue << " ";
        ofs << bin.f.size() << " ";
        for (int j=0; j<rowOfA.size(); ++j)
            ofs << rowOfA[j] << " ";

        ofs << " ;" << std::endl;

        // Remove the currently maximal bin
        m_BinSpace.RemoveFeaturesOfMaximalBin(bin);

        // Potentially add supports
        AddPerpendicularSupport(p, bin);
        AddSymmetricSupport(p, bin);              

    }

    ofs << "];" << std::endl;

    qDebug("'Negative' inequalities going out...");

    ofs << "C = [ C; ";

    QVector <const PlaneSpaceBin *> bins = m_BinSpace.GetAllPopulatedBins();

    QVector <float> maxColumn(NUM_FEATURE_TYPES, 0.0f);

    for (int i=0; i<bins.size(); ++i) {

        QVector <float> row(NUM_FEATURE_TYPES, 0.0f);

        for (int j=0; j<bins[i]->f.size(); ++j) {

            //ignore supports
            if (bins[i]->f[j].ftype == P_SYMSUPPORT || bins[i]->f[j].ftype == P_PERPSUPPORT)
                continue;

            row[bins[i]->f[j].ftype]+= bins[i]->f[j].value;
        }

        int numNonZero = 0;
        for (int j=0; j<row.size(); ++j) {
            if (row[j] > 0.0f)
                ++numNonZero;
        }

        if (numNonZero == 1) {

            for (int j=0; j<row.size(); ++j) {

                if (row[j] > 0.0f)
                    maxColumn[j] = qMax(row[j], maxColumn[j]);
            }

        }

    }

    for (int i=0; i<maxColumn.size(); ++i) {

        if (maxColumn[i] > 0.0f) {

            for (int j=0; j<maxColumn.size(); ++j) {
                if (i==j)
                     ofs << maxColumn[i] << " ";
                else
                    ofs << 0.0f << " ";
            }
            ofs << " ;" << std::endl;

        }

    }

    for (int i=0; i<bins.size(); ++i) {

        QVector <float> row(NUM_FEATURE_TYPES, 0.0f);

        for (int j=0; j<bins[i]->f.size(); ++j)
            row[bins[i]->f[j].ftype]+= bins[i]->f[j].value;

        int numNonZero = 0;
        for (int j=0; j<row.size(); ++j) {
            if (row[j] > 0.0f)
                ++numNonZero;
        }

        if (numNonZero > 1) {

            for (int j=0; j<row.size(); ++j)
                ofs << row[j] << " ";

            ofs << " ;" << std::endl;

        }

    }

    ofs << "];" << std::endl;

    ofs.close();

    qDebug("Done.");


}

void PlaneMesh::DoAnalysisOnUserData() {

    static int nPlanes = 0;
    static int nNoFeaturePlanesAllModels = 0;

    int nPlanesThisModel = 0;
    int nNoFeaturePlanesThisModel = 0;

    m_BinSpace.Clear();
    ClearPlanes();

    // Compute features for the mesh, incorporating GUI params
    ComputeMeshFeatures();
    PopulateBinSpaceWithFeatures();

    const BinSpace binSpaceFeat = m_BinSpace;

    QVector <int> featureCounts(NUM_FEATURE_TYPES, 0);

    for (int i=0; i<figureCuts.size(); ++i) {

        m_BinSpace = binSpaceFeat;

        for (int j=0; j<figureCuts[i].size(); ++j) {

            ++nPlanes;
            ++nPlanesThisModel;

            const Plane & p = figureCuts[i][j].params;

            int theta, phi, r;
            m_BinSpace.CartesianToPolar(p.norm, p.pt, theta, phi, r);

            const PlaneSpaceBin bin = m_BinSpace.GetBin(theta, phi, r);

            if (bin.f.empty()) {
                ++nNoFeaturePlanesAllModels;
                ++nNoFeaturePlanesThisModel;
            }

            QVector <bool> myFeatCounts(NUM_FEATURE_TYPES, false);
            for (int k=0; k<bin.f.size(); ++k) {
                myFeatCounts[bin.f[k].ftype] = true;
            }

            for (int k=0; k<featureCounts.size(); ++k) {
                if (myFeatCounts[k]) {
                    ++featureCounts[k];
                }
            }

            //add this cut to the space
            PlanarCut newCut;
            newCut.params.norm=bin.cartesianN;
            newCut.params.pt=bin.cartesianD;

            SetPlaneParamsFromNormPt(newCut.params);

            // Potentially add supports
            AddPerpendicularSupport(newCut.params, bin);
            AddSymmetricSupport(newCut.params, bin);

            /*
            if (bin.f.size() > 0) {

                figureCuts[i].removeAt(j);
                --j;

            }
            else {

                ++nNoFeaturePlanesAllModels;
                ++nNoFeaturePlanesThisModel;

                //let's add a cutplane at the NEAREST POPULATED FEATURE POINT
                float nearDist;
                PlaneSpaceBin nearBin = m_BinSpace.GetNearestPopulatedBin(theta, phi, r, nearDist);

                qDebug() << "Model " << meshfilename << " User " << i << " Cut " << j << " has no feature coverage."
                        << "  Nearest feature is " << featureTypeStrings[nearBin.f.first().ftype] << ", distance " << nearDist << ".";

                PlanarCut newCut;
                newCut.params.norm=nearBin.cartesianN;
                newCut.params.pt=nearBin.cartesianD;

                SetPlaneParamsFromNormPt(newCut.params);

                GetPlanarCut(newCut);
                AddPlanarCut(newCut);

            }
            */

        }
    }

    for (int i=0; i<NUM_FEATURE_TYPES; ++i)
        qDebug() << featureCounts[i];

    qDebug() << nNoFeaturePlanesThisModel;
    qDebug() << nPlanesThisModel;

    qDebug() << "For model " << meshfilename << ", " << nNoFeaturePlanesThisModel << " of " << nPlanesThisModel << " have no feature coverage.";
    qDebug() << "Across all models " << nNoFeaturePlanesAllModels << " of " << nPlanes << " have no feature coverage.";

    selectedPlaneIndex = INT_MAX;

}

void PlaneMesh::AddNewAnnotationPatch() {

    if (selectedPlaneIndex < 0 || selectedPlaneIndex >= m_vPlanarCuts.size())
        return;

    if (!m_vPlanarCuts[selectedPlaneIndex].annotatePatch.empty() && !m_vPlanarCuts[selectedPlaneIndex].annotatePatch.last().empty()) {

        QList <QList <Vector3> > & patch = m_vPlanarCuts[selectedPlaneIndex].annotatePatch.last();

        //calculate rotation midpoint for current patch
        float l1 = FLT_MAX, l2 = FLT_MAX;
        float h1 = -FLT_MAX, h2 = -FLT_MAX;

        Plane & params = m_vPlanarCuts[selectedPlaneIndex].params;

        //(first find bbox wrt rotaxis of plane)

        for (int i=0; i<patch.size(); ++i) {
            for (int j=0; j<patch[i].size(); ++j) {

                Vector3 v = patch[i][j] - params.cent;

                l1 = qMin(patch[i][j].x, l1);
                l2 = qMin(patch[i][j].y, l2);

                h1 = qMax(patch[i][j].x, h1);
                h2 = qMax(patch[i][j].y, h2);

            }
        }

        //midpt is just halfway between wrt each axis
        Vector3 midPt;

        if (!patch.empty()) {

            float dir1 = ((l1 + h1) / 2.0f);
            float dir2 = ((l2 + h2) / 2.0f);
            midPt = params.cent + params.rotAxis1 * dir1 + params.rotAxis2 * dir2 ;

            //set midpoint
            for (int i=0; i<patch.size(); ++i) {
                for (int j=0; j<patch[i].size(); ++j) {

                    patch[i][j].x-=dir1;
                    patch[i][j].y-=dir2;

                }
            }

        }
        else
            midPt = Vector3(0,0,0);

        //midPt = Vector3(0,0,0);

        m_vPlanarCuts[selectedPlaneIndex].annotatePatchMidpoint.push_back(midPt);

    }
    else {
        m_vPlanarCuts[selectedPlaneIndex].annotatePatchMidpoint.push_back(Vector3(0,0,0));
    }

    //now add new patch (and params)
    QList <QList <Vector3> > newPatch;
    m_vPlanarCuts[selectedPlaneIndex].annotatePatch.push_back(newPatch);

}

void PlaneMesh::AddToAnnotatePlane(int x, int y, bool newStroke) {

    //use gluunproject to generate outward vector
    glGetIntegerv(GL_VIEWPORT, origViewport);
    glGetDoublev( GL_PROJECTION_MATRIX, origProj );
    glGetDoublev( GL_MODELVIEW_MATRIX, origModl );

    double px, py, pz;
    gluUnProject(x, origViewport[3] - y, 0.5, origModl, origProj, origViewport, &px, &py, &pz);

    //find intersection point with selectedplane
    Vector3 rayDir = Vector3(px, py, pz) - camera->GetEye();
    rayDir.Normalize();

    Vector3 intersectPt;
    Plane & p = m_vPlanarCuts[selectedPlaneIndex].params;
    bool bIntersect = GetRayPlaneIntersect(p, camera->GetEye(), rayDir, intersectPt);

    if (!bIntersect)
        return;

    Vector3 planePt;

    planePt.x = (intersectPt - p.cent).Dot(p.rotAxis1);
    planePt.y = (intersectPt - p.cent).Dot(p.rotAxis2);
    planePt.z = 0.0f;

    //add a point to plane's last curve in annotationlist
    if (m_vPlanarCuts[selectedPlaneIndex].annotatePatch.empty()) {
        QList <QList <Vector3> > patch;
        m_vPlanarCuts[selectedPlaneIndex].annotatePatch.push_back(patch);
    }

    if (newStroke) {
        QList <Vector3> stroke;
        stroke.push_back(planePt);

        m_vPlanarCuts[selectedPlaneIndex].annotatePatch.last().push_back(stroke);
    }
    else
        m_vPlanarCuts[selectedPlaneIndex].annotatePatch.last().last().push_back(planePt);

    //qDebug("Added to stroke %i pt %f,%f,%f", m_vPlanarCuts[selectedPlaneIndex].annotations.size() - 1, intersectPt.x, intersectPt.y, intersectPt.z);

}

void PlaneMesh::SelectAnnotatePlane(int, int) {   

    //set selectedplaneindex incorporating camera parameters (dot product) as well as projected distance
    float maxPlaneVal = -FLT_MAX;
    int maxPlaneIndex = 0;

    for (int i = 0; i < m_vPlanarCuts.size(); ++i) {                        

        Plane & p = m_vPlanarCuts[i].params;

        //rotate all norms ABOUT symmetry axis to face camEye
        //TODO: make enable/disable
        if (!symLines.empty()) {

            Vector3 aP = symLines.first().p;
            Vector3 aDir = symLines.first().dir;
            aDir.Normalize();

            Vector3 normEye = camera->GetEye();
            normEye.Normalize();

            float dotProd =aDir.Dot(p.norm);

            if (fabs(aDir.Dot(normEye)) < 0.99f) {

                Vector3 rotAxis = aDir.Cross(normEye);
                rotAxis.Normalize();

                if (fabs(dotProd) < 0.99f) {

                    p.norm = GetRotatedAxis(acosf(dotProd), aDir, rotAxis);
                    p.norm.Normalize();

                    SetPlaneParamsFromNormPt(p);

                }
                else {

                    float dotProd2 = aDir.Dot(p.rotAxis1);

                    p.rotAxis1 = GetRotatedAxis(acosf(dotProd2), aDir, rotAxis);
                    p.rotAxis1.Normalize();
                    p.rotAxis2 = p.norm.Cross(p.rotAxis1);
                    p.rotAxis2.Normalize();

                }

            }

        }

        Vector3 camPosProjNorm = p.norm * p.norm.Dot(camera->GetEye());
        Vector3 camViewDir = camera->GetCent() - camera->GetEye();
        camViewDir.Normalize();

        //check if viewing AWAY from plane
        //if (camViewDir.Dot ( camPosProjNorm ) > 0.0f)
        //    continue;

        float camDotProduct = fabs( p.norm.Dot( camViewDir ) );

        float camDist = fabs(p.norm.Dot( camera->GetEye() ) - p.pt);

        //float eachVal = (camDotProduct - minAngle) * (1.0f / (camDist + 1.0f));
        float eachVal = camDotProduct * (100.0f - camDist);

        if (eachVal > maxPlaneVal) {
            maxPlaneVal = eachVal;
            maxPlaneIndex = i;
        }

    }

    selectedPlaneIndex = maxPlaneIndex;

}

void PlaneMesh::DrawAnnotations() {   

    glDisable(GL_LIGHTING);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glLineWidth(3.0f);

    const float fMinAngle = cosf(75.0f / 180.0f * M_PI);

    for (int i = 0; i < m_vPlanarCuts.size(); ++i) {

        Plane & p = m_vPlanarCuts[i].params;

        Vector3 camViewDir = camera->GetEye() - p.cent;
        camViewDir.Normalize();

        float camDotProduct = fabs( p.norm.Dot( camViewDir ) ) - fMinAngle;
        //glColor4f(0, 0, 0, camDotProduct);
        glColor4f(0, 0, 0, 1);

        //if (camDotProduct < 0.0f)
        //    continue;

        for (int j = 0; j< m_vPlanarCuts[i].annotatePatch.size(); ++j) {

            //pick midpoint and axes for this patch (is the patch done?)
            Vector3 midPt;
            Vector3 axis1;
            Vector3 axis2;

            if (j < m_vPlanarCuts[i].annotatePatchMidpoint.size()) {

                midPt = m_vPlanarCuts[i].annotatePatchMidpoint[j];

                //axes are defined by some rotation that makes rotaxes of plane face camera maximally
                /*
                float m[16];

                glGetFloatv(GL_MODELVIEW_MATRIX, m);

                Vector3 right = Vector3(m[0], m[4], m[8]);
                Vector3 up = Vector3(m[1], m[5], m[9]);

                //rotate axes properly (reproject)
                for (int i=0; i<patch.size(); ++i) {
                    for (int j=0; j<patch[i].size(); ++j) {

                        Vector3 tempPatch = params.rotAxis1 * patch[i][j].x + params.rotAxis2 * patch[i][j].y;

                        patch[i][j].x = tempPatch.Dot(right);
                        patch[i][j].y = tempPatch.Dot(up);

                    }
                }

                //let's do a projection

                axis1.x = m[0]; //right vector
                axis1.y = m[4];
                axis1.z = m[8];

                axis2.x = m[1]; //up vector
                axis2.y = m[5];
                axis2.z = m[9];
                */

                float m[16];

                glGetFloatv(GL_MODELVIEW_MATRIX, m);

                Vector3 right = Vector3(m[0], m[4], m[8]);
                Vector3 up = Vector3(m[1], m[5], m[9]);

                if (fabs(right.Dot(p.rotAxis1)) < fabs(right.Dot(p.rotAxis2))) { //flip axes

                    if (right.Dot(p.rotAxis2) >= 0.0f)
                        axis2 = right;
                    else
                        axis2 = right * (-1.0f);

                    if (up.Dot(p.rotAxis1) >= 0.0f)
                        axis1 = up;
                    else
                        axis1 = up * (-1.0f);

                }
                else { //don't flip
                    if (right.Dot(p.rotAxis1) >= 0.0f)
                        axis1 = right;
                    else
                        axis1 = right * (-1.0f);

                    if (up.Dot(p.rotAxis2) >= 0.0f)
                        axis2 = up;
                    else
                        axis2 = up * (-1.0f);
                }

                /* don't use billboards
                midPt = p.cent;
                axis1 = p.rotAxis1;
                axis2 = p.rotAxis2;
                */

            }
            else {

                midPt = p.cent;
                axis1 = p.rotAxis1;
                axis2 = p.rotAxis2;

            }

            //qDebug() << "midPt of stroke " << j << " is " << midPt.x << " " << midPt.y << " " << midPt.z;

            for (int k = 0; k < m_vPlanarCuts[i].annotatePatch[j].size(); ++k) {

                glBegin(GL_LINE_STRIP);
                for (int l = 0; l< m_vPlanarCuts[i].annotatePatch[j][k].size(); ++l) {

                    Vector3 & pt = m_vPlanarCuts[i].annotatePatch[j][k][l];

                    Vector3 drawPt = midPt + axis1 * pt.x + axis2 * pt.y;
                    glVertex3f(drawPt.x, drawPt.y, drawPt.z);

                }
                glEnd();

            }
        }

    }

    glDisable(GL_BLEND);
    glColor3f(1, 1, 1);
    glLineWidth(1.0f);

}

void PlaneMesh::MeshDecimate() {

    if (inputMesh)
        inputMesh->Decimate(m_vMeshFaceSeg);

    MeshUpdateTrimesh();

}

void PlaneMesh::MeshNoisify() {

    if (inputMesh) {
        inputMesh->Noisify();
    }

    MeshUpdateTrimesh();

}

void PlaneMesh::MeshUpdateTrimesh() {

    QVector <Vector3> & mVerts = inputMesh->getVerts();
    QVector <QVector <int> > & mFaces = inputMesh->getFaces();

    themesh->vertices.clear();
    for (int i=0; i<mVerts.size(); ++i) {

        point p;
        p[0] = mVerts[i].x;
        p[1] = mVerts[i].y;
        p[2] = mVerts[i].z;

        themesh->vertices.push_back(p);

    }

    themesh->faces.clear();
    for (int i=0; i<mFaces.size(); ++i) {

        TriMesh::Face f(mFaces[i][0], mFaces[i][1], mFaces[i][2]);

        themesh->faces.push_back(f);

    }

}

void PlaneMesh::UpdateBinSpaceDispList() {

    m_BinSpace.UpdateDispList();

}

void PlaneMesh::SetCoverAllSegments(bool b) {

    m_bDoCoverAllSegments = b;

}

void PlaneMesh::CutsSegmentify() {   

    int nSegments = 0;

    for (int i=0; i<m_vMeshFaceSeg.size(); ++i) {
        nSegments = qMax(nSegments, m_vMeshFaceSeg[i]);
    }

    ++nSegments;

    m_bUseFaceSeg = true;

    QList <PlanarCut> cuts;   

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        qDebug() << "Working cut " << i << " across segments.";

        for (int j=0; j<nSegments; ++j) {

            PlanarCut cut;
            cut.params = m_vPlanarCuts[i].params;
            cut.nearSeg = j;

            GetPlanarCut(cut);

            if (!cut.tris.empty())
                cuts.push_back(cut);
        }

    }

    m_vPlanarCuts.clear();
    m_vPlanarCuts = cuts;

    selectedPlaneIndex = INT_MAX;

}

void PlaneMesh::AddRandomCuts() {

    ClearPlanes();

    qsrand(time(0));

    for (int i=0; i<5; ++i) {

        PlanarCut p;

        while (p.area <= 0.05f) {

            float randx = double(qrand()) / double(RAND_MAX) * 2.0f - 1.0f;
            float randy = double(qrand()) / double(RAND_MAX) * 2.0f - 1.0f;
            float randz = double(qrand()) / double(RAND_MAX) * 2.0f - 1.0f;

            float randd = double(qrand()) / double(RAND_MAX) * 2.0f - 1.0f;

            p.params.norm = Vector3(randx, randy, randz);
            p.params.norm.Normalize();
            p.params.pt = randd;

            qDebug() << p.params.norm.x << p.params.norm.y << p.params.norm.z << p.params.pt;

            SetPlaneParamsFromNormPt(p.params);

            GetPlanarCut(p);

        }

        AddPlanarCut(p);

    }

}

void PlaneMesh::AddPCAPlanes() {

    ClearPlanes();

    GetKeyPlanes_GlobalPCA();

    for (int i=0; i<globalpcaPlanes.size(); ++i) {

        PlanarCut p;

        p.params = globalpcaPlanes[i];

        SetPlaneParamsFromNormPt(p.params);
        GetPlanarCut(p);
        AddPlanarCut(p);

    }


}

void PlaneMesh::GetClosestCurvePointForNav(const Vector3 pt, float & closestLen, Vector3 & closestPt) {

    closestLen = FLT_MAX;

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        for (int j=0; j<m_vPlanarCuts[i].chains.size(); ++j) {

            MeshChain & chain = m_vPlanarCuts[i].chains[j];

            for (int k=0; k<m_vPlanarCuts[i].chains[j].segments.size(); ++k) {

                float len = (chain.segments[k].pt1 - pt).LengthSquared();

                if (len < closestLen) {
                    closestLen = len;
                    closestPt = chain.segments[k].pt1;
                }

            }

        }

    }

    if (closestLen == FLT_MAX) {
        closestPt = Vector3(0,0,0);
        closestLen = pt.LengthSquared();
    }

    closestLen = sqrtf(closestLen);

}


QList <int> PlaneMesh::GetRandomSet(const int lo, const int hi, const int size)
{

    QList <int> s;
    for (int i=lo; i<=hi; ++i)
        s.push_back(i);

    QList <int> s2;
    for (int i=0; i<size; ++i) {
        int r = qrand() % s.size();
        s2.push_back(s[r]);
        s.removeAt(r);
    }

    return s2;

}

QVector <Plane> PlaneMesh::LoadUserPlanes(const int uNum, const int mNum)
{

    QVector <Plane> pvec;
    QFile inFile(GetUserModelFilename(uNum, mNum));

    if (!inFile.open( QIODevice::ReadOnly | QIODevice::Text )) {
        qDebug() << "File " << GetUserModelFilename(uNum, mNum) << " doesn't exist.";
        return pvec;
    }

    QTextStream ifs( &inFile );

    //read # of planes for user's markup of model
    QStringList line = ifs.readLine().split(" ");
    int num = line[1].toInt();

    //read past param list (nx, ny, ...
    ifs.readLine();

    //add plane parameters of each user plane to vector
    for (int j=0; j<num; ++j) {

        line = ifs.readLine().split(" ");

        Plane p;

        p.norm=Vector3(line[1].toFloat(),line[2].toFloat(),line[3].toFloat());
        p.cent=Vector3(line[4].toFloat(),line[5].toFloat(),line[6].toFloat());
        p.pt=p.norm.Dot(p.cent);
        p.rotAxis1=Vector3(line[8].toFloat(),line[9].toFloat(),line[10].toFloat());
        p.rotAxis2=Vector3(line[11].toFloat(),line[12].toFloat(),line[13].toFloat());

        pvec.push_back(p);

    }

    //close file
    inFile.close();

    return pvec;

}

QVector <Plane> PlaneMesh::ComputeSuperproxy(const QVector <QVector <Plane> > & pvecs)
{

    //we bin each user's plane
    m_BinSpace.Clear();

    int curFeatIndex=0;
    QMap <int, int> featToUserMap;
    QMap <int, int> featToUsersCutMap;

    for (int i=0; i < pvecs.size(); ++i) {
        for (int j=0; j < pvecs[i].size(); ++j) {

            const float fGroupDist = 0.08f;
            m_BinSpace.AddPointFeature(pvecs[i][j].norm, pvecs[i][j].pt, fGroupDist, curFeatIndex, i, 1.0f);

            featToUserMap[curFeatIndex]=i;
            featToUsersCutMap[curFeatIndex]=j;

            curFeatIndex++;

        }
    }

    /*
     * This algo chooses amongst the groups of planes ones that are
     * clustered together/well represented
     */

    float magicRatioThreshold = 2.75f / 12.0f * pvecs.size();

    QVector <Plane> proxy;

    while (true) {

        //eachMaxBin = m_BinSpace.GetMaxFeaturesBin();
        //if (eachMaxBin.f.size() < 4)
        PlaneSpaceBin eachMaxBin = m_BinSpace.GetMaxValueBin();
        if (eachMaxBin.totalvalue < magicRatioThreshold && !proxy.empty())
            break;

        Plane p;
        p.norm=eachMaxBin.cartesianN;
        p.pt=0.0f;

        for (int j=0; j<eachMaxBin.f.size(); ++j) {

            int featInd = eachMaxBin.f[j].fi;
            int uInd = featToUserMap[featInd];
            int cInd = featToUsersCutMap[featInd];

            Q_ASSERT ( uInd < pvecs.size() );
            Q_ASSERT ( cInd < pvecs[uInd].size() );

            if ( p.norm.Dot(pvecs[uInd][cInd].norm) > 0.0f )
                p.pt+=pvecs[uInd][cInd].pt;
            else
                p.pt-=pvecs[uInd][cInd].pt;

        }

        p.pt/=float(eachMaxBin.f.size());
        SetPlaneParamsFromNormPt(p);

        //float fRad = float(m_BinSpace.GetIncr()) * 0.5f;
        //LocalSearchFindMaxAreaPlane(bestPcut, fRad, fRad/32.0f);

        m_BinSpace.RemoveFeaturesOfMaximalBin(eachMaxBin);

        proxy.push_back(p);

    }

    return proxy;

}


/*
 * Purpose: show that a 12 user study was enough, since the superproxy tends to be stable after that.
 */
void PlaneMesh::DoUserStudyAnalysis() {

    //pseudocode:

    //iterate through every n = 1 to number of good (non-throwaway) users in study
        //iterate through every model in user study sequence
        //for each model:
            //for numerous times (X=10) we want to do the following:
                //compute superproxy S1 for random subset of users of size n
                //compute superproxy S2 for random subset of users of size n
                //calculate "difference" between superproxies (symmetrically):
                    //for each p1 in S1
                        //add distance to nearest p2 in S2 to total distance
                    //for each p2 in S2
                        //add distance to nearest p1 in S1 to total distance
                    //divide total distance by number of planes in S1+S2
                    //add to avg distance, scaled by 1/X
            //add average difference to the total differences
        //divide total differences by number of models (19)

    const int num_models = 19;
    const int meshes[num_models] = {2, 21, 22, 24, 25, 30, 62, 63, 64, 65, 66, 105, 141, 142, 143, 144, 145, 342, 385};

    //iterate through every n = 1 to number of good (non-throwaway) users in study
    for (int i=1; i<NUM_HUMAN_DATASETS; ++i) {

        //iterate through every model in user study sequence
        float totaldiff = 0.0f;
        for (int j=0; j<num_models; ++j) {

            //for numerous times (X=10) we want to do the following:
            const int NUMTIMESPERMODEL = 10;
            float modeldiff = 0.0f;
            for (int x=0; x<NUMTIMESPERMODEL; ++x) {

                //compute superproxy S1 for random subset of users of size n
                QList <int> set1 = GetRandomSet(1, NUM_HUMAN_DATASETS, i);

                QVector <QVector <Plane> > set1planes;
                for (int k=0; k<set1.size(); ++k)
                    set1planes.push_back(LoadUserPlanes(set1[k], meshes[j]));

                QVector <Plane> proxy1 = ComputeSuperproxy(set1planes);

                //compute superproxy S2 for random subset of users of size n
                QList <int> set2 = GetRandomSet(1, NUM_HUMAN_DATASETS, i);

                QVector <QVector <Plane> > set2planes;
                for (int k=0; k<set2.size(); ++k)
                    set2planes.push_back(LoadUserPlanes(set2[k], meshes[j]));

                QVector <Plane> proxy2 = ComputeSuperproxy(set2planes);

                //calculate "difference" between superproxies (symmetrically):
                float eachdiff = 0.0f;

                //for each p1 in S1, add mindist to a plane in S2
                for (int k=0; k<proxy1.size(); ++k)
                    eachdiff += GetPlaneDistance(proxy1[k], proxy2);

                //for each p2 in S2, add mindist to a plane in S1
                for (int k=0; k<proxy2.size(); ++k)
                    eachdiff += GetPlaneDistance(proxy2[k], proxy1);

                //divide total distance by number of planes in S1+S2
                eachdiff /= float(proxy1.size() + proxy2.size());

                //add to avg distance, scaled by 1/X
                modeldiff += eachdiff / float(NUMTIMESPERMODEL);

                //qDebug() << modeldiff << proxy1.size() << proxy2.size() << set1 << set2;

            }

            qDebug() << modeldiff;

            //add calculated difference to total differences
            totaldiff += modeldiff;

        }

        //divide total differences by number of models (19)
        totaldiff /= float(num_models);

        qDebug() << totaldiff << " mean for set size " << i;

    }

}

void PlaneMesh::SetAestheticCuts(bool b, float f)
{
    m_baesthetic = b;
    m_baestheticthresh = f;
}

void PlaneMesh::ComputePaperPuppetStuff()
{

    qDebug() << "Computing paper puppet stuff!!!!!!";
    //need to compute pairplane intersections, to define lines to cut at for puppet
    m_puppetcuts.clear();
    m_puppetcuts.resize(m_vPlanarCuts.size());

    for (int i=0; i<m_vPlanarCuts.size(); ++i) {

        for (int j=i+1; j<m_vPlanarCuts.size(); ++j) {

            //intersect planes i and j to find ray (no intersection may be possible if planes parallel)
            Vector3 ipt;
            Vector3 idir;

            bool intersect = GetPlanePlaneIntersect(m_vPlanarCuts[i].params, m_vPlanarCuts[j].params, ipt, idir);
            if (!intersect)
                continue;

            //find hit points of ray with mesh
            QVector <Vector3> & verts = inputMesh->getVerts();
            QVector <QVector <int> > & faces = inputMesh->getFaces();
            QVector <Vector3> intpts;
            for (int k=0; k<faces.size(); ++k) {

                Triangle t;
                t.p1 = verts[faces[k][0]];
                t.p2 = verts[faces[k][1]];
                t.p3 = verts[faces[k][2]];

                Vector3 intpt;
                if (GetRayTriIntersect(ipt, idir, t, intpt)) {
                    intpts.push_back(intpt);
                }

            }

            //create a midpoint at each interior interval
            //sort first... assuming closed surface, there should be 2 * n intersections
            for (int k=0; k<intpts.size()-1; ++k) {
                for (int l=k+1; l<intpts.size(); ++l) {

                    float dot1 = (intpts[k]-ipt).Dot(idir);
                    float dot2 = (intpts[l]-ipt).Dot(idir);

                    if (dot2 < dot1) {
                        Vector3 temp = intpts[k];
                        intpts[k] = intpts[l];
                        intpts[l] = temp;
                    }

                }
            }

            //for each intersect pair p1, p2, make line for plane i from p1 to midpt, for plane j from midpt to p2
            //(they both perform half the cut)
            for (int k=0; k<intpts.size(); k+=2) {

                Vector3 p1 = intpts[k];
                Vector3 p2 = intpts[k+1];
                Vector3 midpt = (p1 + p2) * 0.5f;

                PuppetCut cut1;
                cut1.otherPlane = j;
                cut1.pt1 = p1;
                cut1.pt2 = midpt;

                PuppetCut cut2;
                cut2.otherPlane = i;
                cut2.pt1 = midpt;
                cut2.pt2 = p2;

                m_puppetcuts[i].push_back(cut1);
                m_puppetcuts[j].push_back(cut2);

            }


        }

    }

}

void PlaneMesh::DrawPaperPuppetStuff()
{

    if (selectedPlaneIndex < 0 || selectedPlaneIndex >= m_vPlanarCuts.size())
        return;

    const Plane & p = m_vPlanarCuts[selectedPlaneIndex].params;

    GLint v[4];
    glGetIntegerv(GL_VIEWPORT, v);

    glDisable(GL_LIGHTING);

    float aspect = float(v[3]) / float(v[2]);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1, 1, -aspect, aspect, 0.01f, 10.0f);

    Vector3 cent = p.norm * p.pt;
    Vector3 eye = cent + p.norm;
    Vector3 up;
    if (fabs((cent - eye).Dot(Vector3(0,1,0))) < 0.9f) {
        up = (cent - eye).Cross(Vector3(0,1,0));
    }
    else {
        up = (cent - eye).Cross(Vector3(1,0,0));
    }

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(eye.x, eye.y, eye.z, cent.x, cent.y, cent.z, up.x, up.y, up.z);

    //cent.Debug("cent");
    //eye.Debug("eye");
    //up.Debug("up");

    glDepthMask(GL_FALSE);

    //first draw silhouette for this plane
    glLineWidth(2.0f);
    glEnable(GL_LINE_STIPPLE);

    for (int j=0; j<2; ++j) {
        for (int i=0; i<m_vPlanarCuts[selectedPlaneIndex].chains.size(); ++i) {
            if (j == 0) {
                glLineStipple(1,0x00ff);
                SetColorByIndex(selectedPlaneIndex);
            }
            else if (j == 1) {
                glLineStipple(1,0xff00);
                glColor3f(.75,.75,.75);
            }
            m_vPlanarCuts[selectedPlaneIndex].chains[i].Draw();
        }
    }


    const int i = selectedPlaneIndex;
    for (int k=0; k<2; ++k) {

        for (int j=0; j<m_puppetcuts[i].size(); ++j) {

            const Vector3 & p1 = m_puppetcuts[i][j].pt1;
            const Vector3 & p2 = m_puppetcuts[i][j].pt2;

            if (k == 0) {
                glLineStipple(1,0x00ff);
                SetColorByIndex(m_puppetcuts[i][j].otherPlane);
            }
            else if (k == 1) {
                glLineStipple(1,0xff00);
                glColor3f(.75,.75,.75);
            }

            glBegin(GL_LINES);
            glVertex3f(p1.x, p1.y, p1.z);
            glVertex3f(p2.x, p2.y, p2.z);
            glEnd();

        }

    }

    glDepthMask(GL_TRUE);

    glLineWidth(1.0f);
    glDisable(GL_LINE_STIPPLE);
    glColor3f(1,1,1);

}

void PlaneMesh::FlipMeshNormals()
{

    if (inputMesh) {
        inputMesh->FlipFaceNorms();
    }

}

void PlaneMesh::SetBackgroundColour(const QColor & c)
{
    m_backColor = c;
}

QColor & PlaneMesh::GetBackgroundColour()
{
    return m_backColor;
}

void PlaneMesh::SetMeshLightColour(const QColor & c)
{
    m_meshLightColor = c;
}
